<?php

class BusinessWiki_Dialogs {
    private static $dialogs = array();
    private static $render = array();
    
    public static function register( $dialogs ) {
        global $wgAjaxExportList;
        
        foreach ( $dialogs as $dialog ) {
            $file = dirname( __FILE__ ) . '/dialogs/' . $dialog . '.php';
            
            if ( !file_exists( $file ) ) {
                throw new Exception( "Dialog not found: '{$dialog}'" );
            }
            
            require_once( $file );        

            $class = 'BusinessWiki_Dialog_' . $dialog;
            
            if ( class_exists( $class ) ) {
                if ( in_array( 'ajaxContent', get_class_methods( $class ) ) ) {
                    eval("
                    	function wfSajaxBusinessWikiDialog_{$dialog} () {
                    		\$params = func_get_args();
                    		return call_user_func_array( array( $class, 'ajaxContent' ), \$params );
                    	}                    
                    ");
                    
                    $wgAjaxExportList[] = "wfSajaxBusinessWikiDialog_{$dialog}";
                }
            }
            
            self::$dialogs[] = $dialog;
        }
    }
    
    public static function prepare( $dialog ) {
        if ( in_array( $dialog, self::$render ) ) {
            return true;
        }

        if ( in_array( $dialog, self::$dialogs ) ) {
            self::$render[] = $dialog;
            
            return true;
        } else {
            return false;
        }
    }
    
    public static function render( OutputPage &$out ) {
        foreach ( self::$render as $dialog ) {
            $dialog = 'BusinessWiki_Dialog_' . $dialog;
            
            $dialog = new $dialog();
            $out->addHTML( (string) $dialog );
            
            if (!empty($dialog->scripts) && is_array($dialog->scripts)) {
                foreach ($dialog->scripts as $script) {
                    $out->addScriptFile($script);
                }
            }
        }
        
        return true;
    }
}

interface IBusinessWiki_Dialog {
    public function __toString();
}

abstract class BusinessWiki_Dialog implements IBusinessWiki_Dialog {
    public $id;
    public $title;
    protected $buttons;
    protected $params;
    public $scripts;
    
    public function __construct( $className ) {
        global $wgMessageCache;
        
        $this->id = str_replace( 'BusinessWiki_Dialog_', '', $className );
        
        $langFile = dirname( __FILE__ ) . '/dialogs/' . $this->id . '.i18n.php';
        
        $wgMessageCache->loadMessagesFile( $langFile );
        
        $this->title = wfMsg( strtolower( $this->id ) . '-title' );
    }
    
    abstract protected function content();
    
    public function __toString() {
        return BusinessWiki_Xml::dialog(
            $this->id,
            $this->title,
            $this->content(),
            $this->buttons,
            $this->params
        );
    }
}