<?php

class BusinessWiki_Dialog_PageRestrictions extends BusinessWiki_Dialog {
    public function __construct() {
        parent::__construct( __CLASS__ );
        
        $this->buttons = array();
        
        $this->buttons[wfMsg('pagerestrictions-button-cancel')] = <<<JS
function() {
	$(this).dialog('close');
}        
JS;

        $this->buttons['Ok'] = <<<JS
function() {
	_this = $(this);
	params = {
		id: parseInt(wgArticleId)
    };
	$('input:checked').each(function() {
		if ($(this).hasClass('permission')) {
			if(typeof(params[$(this).attr('name')]) == 'undefined') {
				params[$(this).attr('name')] = [];
			}
			if (!(value = parseInt(this.value))) {
				value = this.value;
			}
    		params[$(this).attr('name')].push(value);
    	}
	});
	$.businesswiki.sajax('wfSajaxBusinessWikiDialog_PageRestrictions_save', [$.compactJSON(params)], function() {
		_this.dialog('close');
	});
}        
JS;

        $this->params = array(
            'width' => 400,
            'height' => 400,
        );
    }
    
    public function content() {  }
    
    public static function ajaxContent( $articleId ) {
        global $wgBusinessWikiActions, $wgJsMimeType, $wgContLanguageCode;
        
        require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'PageRestrictions.i18n.php';
        
        $title = BusinessWiki_Title::newFromID( $articleId );
        $r =& BusinessWikiPermissions_Restrictions::getInstanceFromTitle( $title );
        $restrictions = $r->restrictions();
        
        $out =
            Xml::openElement( 'div', array( 'style' => 'height: 300px; overflow-x: hidden; overflow-y: scroll' ) ) .
            Xml::openElement( 'table', array( 'style' => 'width: 100%' ) ) .
            Xml::openElement( 'thead' ) .
            Xml::openElement( 'tr' ) .
            Xml::element( 'th', array( 'style' => 'width: 210px; text-align: left' ), $messages[$wgContLanguageCode]['pagerestrictions-group'] );
        foreach ( $wgBusinessWikiActions as $action ) {
            $out .= Xml::element( 'th', null, $messages[$wgContLanguageCode]['pagerestrictions-' . $action] );
        }
        $out .=
            Xml::closeElement( 'tr' ) .
            Xml::closeElement( 'thead' ) .
            Xml::openElement( 'tbody' );
        
        $groups = Groups::getGroups();
        foreach ($groups as &$group) {
            $group = Group::newFromName($group);
            $attribs = array( 'class' => 'permission', 'value' => 'g-' . $group->id );
            $out .=
                Xml::openElement( 'tr' ) .
                Xml::tags( 'td', array( 'style' => 'text-align: left' ), $group->name );
            foreach ( $wgBusinessWikiActions as $action ) {
                $realAttribs = $attribs;
                if ( in_array(
                        -1, $restrictions[$action][BusinessWikiPermissions_Restrictions::user]
                ) ) {
                    $realAttribs = array_merge( $realAttribs, array( 'disabled' => 'disabled' ) );
                }
                if (!is_array($restrictions[$action][BusinessWikiPermissions_Restrictions::group])) {
                    $checked = false;
                } else {
                    $checked = in_array($group->id, $restrictions[$action][BusinessWikiPermissions_Restrictions::group]);
                }
                $out .= Xml::tags( 'td', null, Xml::check( $action, $checked, $realAttribs ) );
            }
            $out .= Xml::closeElement( 'tr' );
         }
         $out .=   
            Xml::closeElement( 'tbody' ) .
            Xml::closeElement( 'table' ) .
            
            
            Xml::openElement( 'table', array( 'style' => 'width: 100%' ) ) .
            Xml::openElement( 'thead' ) .
            Xml::openElement( 'tr' ) .
            Xml::element( 'th', array( 'style' => 'width: 210px; text-align: left' ), $messages[$wgContLanguageCode]['pagerestrictions-user'] );
        foreach ( $wgBusinessWikiActions as $action ) {
            $out .= Xml::element( 'th', null, $messages[$wgContLanguageCode]['pagerestrictions-' . $action] );
        }
        $out .=
            Xml::closeElement( 'tr' ) .
            Xml::closeElement( 'thead' ) .
            Xml::openElement( 'tbody' );
        $dbr =& wfGetDB( DB_SLAVE );
        $res = $dbr->select(
            'user',
            array( 'user_id', 'user_name', 'user_real_name' ),
            array(),
            __METHOD__
        );

        $out .=
            Xml::openElement( 'tr' ) .
            Xml::tags( 'td', array( 'style' => 'text-align: left' ), $messages[$wgContLanguageCode]['pagerestrictions-allusers'] );
        foreach ( $wgBusinessWikiActions as $action ) {
            $out .= Xml::tags( 'td', null, Xml::check( $action, in_array(
                -1, $restrictions[$action][BusinessWikiPermissions_Restrictions::user]
            ), array( 'class' => 'permission all', 'value' => -1 ) ) );
        }
        $out .= Xml::closeElement( 'tr' );
        
        foreach ( $res as $row ) {
            $user = User::newFromRow( $row );
            $attribs = array( 'class' => 'permission', 'value' => $row->user_id );
            $out .=
                Xml::openElement( 'tr' ) .
                Xml::tags( 'td', array( 'style' => 'text-align: left' ), (
                    ( $name = $user->getRealName() )
                        ? "{$name} ({$user->getName()})"
                        : $user->getName()
                ) );
            foreach ( $wgBusinessWikiActions as $action ) {
                $realAttribs = $attribs;
                if ( is_array($restrictions[$action][BusinessWikiPermissions_Restrictions::user]) && in_array(
                        -1, $restrictions[$action][BusinessWikiPermissions_Restrictions::user]
                ) ) {
                    $realAttribs = array_merge( $realAttribs, array( 'disabled' => 'disabled' ) );
                }
                if ( !$title->isFirstContributor( $user ) ) {
                    if (!is_array($restrictions[$action][BusinessWikiPermissions_Restrictions::user])) {
                        $checked = false;
                    } else {
                        $checked = in_array($row->user_id, $restrictions[$action][BusinessWikiPermissions_Restrictions::user]);
                    }
                    $out .= Xml::tags( 'td', null, Xml::check( $action, $checked, $realAttribs ) );
                } else {
                    $out .= Xml::tags( 'td', null, Xml::check(
                        $action,
                        true,
                        array_merge( $attribs, array( 'class' => 'permission first', 'disabled' => 'disabled' ) )
                    ) );
                }
            }
            $out .= Xml::closeElement( 'tr' );
         }
         $out .=   
            Xml::closeElement( 'tbody' ) .
            Xml::closeElement( 'table' ) .
            Xml::closeElement( 'div' );
        
        $out .= Xml::openElement( 'script', array( 'type' => $wgJsMimeType ) ) . "\n";
        $out .= <<<JS
$(document).ready(function() {
	$('input').each(function() {
		if ($(this).hasClass('permission') && -1 == parseInt(this.value)) {
			$(this).click(function() {
				name = $(this).attr('name');
				var column = $('input:checkbox').not('.all, .first').filter(function() {
					return $(this).attr('name') == name;
				});
				if (this.checked) {
					column.each(function() {
						this.checked = false;
						$(this).attr('disabled', 'disabled');
					});
				} else {
					column.removeAttr('disabled');
				}
			});
    	}
	});
});
JS;
        $out .= "\n" . Xml::closeElement( 'script' ) . "\n";

        return $out;        
    }
    
    public static function ajaxSave( $params ) {
        $r =& BusinessWikiPermissions_Restrictions::getInstanceFromId( $params['id'] );
        
        $r->restrictions( $params );
    }
}

function wfSajaxBusinessWikiDialog_PageRestrictions_save( $params ) {
    BusinessWiki_Dialog_PageRestrictions::ajaxSave( json_decode( $params, true ) );
    
    PageIndexView::onArticleChangeComplete();
    
    return (string) true;
}

$wgAjaxExportList[] = 'wfSajaxBusinessWikiDialog_PageRestrictions_save';