<?php

/**
 * MediaWiki Attachments extension
 */

if ( ! defined( 'MEDIAWIKI' ) )
	die( 1 );
	
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'Attachments',
	'author' => 'Paweł Szreder',
	'version'        => '0.1',
	'description' => 'Adds attachments box',
	'descriptionmsg' => 'attachments_desc',
	'url' => 'http://www.bizneswiki.pl'
);

global $wgExtensionFunctions, $wgHooks, $wgExtraNamespaces;

require_once( 'Attachments.i18n.php' );
$wgAutoloadClasses['AttachmentsTemplate'] = dirname( __FILE__ ) . '/AttachmentsTemplate.php';

$wgExtensionFunctions[] = 'wfAttachments';
$wgHooks['BeforePageDisplay'][] = 'Attachments::onBeforePageDisplay';
$wgHooks['LanguageGetMagic'][] = 'wfAttachmentsMagic';
$wgHooks['UploadComplete'][] = 'Attachments::onUploadComplete';
$wgHooks['FileDelete'][] = 'Attachments::onFileDelete';

$wgHooks['MakeGlobalVariablesScript'][] = 'Attachments::onMakeGlobalVariablesScript';
$wgHooks['EditPage::showEditForm:fields'][] = 'Attachments::onEditPage__showEditForm_fields';
$wgHooks['ArticleSaveComplete'][] = 'Attachments::onArticleSaveComplete';

function wfAttachments() {
	global $wgParser, $wgMessageCache, $wgAttachmentsMessages;
	
	$wgParser->setHook( 'Attachments', 'Attachments' );
	
	foreach ( $wgAttachmentsMessages as $lang => $messages) {
		$wgMessageCache->addmessages( $messages, $lang );
	}
}

function wfAttachmentsMagic( &$magicWords, $langCode ) {
	$magicWords['attachments'] = array( 0, 'attachments' );

	return true;
}

class Attachments {
	static $token;
	
	private $title;
	private $linker;
	private $titleArray;
	private $count;
	
	static $knownMediaTypes = array( 'ARCHIVE', 'BITMAP', 'OFFICE', 'TEXT' );
	
	public function __construct( Title $title ) {
		$this->title =& $title;
		
		$dbr =& wfGetDB( DB_SLAVE );
		$res = $dbr->select(
			'file',
			array ( "{$dbr->addQuotes( NS_IMAGE )} AS page_namespace", 'file_name AS page_title' ),
			array( 'file_from' => $this->title->getArticleID() ),
			__METHOD__
		);
		$this->titleArray = TitleArray::newFromResult( $res );
		$this->count = iterator_count( $this->titleArray );
	}

	public static function onBeforePageDisplay( &$out ) {
		global $wgStylePath, $wgDefaultSkin;
		
		$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/Attachments.css" );
		$out->addScriptFile( "/extensions/Attachments/Attachments.js" );
			
		return true;
	}

	public static function onUploadComplete( &$image ) {
		$dbw =& wfGetDB( DB_MASTER );
		$dbw->begin();
		if ( !empty( $image->mDestPageToken ) && $image->mDestPageToken != 'undefined' ) {
		    $dbw->delete( 'file', array( 'file_name' => $image->mDestName ) );
			$dbw->insert( 'file', array( 'file_name' => $image->mDestName, 'file_from_token' => $image->mDestPageToken ) );
		} elseif ( !is_null( $image->mDestPageId ) ) {
		    $dbw->delete( 'file', array( 'file_name' => $image->mDestName ) );
			$dbw->insert( 'file', array( 'file_name' => $image->mDestName, 'file_from' => $image->mDestPageId ) );
		}
		$dbw->commit();
		
		return true;
	}
	
	public static function onFileDelete( $file, $oldimage, $article, $user, $reason ) {
		$dbw =& wfGetDB( DB_MASTER );
		$dbw->begin();
		$dbw->delete( 'file', array( 'file_name' => $file->name ) );
		$dbw->commit();
		
		return true;
	}
	
	public static function onMakeGlobalVariablesScript( &$vars ) {
		global $wgRequest, $wgUser;
		
		if ( $wgRequest->getVal( 'action' ) != 'edit' ) return true;
		
		if ( empty( self::$token ) ) {
			self::$token = htmlspecialchars( md5( $wgUser->editToken() . time() ) );
		}		
		
		$vars['wgSubmissionToken'] = self::$token;
		
		return true;
	}
	
	public static function onEditPage__showEditForm_fields( EditPage &$editpage, OutputPage &$out ) {
		global $wgUser;
		
		if ( empty( self::$token ) ) {
			self::$token = htmlspecialchars( md5( $wgUser->editToken() . time() ) );
		}
		
		$out->addHTML( Xml::hidden( 'wpSubmissionToken', self::$token ) );
		
		return true;
	}
	
	public static function onArticleSaveComplete( Article &$article ) {
		global $wgRequest;
		
		if (in_array($article->getTitle()->getNamespace(), array(NS_MAIN, NS_CATEGORY))) {
    		$token = $wgRequest->getVal( 'wpSubmissionToken' );
    		if ( !empty( $token ) ) {
    			$dbw =& wfGetDB( DB_MASTER );
    			$dbw->begin();
    			$dbw->update( 'file', array( 'file_from' => $article->getID(), 'file_from_token' => null ), array( 'file_from_token' => $token ) );
    			$dbw->commit();
    		}
		}
		
		return true;
	}
	
	public static function byteConvert( $bytes ) {
		$s = array( 'B', 'KB', 'MB', 'GB', 'TB', 'PB' );
		$e = floor( log( $bytes ) / log( 1024 ) );
		
		return sprintf( '%.2f&nbsp;' . $s[$e], ( $bytes / pow( 1024, floor( $e ) ) ) );
	}
	
	public static function getIcon( File $file, $path = null, $size = 24 ) {
		global $wgStylePath, $wgDefaultSkin;
		
		if ( is_null( $path ) ) $path = "{$wgStylePath}/{$wgDefaultSkin}/Attachments/";
		
		if ( in_array( $file->media_type, self::$knownMediaTypes ) ) {
			if ( $file->media_type == MEDIATYPE_OFFICE ) {
				$ext = $file->getExtension();
				if ( in_array( $ext, array( 'pdf' ) ) ) {
					$icon = "pdf_{$size}.gif";
				} elseif ( in_array( $ext, array( 'doc', 'docx', 'odt', 'rtf', 'psw', 'ott', 'docm', 'wps' ) ) ) {
					$icon = "word_{$size}.gif";
				} elseif ( in_array( $ext, array( 'ots', 'ods', 'xls', 'xlt', 'csv', 'pxl', 'xlm', 'xlsm', 'xlsx', 'xltx' ) ) ) {
					$icon = "excel_{$size}.gif";
				} elseif ( in_array( $ext, array( 'odp', 'otp', 'ppt', 'pps', 'pptx', 'ppsx' ) ) ) {
					$icon = "powerpoint_{$size}.gif";
				} else {
					$icon = strtolower( $file->media_type ) . "_{$size}.gif";
				}
			} else {
				$icon = strtolower( $file->media_type ) . "_{$size}.gif";
			}
		} else {
			$icon = "unknown_{$size}.gif";
		}
		
		return $path . $icon;
	}
		
	public static function getTableFCKeditor( $pageId, $token ) {
		$dbr =& wfGetDB( DB_SLAVE );
		$res = $dbr->select(
			'file',
			array ( "{$dbr->addQuotes( NS_IMAGE )} AS page_namespace", 'file_name AS page_title' ),
			array( "(file_from = {$pageId} AND file_from > 0) OR file_from_token = {$dbr->addQuotes( $token )}" ),
			__METHOD__
		);
		$titleArray = TitleArray::newFromResult( $res );
		if ( iterator_count( $titleArray ) > 0 ) {
			$out = Xml::openElement( 'table' );
			foreach ( $titleArray as $title ) {
				$file = wfFindFile( $title );
				$icon = self::getIcon( $file );
				$out .= Xml::openElement( 'tr' );
				$out .= Xml::tags( 'td', array( 'class' => 'link', 'style' => "background-image: url({$icon});"), Xml::element( 'a', array(
					'href' => $title->getLocalURL(),
					'target' => '_blank'
				), $title->getBaseText() ) . ' (' . self::byteConvert( $file->getSize() ) . ')' );
				//$out .= Xml::tags( 'td', array( 'class' => 'size' ),  );
				$out .= Xml::tags( 'td', array( 'class' => 'addlink' ), Xml::input( '', '', "Insert link", array(
					'type' => 'button',
					'onclick' => 'InsertLink(this);',
					'style' => 'display: none;'
				) ) );				
				$out .= Xml::tags( 'td', array( 'class' => 'delete' ), Xml::input( '', '', wfMsg( 'delete' ), array(
					'type' => 'button',
					'onclick' => 'DeleteAttachment(this);'
				) ) );				
				$out .= Xml::closeElement( 'tr' );
			}
			$out .= Xml::closeElement( 'table' );
		} else {
			$out = Xml::span( wfMsg( 'no_attachments' ), null );
		}
		
		return $out;	
	}
	
	public function __toString() {
		if ( $this->count > 0 ) {
			global $wgUser;
			
			$skin = $wgUser->getSkin();
			
			$template = new AttachmentsTemplate();
			$template->set( 'count', $this->count );
			$files = array();
			foreach ( $this->titleArray as $title ) {
				if ( false !== $file = wfFindFile( $title ) ) {
					$files[] = array(
						'icon' => self::getIcon( $file ),
						'link' => $skin->makeMediaLinkObj( $title, $title->getBaseText() ),
						'size' => self::byteConvert( $file->getSize() ),
						'description' => $skin->link( $title, wfMsg('attachments-showdescription'), array( 'style' => 'float: right;' ) )
					);					
				}
			}
			$template->set( 'files', $files );
						
			return $template->fetch();
		}
				
		return '';
	}
}
