<?php

/**
 * MediaWiki BiznesWikiViews extension
 */

if ( ! defined( 'MEDIAWIKI' ) )
	die( 1 );

$dir = dirname(__FILE__);

require_once( "{$dir}/ExtendedQuickTemplate.php" );

require_once( "{$dir}/Hooks.php" );

require_once( "{$dir}/SpecialAdministration.php" );

require_once( "{$dir}/api/ApiUpload.php" );
$wgAPIModules['upload'] = 'ApiUpload';

$wgAutoloadClasses['PreferencesForm2'] = "{$dir}/SpecialPreferences2.php";

$wgAutoloadClasses['PageHistory2'] = "{$dir}/PageHistory2.php";

$wgAutoloadClasses['SpecialSearch2'] = "{$dir}/SpecialSearch2.php";

$wgAutoloadClasses['SpecialRecentchanges2'] = "{$dir}/SpecialRecentchanges2.php";
$wgAutoloadClasses['ChangesList2'] = "{$dir}/SpecialRecentchanges2.php";
$wgSpecialPages['Recentchanges2'] = 'SpecialRecentChanges2';
$wgHooks['LanguageGetSpecialPageAliases'][] = 'wfSpecialRecentChanges2LocalizedPageName';
function wfSpecialRecentChanges2LocalizedPageName( &$specialPageArray, $code ) {
	$specialPageArray[ 'SpecialRecentChanges2' ][] = 'Recent changes';

	return true;
}

$wgAutoloadClasses['SpecialPageIndex'] = "{$dir}/SpecialPageIndex.php";

$wgHooks['SpecialPageBeforeGetPage'][] = 'wfSpecialPageBeforeGetPage';

function wfSpecialPageBeforeGetPage( &$className ) {
	switch ( $className ) {
		case 'SpecialRecentchanges':
			$className = 'SpecialRecentchanges2'; break;
	}
	
	return true;
}

/**
 * Views hooks
 */
$wgHooks['ArticlePageView'][] = 'wfBiznesWikiViews';
$wgHooks['CategoryPageView'][] = 'wfBiznesWikiViewsAbort';
$wgHooks['SkinTemplateOutputPageBeforeExec'][] = 'wfBiznesWikiViewsCustomTitle';

function wfBiznesWikiViewsCustomTitle( SkinTemplate &$template, &$templateEngine ) {
	global $wgOut, $wgArticle, $wgRequest;
	
	if ( $wgArticle instanceof Article ) {
		$view = new BiznesWikiViewsController( $wgOut, $wgArticle, $template->mTitle, $template->mUser, $wgRequest );
		
		$pageTitle = $view->getPageTitle();
		$templateEngine->set( 'title', $pageTitle );
		$templateEngine->set( 'displaytitle', $pageTitle );
	}
	
	return true;
}

function wfBiznesWikiViews( OutputPage &$output, Article &$article, Title &$title, User &$user, WebRequest &$request ) {
	$view = new BiznesWikiViewsController( $output, $article, $title, $user, $request );

	return $view->display();
}

function wfBiznesWikiViewsAbort( Article &$article ) {
	return false;
}

/**
 * Strategy
 */

interface DisplayStrategy {
	public static function getPageTitle( Article &$article );
	
	public function render( Article &$article );
}

class BiznesWikiViewsStrategy implements DisplayStrategy {
	public static function getPageTitle( Article &$article ) {
		global $wgOut;
		
		return $wgOut->getPageTitle();
	}
	
	public function render( Article &$article ) {
		global $wgOut;
		
		$title = $article->getTitle(); 
		$ns = $title->getNamespace();
		
		if ( $ns % 2 == 1 ) {
			$wgOut->redirect( Title::newFromText( $title->getBaseText(), ( $ns - 1 ) )->getLocalURL() );
		}/* elseif ( ( !$article->exists() || ( $ns == NS_USER ) ) && Title::newMainPage()->exists() ) {
			$wgOut->redirect( Title::newMainPage()->getLocalURL() );
		}*/
		
		return true;
	}
}

$wgAutoloadClasses['ArticlePageView'] = "{$dir}/ArticlePageView.php";

class ArticlePageViewStrategy implements DisplayStrategy {
	public static function getPageTitle( Article &$article ) {
		return ArticlePageView::newFromArticle( $article )->getPageTitle();
	}
	
	public function render( Article &$article ) {
		return ArticlePageView::newFromArticle( $article )->render();	
	}
}

$wgAutoloadClasses['CategoryPageView'] = "{$dir}/CategoryPageView.php";
$wgExtensionMessagesFiles['CategoryPageView'] = "{$dir}/CategoryPageView.i18n.php";

class CategoryPageViewStrategy implements DisplayStrategy {
	public static function getPageTitle( Article &$article ) {
		return CategoryPageView::newFromArticle( $article )->getPageTitle();
	}
	
	public function render( Article &$article ) {
		return CategoryPageView::newFromArticle( $article )->render();	
	}
}

$wgAutoloadClasses['ImagePageView'] = "{$dir}/ImagePageView.php";
//$wgExtensionMessagesFiles['ImagePageView'] = "{$dir}/ImagePageView.i18n.php";

class ImagePageViewStrategy implements DisplayStrategy {
	public static function getPageTitle( Article &$article ) {
		return ImagePageView::newFromArticle( $article )->getPageTitle();
	}
	
	public function render( Article &$article ) {
		return ImagePageView::newFromArticle( $article )->render();	
	}
}
if ( defined( 'LIVEBLEND' ) ) {
$wgAutoloadClasses['QuestionPageView'] = "{$dir}/QuestionPageView.php";

class QuestionPageViewStrategy implements DisplayStrategy {
	public static function getPageTitle( Article &$article ) {
		return QuestionPageView::newFromArticle( $article )->getPageTitle();
	}

	public function render( Article &$article ) {
		return QuestionPageView::newFromArticle( $article )->render();	
	}
}

$wgAutoloadClasses['ConceptPageView'] = "{$dir}/ConceptPageView.php";

class ConceptPageViewStrategy implements DisplayStrategy {
	public static function getPageTitle( Article &$article ) {
		return ConceptPageView::newFromArticle( $article )->getPageTitle();
	}
	
	public function render( Article &$article ) {
		return ConceptPageView::newFromArticle( $article )->render();	
	}
}
}

class BiznesWikiViewsController {
	private $displayStrategy;
	private $output, $article, $title, $user, $request;
	
	public function __construct( OutputPage &$output, Article &$article, Title &$title, User &$user, WebRequest &$request ) {
		if ( !$article->exists() ) {
			$displayStrategy = new BiznesWikiViewsStrategy();
		} else {
			switch ( $article->getTitle()->getNamespace() ) {
				case NS_MAIN:
					$displayStrategy = new ArticlePageViewStrategy(); break;
				case NS_CATEGORY:
					$displayStrategy = new CategoryPageViewStrategy(); break;
				case NS_IMAGE:
					$displayStrategy = new ImagePageViewStrategy(); break;				
				default:
					if ( defined( 'LIVEBLEND' ) ) {
						switch ( $article->getTitle()->getNamespace() ) {
							case NS_QUESTION:
								$displayStrategy = new QuestionPageViewStrategy(); break;
							case NS_CONCEPT:
								$displayStrategy = new ConceptPageViewStrategy(); break;
							default:
								$displayStrategy = new BiznesWikiViewsStrategy();
						}
					}
					else
					$displayStrategy = new BiznesWikiViewsStrategy();
			}
		}
		
		$this->article =& $article;
		$this->displayStrategy = $displayStrategy;
	}
	
	public function getPageTitle() {
		return $this->displayStrategy->getPageTitle( $this->article );
	}
	
	public function display() {
		return $this->displayStrategy->render( $this->article );
	}
}
