<?php

$dir = dirname(__FILE__);
$class = basename( __FILE__, '.php' );

$wgAutoloadClasses["{$class}_Body"] = "{$dir}/{$class}/{$class}_Body.php";

class ImagePageView extends ImagePageView_Body {
	public static function newFromArticle( Article &$article ) {
		return new self( $article );
	}
	
	public function getPageTitle() {
		return $this->title->getBaseText();
	}
	
	public function render() {
		global $wgOut, $wgRequest, $wgUser, $wgImageLimits, $wgContLang;
		
		$wgOut->setPagetitle( $this->title->getPrefixedText() );
		
		/*
		 * desc
		 */
		$icon = Attachments::getIcon( $this->article->img, null, 32 );
		$wgOut->addHTML(
			Xml::openElement( 'div', array( 'id' => 'filedesc', 'style' => "background-image: url({$icon})" ) ) . "\n" .
			Xml::tags( 'strong', null,
				Xml::element( 'a', array(
					'href' => $this->article->displayImg->getUrl()
				), $this->title->getBaseText() )
			) . Xml::element( 'br' ) . "\n" .
			$this->article->displayImg->getLongDesc() . "\n" .
			Xml::closeElement( 'div' ) . "\n"
		);
		
		if( UploadForm::userCanReUpload( $wgUser, $this->article->img->name ) ) {
			$ulink = Xml::element( 'button', array(
				'onclick' => "document.location = '{$this->article->getUploadUrl()}';"
			), wfMsg( 'uploadnewversion-linktext' ) ) . "\n";
		} else { $ulink = ''; }
		
		if ( $wgUser->isAllowed('delete') ) {
			$dlink = Xml::element( 'button', array(
				'onclick' => "document.location = '{$this->article->getTitle()->getLocalUrl()}&action=delete';"
			), ucfirst( wfMsg( 'filehist-deleteall' ) ) ) . "\n";			
		} else { $dlink = ''; }
		$wgOut->addHTML(
			Xml::openElement( 'div', array( 'style' => 'text-align: center;' ) ) . "\n" .
			Xml::element( 'button', array( 'onclick' => "document.location = '{$this->article->displayImg->getUrl()}';" ), 'Download' ) . "\n" .
			$ulink .
			$dlink .
			Xml::closeElement( 'div' ) . "\n"
		);
		
		/*
		 * preview
		 */
		$sizeSel = intval( $wgUser->getOption( 'imagesize') );
		if( !isset( $wgImageLimits[$sizeSel] ) ) {
			$sizeSel = User::getDefaultOption( 'imagesize' );

			if( !isset( $wgImageLimits[$sizeSel] ) ) {
				$sizeSel = 0;
			}
		}
		list( $maxWidth, $maxHeight ) = $wgImageLimits[$sizeSel];
		$sk = $wgUser->getSkin();

		if ( $this->article->displayImg->exists() ) {
			# image
			$width_orig = $this->article->displayImg->getWidth();
			$width = $width_orig;
			$height_orig = $this->article->displayImg->getHeight();
			$height = $height_orig;
			$mime = $this->article->displayImg->getMimeType();
			$showLink = false;

			// wfRunHooks( 'ImageOpenShowImageInlineBefore', array( &$this , &$wgOut ) )	;

			if ( $this->article->displayImg->allowInlineDisplay() ) {
				# image

				# "Download high res version" link below the image
				#$msgsize = wfMsgHtml('file-info-size', $width_orig, $height_orig, $sk->formatSize( $this->displayImg->getSize() ), $mime );
				# We'll show a thumbnail of this image
				if ( $width > $maxWidth || $height > $maxHeight ) {
					# Calculate the thumbnail size.
					# First case, the limiting factor is the width, not the height.
					if ( $width / $height >= $maxWidth / $maxHeight ) {
						$height = round( $height * $maxWidth / $width);
						$width = $maxWidth;
						# Note that $height <= $maxHeight now.
					} else {
						$newwidth = floor( $width * $maxHeight / $height);
						$height = round( $height * $newwidth / $width );
						$width = $newwidth;
						# Note that $height <= $maxHeight now, but might not be identical
						# because of rounding.
					}
					$msgbig  = wfMsgHtml( 'show-big-image' );
					$msgsmall = wfMsgExt( 'show-big-image-thumb', 'parseinline',
						$wgContLang->formatNum( $width ),
						$wgContLang->formatNum( $height )
					);
				} else {
					# Image is small enough to show full size on image page
					$msgbig = htmlspecialchars( $this->article->displayImg->getName() );
					$msgsmall = wfMsgExt( 'file-nohires', array( 'parseinline' ) );
				}

				$thumbnail = $this->article->displayImg->transform( array( 'width' => $width, 'height' => $height ) );

				$anchorclose = "<br />";
				if( $this->article->displayImg->mustRender() ) {
					$showLink = true;
				} else {
					$anchorclose .=	$msgsmall;
				}

				if ( $thumbnail ) {
					$options = array(
						'alt' => $this->article->displayImg->getTitle()->getPrefixedText(),
						'file-link' => true,
					);
					$wgOut->addHTML(
						Xml::openElement( 'div', array( 'id' => 'file', 'class' => 'fullImageLink' ) ) . "\n" .
						Xml::element( 'strong', null, wfMsg( 'preview' ) ) . Xml::element( 'br' ) . "\n" .
						$thumbnail->toHtml( $options ) .
						$anchorclose .
						Xml::closeElement( 'div' ) . "\n"
					);
				}
			} else {
				#if direct link is allowed but it's not a renderable image, show an icon.
				if ( $this->article->displayImg->isSafeFile() ) {
					$icon= $this->article->displayImg->iconThumb();

					$wgOut->addHTML(
						Xml::openElement( 'div', array( 'id' => 'file', 'class' => 'fullImageLink' ) ) . "\n" .
						$icon->toHtml( array( 'desc-link' => true ) ) . "\n" .
						Xml::closeElement( 'div' ) . "\n"
					);
				}

				$showLink = true;
			}
		}
		
		/*
		 * summmary
		 */
		$summary = $this->article->img->getDescription();
		if ( !empty( $summary ) ) {
			$wgOut->addHTML(
				Xml::element( 'h2', null, "Summary" ) .
				Xml::openElement( 'p', array( 'style' => 'margin-bottom: 20px;') ) . 
				$summary .
				Xml::closeElement( 'p' ) . "\n"
			);
		}		
		
		/*
		 * filehist
		 */
		$wgOut->addHTML(
			Xml::openElement( 'p' ) . 
			Xml::element( 'h2', array( 'style' => 'float: left; padding-right: 20px;' ), wfMsg( 'filehist' ) ) .
			wfMsg( 'filehist-help' ) .
			Xml::closeElement( 'p' ) . "\n" .
			Xml::openElement( 'div', array( 'class' => 'visualClear' ) ) . Xml::closeElement( 'div' ) . "\n"
		);
		if ( $this->article->img->exists() ) {
			$list = new ImageHistoryList_BiznesWiki( $this->article );
			$file = $this->article->img;
			$html =
				$list->beginImageHistoryList() .
				$list->imageHistoryLine( true, $file );
			// old image versions
			$hist = $this->article->img->getHistory();
			foreach( $hist as $file ) {
				$html .= $list->imageHistoryLine( false, $file );
			}
			$html .= $list->endImageHistoryList();
			$wgOut->addHTML( $html );
		}
		
		$this->article->img->resetHistory(); // free db resources
		
		/*
		 * links
		 */
		$wgOut->addHTML( Xml::element( 'h2',
			array( 'id' => 'filelinks' ),
			wfMsg( 'imagelinks' ) ) . "\n" );
		$this->article->imageDupes();
		if ( $this->article->img->isLocal() ) {
			$this->article->imageRedirects();
		}
		$this->imageLinks();
		
		/*
		 * comments
		 */
		$comments = new Comments( $this->article );
		$wgOut->addHTML( $comments );
		
		return false;
	}
	
	function imageLinks() {
		global $wgUser, $wgOut, $wgLang;

		$dbr = wfGetDB( DB_SLAVE );

		$res = $dbr->select(
			array( 'imagelinks', 'page' ),
			array( 'page_id', 'page_namespace', 'page_title' ),
			array( 'il_to' => $this->article->mTitle->getDBkey(), 'il_from = page_id' ),
			__METHOD__,
			array( 'LIMIT' => $limit + 1)	
		);
		
		$res2 = $dbr->select(
			array( 'file' ),
			array( 'file_from' ),
			array( 'file_name' => $this->article->mTitle->getDBkey() ),
			__METHOD__
		);
		
		if ( ( $dbr->numRows( $res ) + $dbr->numRows( $res2 ) ) == 0 ) {
			$wgOut->addHTML( "<div id='mw-imagepage-nolinkstoimage'>\n" );
			$wgOut->addWikiMsg( 'nolinkstoimage' );
			$wgOut->addHTML( "</div>\n" );
			return;
		}
		
		$links = array();

		while ( $s = $res2->fetchObject() ) {
			$title = Title::newFromID( $s->file_from );
			if ( !( $title instanceof Title ) ) {
				continue;
			} elseif ( array_key_exists( $s->file_from, $links ) ) {
				continue;
			}

			$links[$s->file_from] = $title;
		}
		$res2->free();

		while ( $s = $res->fetchObject() ) {
			$title = Title::makeTitle( $s->page_namespace, $s->page_title );
			if ( array_key_exists( $s->file_from, $links ) ) {
				continue;
			}
			
			$links[$s->page_id] = $title;
		}
		$res->free();

		$skin = $wgUser->getSkin();
		
		$html = "<ul class='mw-imagepage-linkstoimage'>\n";
		foreach ( $links as $title ) {
			$html .= '<li>' . $skin->link( $title ) . '</li>' . "\n";
		}
		$html .= "</ul></div>\n";

		$wgOut->addHTML( "<div id='mw-imagepage-section-linkstoimage'>\n" );
		$wgOut->addWikiMsg( 'linkstoimage', count( $links ) );
		$wgOut->addHTML( $html );
		
		// Add a links to [[Special:Whatlinkshere]]
		if ( $count > $limit )
			$wgOut->addWikiMsg( 'morelinkstoimage', $this->article->mTitle->getPrefixedDBkey() );
	}
}

class ImageHistoryList_BiznesWiki extends ImageHistoryList {
	private $bg = true;
	
	public function __construct( $imagePage ) {
		parent::__construct( $imagePage );
	}
	
	public function beginImageHistoryList() {
		global $wgOut, $wgUser;		
		return Xml::openElement( 'table', array( 'cellspacing' => 0, 'class' => 'filehistory' ) ) . "\n"
			. '<tr>'
			. '<th>' . wfMsgHtml( 'filehist-thumb' ) . '</th>'
			. '<th>' . wfMsgHtml( 'filehist-dimensions' ) . '</th>'
			. '<th>' . wfMsgHtml( 'filehist-datetime' ) . '</th>'
			. '<th>' . wfMsgHtml( 'filehist-user' ) . '</th>'
			. ( $this->current->isLocal() && ($wgUser->isAllowed('delete') || $wgUser->isAllowed('deleterevision') )
				? '<th colspan="2">Actions</th>'
				: '<th>Actions</th>'
			)
			. "</tr>\n";
	}
	
	public function imageHistoryLine( $iscur, $file ) {
		global $wgUser, $wgLang, $wgContLang, $wgTitle;

		$timestamp = wfTimestamp(TS_MW, $file->getTimestamp());
		$img = $iscur ? $file->getName() : $file->getArchiveName();
		$user = $file->getUser('id');
		$usertext = $file->getUser('text');
		$size = $file->getSize();
		$description = $file->getDescription();
		$sha1 = $file->getSha1();

		$local = $this->current->isLocal();
		$row = $css = $selected = '';

		// Thumbnail
		$row .= '<td>';
		if( $file->allowInlineDisplay() && $file->userCan( File::DELETED_FILE ) && !$file->isDeleted( File::DELETED_FILE ) ) {
			$params = array(
				'width' => '32',
				'height' => '32',
			);
			$thumbnail = $file->transform( $params );
			$options = array(
				'alt' => wfMsg( 'filehist-thumbtext', $wgLang->timeAndDate( $timestamp, true ) ),
				'file-link' => true,
			);
			$row .= $thumbnail->toHtml( $options );
		} else {
			$row .= wfMsgHtml( 'filehist-nothumb' );
		}
		$row .= "</td><td>";
		
		// File size
		$row .= " <span style='white-space: nowrap;'>" . $this->skin->formatSize( $size ) . '</span>';
		$row .= "</td>";
		
		// Date/time and image link
		if( $file->getTimestamp() === $this->img->getTimestamp() ) {
			$selected = "class='filehistory-selected'";
		}
		$row .= "<td $selected style='white-space: nowrap;'>";
		if( !$file->userCan(File::DELETED_FILE) ) {
			# Don't link to unviewable files
			$row .= '<span class="history-deleted">' . $wgLang->timeAndDate( $timestamp, true ) . '</span>';
		} else if( $file->isDeleted(File::DELETED_FILE) ) {
			$revdel = SpecialPage::getTitleFor( 'Revisiondelete' );
			# Make a link to review the image
			$url = $this->skin->makeKnownLinkObj( $revdel, $wgLang->timeAndDate( $timestamp, true ),
				"target=".$wgTitle->getPrefixedText()."&file=$sha1.".$this->current->getExtension() );
			$row .= '<span class="history-deleted">'.$url.'</span>';
		} else {
			$url = $iscur ? $this->current->getUrl() : $this->current->getArchiveUrl( $img );
			$row .= Xml::element( 'a', array( 'href' => $url ), $wgLang->timeAndDate( $timestamp, true ) );
		}
		$row .= '</td>';

		// Uploading user
		$row .= '<td>';
		if( $local ) {
			// Hide deleted usernames
			if( $file->isDeleted(File::DELETED_USER) ) {
				$row .= '<span class="history-deleted">' . wfMsgHtml( 'rev-deleted-user' ) . '</span>';
			} else {
				$row .= $this->skin->userLink( $user, $usertext );
			}
		} else {
			$row .= htmlspecialchars( $usertext );
		}
		$row .= '</td>';
		
		// Reversion link/current indicator
		$row .= '<td>';
		if( $iscur ) {
			$row .= wfMsgHtml( 'filehist-current' );
		} elseif( $local && $wgUser->isLoggedIn() && $this->title->userCan( 'edit' ) ) {
			if( $file->isDeleted(File::DELETED_FILE) ) {
				$row .= wfMsgHtml('filehist-revert');
			} else {
				$q = array(
					'action' => 'revert',
					'oldimage' => urlencode( $img ),
					'wpEditToken' => urlencode( $wgUser->editToken( $img ) )
				);
				$row .= $this->skin->link( $this->title,
					wfMsgHtml( 'filehist-revert' ), array( 'class' => 'revert' ),
					$q );
			}
		}
		$row .= '</td>';
		
		// Deletion link
		if( $local && ($wgUser->isAllowed('delete') || $wgUser->isAllowed('deleterevision') ) ) {
			$row .= '<td>';
			# Link to remove from history
			if( $wgUser->isAllowed( 'delete' ) ) {
				$q = array();
				$q['action'] = 'delete';
				if( !$iscur )
					$q['oldimage'] = urlencode( $img );
				$row .= $this->skin->link(
					$this->title,
					wfMsgHtml( $iscur ? 'filehist-deleteall' : 'filehist-deleteone' ), array( 'class' => 'remove' ),
					$q
				);
			}
			# Link to hide content
			if( $wgUser->isAllowed( 'deleterevision' ) ) {
				if( $wgUser->isAllowed('delete') ) {
					$row .= '<br/>';
				}
				$revdel = SpecialPage::getTitleFor( 'Revisiondelete' );
				// If file is top revision or locked from this user, don't link
				if( $iscur || !$file->userCan(File::DELETED_RESTRICTED) ) {
					$del = wfMsgHtml( 'rev-delundel' );
				} else {
					// If the file was hidden, link to sha-1
					list($ts,$name) = explode('!',$img,2);
					$del = $this->skin->makeKnownLinkObj( $revdel, 	wfMsg( 'rev-delundel' ),
						'target=' . urlencode( $wgTitle->getPrefixedText() ) .
						'&oldimage=' . urlencode( $ts ) );
					// Bolden oversighted content
					if( $file->isDeleted(File::DELETED_RESTRICTED) )
						$del = "<strong>$del</strong>";
				}
				$row .= "<tt style='white-space: nowrap;'><small>$del</small></tt>";
			}
			$row .= '</td>';
		}
		
		$result = Xml::tags( 'tr', array( 'class' => ( $this->bg ) ? 'bg' : null ), $row ) . "\n";
		
		$this->bg = ( $this->bg ) ? false : true;

		return $result;
	}
}