<?php

class PageHistory2 extends PageHistory {
	private $gray = true;
	
	/**
	 * Print the history page for an article.
	 *
	 * @returns nothing
	 */
	function history() {
		global $wgOut, $wgRequest, $wgTitle, $wgScript;

		/*
		 * Allow client caching.
		 */
		if( $wgOut->checkLastModified( $this->mArticle->getTouched() ) )
		/* Client cache fresh and headers sent, nothing more to do. */
		return;

		/*
		 * Setup page variables.
		 */
		$wgOut->setPageTitle( wfMsg( 'history-title', $this->mTitle->getPrefixedText() ) );
		$wgOut->setPageTitleActionText( wfMsg( 'history_short' ) );
		$wgOut->setArticleFlag( false );
		$wgOut->setArticleRelated( true );
		$wgOut->setRobotPolicy( 'noindex,nofollow' );
		$wgOut->setSyndicated( true );
		$wgOut->setFeedAppendQuery( 'action=history' );
		$wgOut->addScriptFile( 'history.js' );

		$feedType = $wgRequest->getVal( 'feed' );
		if( $feedType ) {
			return $this->feed( $feedType );
		}

		/*
		 * Fail if article doesn't exist.
		 */
		if( !$this->mTitle->exists() ) {
			$wgOut->addWikiMsg( 'nohistory' );
			return;
		}

		/*
		 * "go=first" means to jump to the last (earliest) history page.
		 * This is deprecated, it no longer appears in the user interface
		 */
		if ( $wgRequest->getText("go") == 'first' ) {
			$limit = $wgRequest->getInt( 'limit', 50 );
			global $wgFeedLimit;
			if( $limit > $wgFeedLimit ) {
				$limit = $wgFeedLimit;
			}
			$wgOut->redirect( $wgTitle->getLocalURL( "action=history&limit={$limit}&dir=prev" ) );
			return;
		}

		/**
		 * Do the list
		 */
		$pager = new PageHistoryPager2( $this );
		$this->linesonpage = $pager->getNumRows();
		$wgOut->addHTML(
			/* $pager->getNavigationBar() . */
			$this->beginHistoryList() .
			$pager->getBody() .
			$this->endHistoryList() /* . */
			/* $pager->getNavigationBar() */
		);
	}

	function beginHistoryList() {
		global $wgTitle, $wgScript, $wgEnableHtmlDiff, $wgContLang;
		
		$this->lastdate = '';
		
		$html =
			Xml::element( 'p', array( 'style' => 'padding: 10px 0;' ), wfMsg( 'histlegend' ) ) . "\n" .
			Xml::openElement( 'form', array( 'action' => $wgScript ) ) . "\n" .
			Xml::hidden( 'title', $wgTitle->getPrefixedDbKey() );
			
		if ( $wgEnableHtmlDiff ) {
			$html .= $this->submitButton( wfMsg( 'visualcomparison'),
				array(
						'name' => 'htmldiff',
						'class'     => 'historysubmit',
						'accesskey' => wfMsg( 'accesskey-visualcomparison' ),
						'title'     => wfMsg( 'tooltip-compareselectedversions' ),
				)
			);
			$html .= $this->submitButton( wfMsg( 'wikicodecomparison'),
				array(
						'class'     => 'historysubmit',
						'accesskey' => wfMsg( 'accesskey-compareselectedversions' ),
						'title'     => wfMsg( 'tooltip-compareselectedversions' ),
				)
			);
		} else {
			$html .= $this->submitButton( wfMsg( 'compareselectedversions'),
				array(
						'class'     => 'historysubmit',
						'accesskey' => wfMsg( 'accesskey-compareselectedversions' ),
						'title'     => wfMsg( 'tooltip-compareselectedversions' ),
				)
			);
		}
		
		$html .=
			Xml::openElement( 'div', array( 'class' => 'visualClear' ) ) . Xml::closeElement( 'div' ) . "\n" .
			Xml::openElement( 'table', array( 'id' => 'pagehistory', 'style' => 'width: 100%;' ) ) . "\n" .
			Xml::openElement( 'thead', array( 'style' => 'text-align: center;' ) ) . "\n" .
			Xml::openElement( 'tr' ) . "\n" .
			Xml::element( 'td', null, '', false ) .
			Xml::element( 'td', null, ( $wgContLang->getCode() == 'pl') ? 'Pokaż rewizję' : 'View revision' ) .
			Xml::element( 'td', null, ( $wgContLang->getCode() == 'pl') ? 'Autor' : 'Author' ) .
			Xml::element( 'td', null, ( $wgContLang->getCode() == 'pl') ? 'Zmiana' : 'Change' ) .
			Xml::element( 'td', null, ucfirst( wfMsg( 'revertmove' ) ) ) .			
			Xml::closeElement( 'tr' ) . "\n" .
			Xml::closeElement( 'thead' ) . "\n" .
			Xml::openElement( 'tbody' ) . "\n";
		
		return $html;
	}	
	
	function endHistoryList() {
		global $wgEnableHtmlDiff;
		
		$html =
			Xml::closeElement( 'tbody' ) . "\n" .
			Xml::closeElement( 'table' ) . "\n" .
			Xml::openElement( 'div', array( 'class' => 'visualClear' ) ) . Xml::closeElement( 'div' ) . "\n";
		
		if ( $wgEnableHtmlDiff ) {
			$html .= $this->submitButton( wfMsg( 'visualcomparison'),
				array(
						'name' => 'htmldiff',
						'class'     => 'historysubmit',
						'accesskey' => wfMsg( 'accesskey-visualcomparison' ),
						'title'     => wfMsg( 'tooltip-compareselectedversions' ),
				)
			);
			$html .= $this->submitButton( wfMsg( 'wikicodecomparison'),
				array(
						'class'     => 'historysubmit',
						'accesskey' => wfMsg( 'accesskey-compareselectedversions' ),
						'title'     => wfMsg( 'tooltip-compareselectedversions' ),
				)
			);
		} else {
			$html .= $this->submitButton( wfMsg( 'compareselectedversions'),
				array(
						'class'     => 'historysubmit',
						'accesskey' => wfMsg( 'accesskey-compareselectedversions' ),
						'title'     => wfMsg( 'tooltip-compareselectedversions' ),
				)
			);
		}
		
		$html .= Xml::closeElement( 'form' ) . "\n";
		
		return $html;
	}
	
	function historyLine( $row, $next, $counter = '', $notificationtimestamp = false, $latest = false, $firstInList = false ) {
		global $wgUser, $wgLang;
		$rev = new Revision( $row );
		$rev->setTitle( $this->mTitle );

		$s = '';
		$arbitrary = $this->diffButtons( $rev, $firstInList, $counter );
		$link = $this->revLink( $rev );

		$s .= Xml::tags( 'td', null, $arbitrary ) . "\n";
		$s .= Xml::tags( 'td', null, $link ) . "\n";
		$s .= Xml::tags( 'td', null, $this->mSkin->revUserLink( $rev, true ) ) . "\n";
		
		$s .= Xml::openElement( 'td' );

		if ( $row->rev_minor_edit ) {
			$s .= ' ' . Xml::element( 'span', array( 'class' => 'minor' ), wfMsg( 'minoreditletter') );
		}

		if ( !is_null( $size = $rev->getSize() ) && $rev->userCan( Revision::DELETED_TEXT ) ) {
			$s .= ' ' . $this->mSkin->formatRevisionSize( $size );
		}

		$s .= $this->mSkin->revComment( $rev, false, true );

		if ($notificationtimestamp && ($row->rev_timestamp >= $notificationtimestamp)) {
			$s .= ' <span class="updatedmarker">' .  wfMsgHtml( 'updatedmarker' ) . '</span>';
		}
		#add blurb about text having been deleted
		if( $rev->isDeleted( Revision::DELETED_TEXT ) ) {
			$s .= ' <tt>' . wfMsgHtml( 'deletedrev' ) . '</tt>';
		}
		
		$s .= Xml::closeElement( 'td' );
		
		$s .= Xml::openElement( 'td', array( 'style' => 'text-align: center;' ) );

		$tools = array();

		if ( !is_null( $next ) && is_object( $next ) ) {
			if (
				$this->mTitle->quickUserCan( 'edit' ) &&
				!$rev->isDeleted( Revision::DELETED_TEXT ) &&
				!$next->rev_deleted & Revision::DELETED_TEXT
			) {
				$undoTooltip = $latest
					? array( 'title' => wfMsg( 'tooltip-undo' ) )
					: array();
				$undolink = $this->mSkin->link(
					$this->mTitle,
					ucfirst( wfMsg( 'revertmove' ) ),
					$undoTooltip,
					array( 'action' => 'edit', 'undoafter' => $next->rev_id, 'undo' => $rev->getId() ),
					array( 'known', 'noclasses' )
				);
				$s .= Xml::tags( 'span', array( 'class' => 'mw-history-undo' ), $undolink ) . "\n";
			}
		}
		
		$s .= Xml::closeElement( 'td' );
		
		$s =
			Xml::openElement( 'tr', $this->gray ? array( 'class' => 'gray' ) : null ) . "\n" .
			$s .
			Xml::closeElement( 'tr' ) . "\n";
			
		$this->gray = $this->gray ? false : true;

		return $s;
	}	
}

class PageHistoryPager2 extends PageHistoryPager {
	function formatRow( $row ) {
		if ( $this->mLastRow ) {
			$latest = $this->mCounter == 1 && $this->mIsFirst;
			$firstInList = $this->mCounter == 1;
			$s = $this->mPageHistory->historyLine(
				$this->mLastRow, $row, $this->mCounter++,
				$this->mTitle->getNotificationTimestamp(), $latest, $firstInList
			);
		} else {
			$s = '';
		}
		$this->mLastRow = $row;
		return $s;
	}	
}