<?php

class PreferencesForm2 extends PreferencesForm {
	function __construct( &$request ) {
		global $wgDefaultUserOptions;
		
		parent::PreferencesForm( $request );
		
		$this->mToggles = array();
		if ( $this->mPosted ) {
			$togs = User::getToggles();
			foreach ( $togs as $tname ) {
				$this->mToggles[$tname] = $request->getCheck( "wpOp$tname" ) ? 1 : $wgDefaultUserOptions[$tname];
			}
		}		
	}
	
	function getToggle( $tname, $trailer = false, $disabled = false ) {
		global $wgUser, $wgLang;

		$this->mUsedToggles[$tname] = true;

		$attribs = array( 'id' => $tname );
		if ( $disabled )
			array_merge( $attribs, array( 'disabled' => 'disabled' ) );
		
		$toggle =
			'<div class="toggle">' .
			Xml::check( "wpOp{$tname}", $wgUser->getOption( $tname ), $attribs ) .
			Xml::tags( 'label', array( 'class' => 'toggletext', 'for' => $tname ), $wgLang->getUserToggle( $tname ) ) .
			( $trailer ? $trailer : '' ) .
			'</div>' . "\n";
		
		return $toggle;
	}

	/*
	function addRow($td1, $td2) {
		return "<tr><td class='mw-label'>$td1</td><td class='mw-input'>$td2</td></tr>";
	}
	*/
	
	function tableRow( $td1, $td2 = null, $td3 = null ) {
		$td3 = is_null( $td3 )
			? '<td class="pref-desc"></td>'
			: '<td class="pref-desc">' . $td3 . '</td>';;

		if ( is_null( $td2 ) ) {
			$td1 = '<td class="pref-label" colspan="2">' . $td1 . '</td>';
			$td2 = '';
		} else {
			$td1 = '<td class="pref-label">' . $td1 . '</td>';
			$td2 = '<td class="pref-input">' . $td2 . '</td>';
		}

		return '<tr>' . $td1 . $td2 . $td3 . '</tr>' . "\n";
	}
	
	function namespacesCheckboxes() {
		$namespaces = array(
			NS_MAIN => wfMsg( 'namespace-pages' ),
			NS_USER => wfMsg( 'namespace-userprofiles' ),
			NS_CATEGORY => wfMsg( 'namespace-folders' ),
			NS_IMAGE => wfMsg( 'namespace-attachments' )
		);

		$checkboxes = '';

		foreach ( $namespaces as $i => $name ) {
			$checkboxes .= Xml::checkLabel( $name, "wpNs{$i}", "wpNs{$i}", $this->mSearchNs[$i] ) . "\n";
		}

		return $checkboxes;
	}
	
	function mainPrefsForm( $status , $message = '' ) {
		global $wgUser, $wgOut, $wgLang, $wgAuth;
		global $wgAllowRealName;
		global $wgEnotifWatchlist, $wgEnotifUserTalk,$wgEnotifMinorEdits;
		global $wgRCShowWatchingUsers, $wgEnotifRevealEditorAddress;
		global $wgEnableEmail, $wgEnableUserEmail, $wgEmailAuthentication;
		global $wgCookieExpiration;
		global $wgEmailConfirmToEdit, $wgEnableMWSuggest;

		$wgOut->setPageTitle( wfMsg( 'preferences' ) );
		$wgOut->setArticleRelated( false );
		$wgOut->setRobotPolicy( 'noindex,nofollow' );
		$script = <<<HEREDOC
function initPreferences2()
{
	$('#preferences').children('ul').eq(0).tabs();
}

addOnloadHook( initPreferences2 );	
HEREDOC;
		$wgOut->addInlineScript( $script );

		$wgOut->disallowUserJs();

		if ( $this->mSuccess || 'success' == $status ) {
			$wgOut->wrapWikiMsg( '<div class="successbox"><strong>$1</strong></div>', 'savedprefs' );
		} else	if ( 'error' == $status ) {
			$wgOut->addWikiText( '<div class="errorbox"><strong>' . $message  . '</strong></div>' );
		} else if ( '' != $status ) {
			$wgOut->addWikiText( $message . "\n----" );
		}

		$togs = User::getToggles();

		$titleObj = SpecialPage::getTitleFor( 'Preferences' );

		# Pre-expire some toggles so they won't show if disabled
		$this->mUsedToggles[ 'shownumberswatching' ] = true;
		$this->mUsedToggles[ 'showupdated' ] = true;
		$this->mUsedToggles[ 'enotifwatchlistpages' ] = true;
		$this->mUsedToggles[ 'enotifusertalkpages' ] = true;
		$this->mUsedToggles[ 'enotifminoredits' ] = true;
		$this->mUsedToggles[ 'enotifrevealaddr' ] = true;
		$this->mUsedToggles[ 'ccmeonemails' ] = true;
		$this->mUsedToggles[ 'uselivepreview' ] = true;
		$this->mUsedToggles[ 'noconvertlink' ] = true;


		if ( !$this->mEmailFlag ) { $emfc = 'checked="checked"'; }
		else { $emfc = ''; }

		if ($wgEmailAuthentication && ($this->mUserEmail != '') ) {
			if( $wgUser->getEmailAuthenticationTimestamp() ) {
				$time = $wgLang->timeAndDate( $wgUser->getEmailAuthenticationTimestamp(), true );
				$d = $wgLang->date( $wgUser->getEmailAuthenticationTimestamp(), true );
				$t = $wgLang->time( $wgUser->getEmailAuthenticationTimestamp(), true );
				$emailauthenticated = wfMsg('emailauthenticated', $time, $d, $t ).'<br />';
				$disableEmailPrefs = false;
			} else {
				$disableEmailPrefs = true;
				$skin = $wgUser->getSkin();
				$emailauthenticated = wfMsg('emailnotauthenticated').'<br />' .
					$skin->makeKnownLinkObj( SpecialPage::getTitleFor( 'Confirmemail' ),
						wfMsg( 'emailconfirmlink' ) ) . '<br />';
			}
		} else {
			$emailauthenticated = '';
			$disableEmailPrefs = false;
		}

		if ($this->mUserEmail == '') {
			$emailauthenticated = wfMsg( 'noemailprefs' ) . '<br />';
		}

		# </FIXME>

		$wgOut->addHTML(
			'<div class="visualClear"></div>' .
			'<form action="' . $titleObj->getLocalUrl() . '" method="post" id="mw-preferences-form">' .
			'<div id="preferences" class="ui-tabs ui-widget-content ui-corner-all">'
		);
		
		$wgOut->addHTML(
			'<ul>' .
			Xml::tags( 'li', null, Xml::tags( 'a', array( 'href' => '#prefs-personal' ), Xml::element( 'span', null, wfMsg( 'prefs-personal' ) ) ) ) .
			Xml::tags( 'li', null, Xml::tags( 'a', array( 'href' => '#prefs-watchlist' ), Xml::element( 'span', null, wfMsg( 'prefs-watchlist' ) ) ) ) .
			Xml::tags( 'li', null, Xml::tags( 'a', array( 'href' => '#prefs-misc' ), Xml::element( 'span', null, wfMsg( 'prefs-misc' ) ) ) ) .
			'</ul>'
		);

		# User data

		$wgOut->addHTML(
			'<div id="prefs-personal">' .
			'<fieldset>' .
			'<legend>' . wfMsg('prefs-personal') . '</legend>' .
			'<table>'
		);

		$userInformationHtml = $this->tableRow( wfMsgHtml( 'username' ), htmlspecialchars( $wgUser->getName() ) );

		if( wfRunHooks( 'PreferencesUserInformationPanel', array( $this, &$userInformationHtml ) ) ) {
			$wgOut->addHTML( $userInformationHtml );
		}

		if ( $wgAllowRealName ) {
			$wgOut->addHTML(
				$this->tableRow(
					Xml::label( wfMsgHtml('yourrealname'), 'wpRealName' ),
					Xml::input( 'wpRealName', 25, $this->mRealName, array( 'id' => 'wpRealName' ) ),
					Xml::tags('div', array( 'class' => 'prefsectiontip' ),
						wfMsgExt( 'prefs-help-realname', 'parseinline' )
					)
				)
			);
		}
		if ( $wgEnableEmail ) {
			$wgOut->addHTML(
				$this->tableRow(
					Xml::label( wfMsgHtml('youremail'), 'wpUserEmail' ),
					Xml::input( 'wpUserEmail', 25, $this->mUserEmail, array( 'id' => 'wpUserEmail', 'autocomplete' => 'off' ) ),
					Xml::tags('div', array( 'class' => 'prefsectiontip' ),
						wfMsgExt( $wgEmailConfirmToEdit ? 'prefs-help-email-required' : 'prefs-help-email', 'parseinline' )
					)
				)
			);
		}
		$wgOut->addHTML(
			'</table>' .
			'</fieldset>'			
		);
		

		# Password
		if( $wgAuth->allowPasswordChange() ) {
			$wgOut->addHTML(
				'<fieldset>' .
				Xml::element( 'legend', null, wfMsg( 'changepassword' ) ) .
				'<table>'
			);
			
			$wgOut->addHTML(
				$this->tableRow(
					Xml::label( wfMsg( 'oldpassword' ), 'wpOldpass' ),
					Xml::password( 'wpOldpass', 25, '', array( 'id' => 'wpOldpass', 'autocomplete' => 'off' ) )
				) .
				$this->tableRow(
					Xml::label( wfMsg( 'newpassword' ), 'wpNewpass' ),
					Xml::password( 'wpNewpass', 25, '', array( 'id' => 'wpNewpass', 'autocomplete' => 'off' ) )
				) .
				$this->tableRow(
					Xml::label( wfMsg( 'retypenew' ), 'wpRetypePass' ),
					Xml::password( 'wpRetypePass', 25, '', array( 'id' => 'wpRetypePass', 'autocomplete' => 'off' ) )
				)
			);
			if( $wgCookieExpiration > 0 ){
				$wgOut->addHTML(
					Xml::tags( 'tr', null,
						Xml::tags( 'td', array( 'colspan' => '2' ),
							$this->getToggle( "rememberpassword" )
						)
					)
				);
			} else {
				$this->mUsedToggles['rememberpassword'] = true;
			}
		}

		$wgOut->addHTML(
			'</table>' .
			'</fieldset>' .
			'</div>'
		);

		# Watchlist
		$wgOut->addHTML(
			Xml::openElement( 'div', array( 'id' => 'prefs-watchlist' ) ) .
			Xml::openElement( 'fieldset' ) .
			Xml::element( 'legend', null, wfMsg( 'prefs-watchlist' ) ) .
			Xml::inputLabel( wfMsg( 'prefs-watchlist-days' ), 'wpWatchlistDays', 'wpWatchlistDays', 3, $this->mWatchlistDays ) .
			Xml::element( 'br' ) . Xml::element( 'br' )
		);
		
		if ( $wgEnableEmail ) {
			$enotifwatchlistpages = ($wgEnotifWatchlist) ? $this->getToggle( 'enotifwatchlistpages', false, $disableEmailPrefs ) : '';
			$enotifminoredits = ($wgEnotifWatchlist && $wgEnotifMinorEdits) ? $this->getToggle( 'enotifminoredits', false, $disableEmailPrefs ) : '';
			
			$wgOut->addHTML(
				Xml::openElement( 'table', array( 'cellpadding' => 0, 'cellspacing' => 0 ) ) .
				Xml::openElement( 'tr' ) .
				Xml::openElement( 'td' ) .
				$enotifwatchlistpages .
				$enotifminoredits
			);
			
			if ( empty( $this->mUserEmail ) ) {
				$wgOut->addHTML(
					Xml::closeElement( 'td' ) .
					Xml::openElement( 'td' ) .
					Xml::tags('div', array( 'class' => 'prefsectiontip' ), wfMsg( 'noemailprefs' ) )
				);
			}
			
			$wgOut->addHTML(
				Xml::closeElement( 'td' ) .
				Xml::closeElement( 'tr' ) .
				Xml::closeElement( 'table' ) .
				Xml::element( 'br' )
			);
		}
		
		$wgOut->addHTML(
			$this->getToggles( array( 'watchlisthideminor', 'watchlisthideown' ) )			
		);

		if( $wgUser->isAllowed( 'createpage' ) || $wgUser->isAllowed( 'createtalk' ) )
			$wgOut->addHTML( $this->getToggle( 'watchcreations' ) );
		foreach( array( 'edit' => 'watchdefault', 'move' => 'watchmoves' ) as $action => $toggle ) {
			if( $wgUser->isAllowed( $action ) )
				$wgOut->addHTML( $this->getToggle( $toggle ) );
		}
		$this->mUsedToggles['watchcreations'] = true;
		$this->mUsedToggles['watchdefault'] = true;
		$this->mUsedToggles['watchmoves'] = true;

		$wgOut->addHTML(
			Xml::closeElement( 'fieldset' ) .
			Xml::closeElement( 'div' )
		);

		# Misc
		#
		$wgOut->addHTML( Xml::openElement( 'div', array( 'id' => 'prefs-misc' ) ) );
		
		$msgUnderline = htmlspecialchars( wfMsg ( 'tog-underline' ) );
		$msgUnderlinenever = htmlspecialchars( wfMsg ( 'underline-never' ) );
		$msgUnderlinealways = htmlspecialchars( wfMsg ( 'underline-always' ) );
		$msgUnderlinedefault = htmlspecialchars( wfMsg ( 'underline-default' ) );
		$uopt = $wgUser->getOption("underline");
		$s0 = $uopt == 0 ? ' selected="selected"' : '';
		$s1 = $uopt == 1 ? ' selected="selected"' : '';
		$s2 = $uopt == 2 ? ' selected="selected"' : '';
		$wgOut->addHTML(
			Xml::openElement( 'fieldset' ) .
			Xml::element( 'legend', null, wfMsg('prefs-previewingoptions') ) .
			"
<div class='toggle'><p><label for='wpOpunderline'>$msgUnderline</label>
<select name='wpOpunderline' id='wpOpunderline'>
<option value=\"0\"$s0>$msgUnderlinenever</option>
<option value=\"1\"$s1>$msgUnderlinealways</option>
<option value=\"2\"$s2>$msgUnderlinedefault</option>
</select></p></div>" .
			$this->getToggle( 'highlightbroken' ) .
			$this->getToggle( 'numberheadings' ) .
			$this->getToggle( 'showtoc' ) .
			Xml::closeElement( 'fieldset' )
		);		
		
		$wgOut->addHTML(
			Xml::openElement( 'fieldset' ) .
			Xml::element( 'legend', null, wfMsg('prefs-editingoptions') ) .
			$this->getToggle( 'editsection' ) .
			$this->getToggle( 'editsectiononrightclick' ) .
			$this->getToggle( 'editondblclick' ) .
			$this->getToggle( 'minordefault' ) .
			$this->getToggle( 'uselivepreview' ) .
			Xml::closeElement( 'fieldset' )
		);
		
		$mwsuggest = $wgEnableMWSuggest ?
			$this->addRow(
				Xml::label( wfMsg( 'mwsuggest-disable' ), 'wpDisableMWSuggest' ),
				Xml::check( 'wpDisableMWSuggest', $this->mDisableMWSuggest, array( 'id' => 'wpDisableMWSuggest' ) )
			) : '';
		$ps = $this->namespacesCheckboxes();
		$wgOut->addHTML(
			Xml::openElement( 'fieldset' ) .
			Xml::element( 'legend', null, wfMsg( 'prefs-searchoptions' ) ) .
			Xml::openElement( 'table' ) .
			$this->addRow(
				Xml::label( wfMsg( 'resultsperpage' ), 'wpSearch' ),
				Xml::input( 'wpSearch', 4, $this->mSearch, array( 'id' => 'wpSearch' ) )
			) .
			$mwsuggest .
			Xml::closeElement( 'table' ) .
			wfMsgExt( 'defaultns', array( 'parse' ) ) .
			$ps .			
			Xml::closeElement( 'fieldset' )
		);
		
		$array = array(
		    'justify', 'hideminor', 'extendwatchlist', 'usenewrc', 'showtoolbar',
		    'editwidth', 'watchdeletion', 'previewontop', 'previewonfirst', 'nocache',
		    'fancysig', 'externaleditor', 'externaldiff', 'showjumplinks',
		    'forceeditsummary', 'watchlisthidebots', 'watchlisthideanons',
		    'watchlisthideliu', 'diffonly', 'showhiddencats', 'norollbackdiff',
			'riched_disable', 'riched_start_disabled', 'riched_use_popup',
			'riched_use_toggle', 'riched_toggle_remember_state', 'riched_disable_ns_main',
			'riched_disable_ns_talk', 'riched_disable_ns_user', 'riched_disable_ns_user_talk',
			'riched_disable_ns_project', 'riched_disable_ns_project_talk', 'riched_disable_ns_image',
			'riched_disable_ns_image_talk', 'riched_disable_ns_mediawiki', 'riched_disable_ns_mediawiki_talk',
			'riched_disable_ns_template', 'riched_disable_ns_template_talk', 'riched_disable_ns_help',
			'riched_disable_ns_help_talk', 'riched_disable_ns_category', 'riched_disable_ns_category_talk'
		);
		
		foreach ( $array as $item ) {
		    $this->mUsedToggles[$item] = true;
		}

		$wgOut->addHTML(
			Xml::openElement( 'fieldset' ) .
			Xml::element( 'legend', null, wfMsg('prefs-otheroptions') )
		);
				
		foreach ( $togs as $tname ) {
			if( !array_key_exists( $tname, $this->mUsedToggles ) ) {
				$wgOut->addHTML( $this->getToggle( $tname ) );
			}
		}
		
		$wgOut->addHTML( Xml::closeElement( 'fieldset' ) );

		$wgOut->addHTML( Xml::closeElement( 'div' ) );
		
		wfRunHooks( 'RenderPreferencesForm', array( $this, $wgOut ) );

		$token = htmlspecialchars( $wgUser->editToken() );
		$skin = $wgUser->getSkin();
		$wgOut->addHTML( "
	<div id='prefsubmit'>
	<div>
		<input type='submit' name='wpSaveprefs' class='btnSavePrefs' value=\"" . wfMsgHtml( 'saveprefs' ) . '"'.$skin->tooltipAndAccesskey('save')." />
		<input type='submit' name='wpReset' value=\"" . wfMsgHtml( 'resetprefs' ) . "\" />
	</div>

	</div>

	<input type='hidden' name='wpEditToken' value=\"{$token}\" />
	<div class='visualClear'></div></div></form>\n" );

		$wgOut->addHTML( Xml::tags( 'div', array( 'class' => "prefcache" ),
			wfMsgExt( 'clearyourcache', 'parseinline' ) )
		);
	}
	
  # User Avatar Functions
  # < user avatar functions >

  /*
   * Look for user Avatar
   * @return object on success false on fail
   */
	/*
  function getUserAvatar( $wgUser ){
    $userAvatar =   RepoGroup::singleton()->findFile($wgUser->mName.'_avatar.jpg');
    return $userAvatar;
  }
*/

  # </ user avatar functions >
}