<?php

class SpecialRecentChanges2 extends SpecialRecentChanges {
	public function webOutput( $rows, $opts ) {
		global $wgOut, $wgUser, $wgRCShowWatchingUsers, $wgShowUpdatedMarker;
		global $wgAllowCategorizedRecentChanges;

		$limit = $opts['limit'];

		if ( !$this->including() ) {
			// Output options box
			$this->doHeader( $opts );
		}

		// And now for the content
		$wgOut->setSyndicated( true );

		$list = ChangesList2::newFromUser( $wgUser );

		$s = $list->beginRecentChangesList();
		$counter = 1;

		$showWatcherCount = $wgRCShowWatchingUsers && $wgUser->getOption( 'shownumberswatching' );
		$watcherCache = array();

		$dbr = wfGetDB( DB_SLAVE );

		foreach( $rows as $obj ){
			if( $limit == 0) {
				break;
			}

			if ( ! ( $opts['hideminor']     && $obj->rc_minor     ) &&
			     ! ( $opts['hidepatrolled'] && $obj->rc_patrolled ) ) {
				$rc = RecentChange::newFromRow( $obj );
				$rc->counter = $counter++;

				if ($wgShowUpdatedMarker
					&& !empty( $obj->wl_notificationtimestamp )
					&& ($obj->rc_timestamp >= $obj->wl_notificationtimestamp)) {
						$rc->notificationtimestamp = true;
				} else {
					$rc->notificationtimestamp = false;
				}

				$rc->numberofWatchingusers = 0; // Default
				if ($showWatcherCount && $obj->rc_namespace >= 0) {
					if (!isset($watcherCache[$obj->rc_namespace][$obj->rc_title])) {
						$watcherCache[$obj->rc_namespace][$obj->rc_title] =
						 	$dbr->selectField( 'watchlist',
								'COUNT(*)',
								array(
									'wl_namespace' => $obj->rc_namespace,
									'wl_title' => $obj->rc_title,
								),
								__METHOD__ . '-watchers' );
					}
					$rc->numberofWatchingusers = $watcherCache[$obj->rc_namespace][$obj->rc_title];
				}
				$s .= $list->recentChangesLine( $rc, !empty( $obj->wl_user ) );
				--$limit;
			}
		}
		$s .= $list->endRecentChangesList();
		
		$wgOut->addHTML( $s );
	}
	
	public function doHeader( $opts ) {
		global $wgScript, $wgOut, $wgRCLinkDays, $wgLang;

		$title = $this->getTitle();
		
		$defaults = $opts->getAllValues();
		$nondefaults = $opts->getChangedValues();
		$opts->consumeValues( array( 'hidemyself', 'hideminor', 'days', 'namespace' ) );

		$options = array_merge( $defaults, $nondefaults );
		$out =
			Xml::openElement( 'form', array( 'action' => $title->getLocalUrl(), 'method' => 'post' ) ) . "\n" .
			Xml::openElement( 'div', array( 'style' => 'padding: 10px 0;') ) . "\n" .
			Xml::checkLabel( wfMsg( 'rcshowhidemine', wfMsg( 'hide' ) ), 'hidemyself', 'hidemyself', $options['hidemyself'] ) . "\n" .
			Xml::checkLabel( wfMsg( 'rcshowhideminor', wfMsg( 'hide' ) ), 'hideminor', 'hideminor', $options['hideminor'] ) . "\n" .
			Xml::closeElement( 'div' ) . "\n" .
			Xml::label( wfMsg( 'showchangesdays1' ), 'days' ) . "\n" .
			Xml::openElement( 'select', array( 'id' => 'days', 'name' => 'days' ) ) . "\n";
		
		$wgRCLinkDays[] = $options['days'];
		sort( $wgRCLinkDays );
		$wgRCLinkDays = array_unique( $wgRCLinkDays );
		
		foreach( $wgRCLinkDays as $value ) {
			$out .= Xml::option( $wgLang->formatNum( $value ), $value, ( $value == $options['days'] ) ) . "\n";
		}
		
		$out .=
			Xml::closeElement( 'select' ) . "\n" .
			wfMsg( 'showchangesdays2' ) . "\n" .
			Xml::element( 'label', array( 'for' => 'namespace', 'style' => 'margin-left: 20px;' ), wfMsg( 'showchangesfor' ) ) . "\n" .
			Xml::openElement( 'select', array( 'id' => 'namespace', 'name' => 'namespace' ) ) . "\n";
		
		$namespaces = array(
			'' => wfMsg( 'namespacesall' ),
			NS_MAIN => wfMsg( 'namespace-pages' ),
			NS_USER => wfMsg( 'namespace-userprofiles' ),
			NS_CATEGORY => wfMsg( 'namespace-folders' ),
			NS_IMAGE => wfMsg( 'namespace-attachments' )
		);
		
		foreach ( $namespaces as $i => $name ) {
			$out .= Xml::option( $name, $i, ( $i === $options['namespace'] ) ) . "\n";
		}
		
		$out .=
			Xml::closeElement( 'select' ) . "\n" .
			Xml::submitButton( wfMsg( 'allpagessubmit' ), array( 'style' => 'margin-left: 40px;' ) ) . "\n" .
			Xml::hidden( 'title', $title->getPrefixedText() ) . "\n";

		$unconsumed = $opts->getUnconsumedValues();
		foreach ( $unconsumed as $key => $value ) {
			$out .= Xml::hidden( $key, $value );
		}
			
		$out .=	Xml::closeElement( 'form' ) . "\n";

		$wgOut->addHTML(
			Xml::fieldset( wfMsg( 'recentchanges-legend' ), $out, array( 'class' => 'rcoptions' ) )
		);

		$this->setBottomText( $wgOut, $opts );
	}
}

class ChangesList2 extends ChangesList {
	private $gray;
	
	public static function newFromUser( &$user ) {
		$sk = $user->getSkin();
		$list = NULL;
		if( wfRunHooks( 'FetchChangesList', array( &$user, &$sk, &$list ) ) ) {
			return new self( $sk );
		} else {
			return $list;
		}
	}
	
	protected function insertArticleLink(&$s, &$rc, $unpatrolled, $watched) {
		# Article link
		# If it's a new article, there is no diff link, but if it hasn't been
		# patrolled yet, we need to give users a way to do so
		$params = ( $unpatrolled && $rc->mAttribs['rc_type'] == RC_NEW )
			? array( 'rcid' => $rc->mAttribs['rc_id'] )
			: array();
		$title = $rc->getTitle();
		if( $this->isDeleted($rc,Revision::DELETED_TEXT) ) {
			$articlelink = $this->skin->link( $title, $title->getBaseText(), $params );
			$articlelink = '<span class="history-deleted">'.$articlelink.'</span>';
		} else {
		    $articlelink = ' '. $this->skin->link( $title, $title->getBaseText(), $params );
		}
		if( $watched )
			$articlelink = "<strong class=\"mw-watched\">{$articlelink}</strong>";
		global $wgContLang;
		$articlelink .= $wgContLang->getDirMark();

		wfRunHooks('ChangesListInsertArticleLink',
			array(&$this, &$articlelink, &$s, &$rc, $unpatrolled, $watched));

		$s .= ' '.$articlelink;
	}
	
	public function insertUserRelatedLinks(&$s, &$rc) {
		if ( $this->isDeleted($rc,Revision::DELETED_USER) ) {
		   $s .= ' <span class="history-deleted">' . wfMsgHtml('rev-deleted-user') . '</span>';
		} else {
		  $s .= $this->skin->userLink( $rc->mAttribs['rc_user'], $rc->mAttribs['rc_user_text'] );
		}
	}
	
	protected function insertTimestamp(&$s, $rc) {
		global $wgLang;
		# Timestamp
		$s .= ' ' . $wgLang->time( $rc->mAttribs['rc_timestamp'], true, true );
	}
	
	protected function recentChangesFlags( $new, $minor, $patrolled, $nothing = '&nbsp;', $bot = false ) {
		$f = $minor ? '<span class="minor">' . 'minor' . '</span>'
				: $nothing;
				
		if ( $f != $nothing ) {
			$f = ' &mdash; ' . $f;
		}
				
		return $f;
	}
	
	protected function insertDiffHist(&$s, &$rc, $unpatrolled) {
		# Diff link
		if( $this->userCan($rc,Revision::DELETED_TEXT) && !in_array( $rc->mAttribs['rc_type'], array( RC_NEW, RC_LOG ) ) ) {
			$rcidparam = $unpatrolled
				? array( 'rcid' => $rc->mAttribs['rc_id'] )
				: array();
			$diffLink = $this->skin->makeKnownLinkObj( $rc->getTitle(), 'last difference',
				wfArrayToCGI( array(
					'curid' => $rc->mAttribs['rc_cur_id'],
					'diff'  => $rc->mAttribs['rc_this_oldid'],
					'oldid' => $rc->mAttribs['rc_last_oldid'] ),
					$rcidparam ),
				'', '', ' tabindex="'.$rc->counter.'"');
			$s .= ' &mdash; ' . $diffLink;
		}
	}
	
	protected function getChangeIcon( $ns, $type, $action, $comment ) {
		switch ( $ns ) {
			case NS_MAIN:
				$iconObj = 'page'; break;
			case NS_USER:
				$iconObj = 'user'; break;
			case NS_IMAGE:
				$iconObj = 'file'; break;
			case NS_CATEGORY:
				$iconObj = 'folder'; break;
		}
		if ( defined( 'LIVEBLEND' ) ) {
			switch ( $ns ) {
				case NS_QUESTION:
					$iconObj = 'question'; break;
				case NS_CONCEPT:
					$iconObj = 'concept'; break;
			}
		}
		if ( empty( $iconObj ) ) {
			$iconObj = 'page';
		}
		switch ( $type ) {
			case RC_NEW:
				$iconAction = 'create'; break;
			case RC_EDIT:
				$iconAction = 'edit'; break;
			case RC_MOVE:
				$iconAction = 'move'; break;
			case RC_LOG:
				$iconAction = $action; break;
		}
		
		if ( $comment == 'New comment' ) {
			list( $iconObj, $iconAction ) = array( 'comment', 'create' );
		}
		
		$bg = $this->gray ? 'gray' : 'white';
		
		return array(
			'file' => "rc_{$iconObj}_{$iconAction}_{$bg}.png",
			'title' => ucfirst( $iconObj ) . ' ' . ucfirst( $iconAction )
		);
	}
	
	protected function insertDateHeader(&$s, $rc_timestamp) {
		global $wgLang;

		# Make date header if necessary
		$date = $wgLang->date( $rc_timestamp, true, true );
		if( $date != $this->lastdate ) {
			if( '' != $this->lastdate ) {
				$s .= "</ul>\n";
			}
			$s .= '<h4 style="margin: 20px 0 10px;">'.$date."</h4>\n<ul class=\"special\" style=\"list-style-type: none; list-style-image: none; margin: 0;\">";
			$this->lastdate = $date;
			$this->rclistOpen = true;
			
			$this->gray = true;
		}
	}
	
	public function recentChangesLine( &$rc, $watched = false ) {
		global $wgContLang, $wgRCShowChangedSize, $wgUser;

		# Extract DB fields into local scope
		// FIXME: Would be good to replace this extract() call with something that explicitly initializes local variables.
		extract( $rc->mAttribs );

		# Should patrol-related stuff be shown?
		$unpatrolled = $wgUser->useRCPatrol() && $rc_patrolled == 0;

		$dateheader = ""; // $s now contains only <li>...</li>, for hooks' convenience.
		$this->insertDateHeader($dateheader,$rc_timestamp);

		$s = '';

		$this->insertArticleLink($s, $rc, $unpatrolled, $watched);
		
		$s .= ' &mdash; ';
		
		$this->insertUserRelatedLinks($s,$rc);
		
		$this->insertTimestamp($s,$rc);
		
		// Moved pages
		if( $rc_type == RC_MOVE || $rc_type == RC_MOVE_OVER_REDIRECT ) {
			$this->insertMove( $s, $rc );
		// Log entries (old format) or log targets, and special pages
		} elseif( $rc_namespace == NS_SPECIAL ) {
			list( $specialName, $specialSubpage ) = SpecialPage::resolveAliasWithSubpage( $rc_title );
			if ( $specialName == 'Log' ) {
				$this->insertLog( $s, $rc->getTitle(), $specialSubpage );
			} else {
				wfDebug( "Unexpected special page in recentchanges\n" );
			}
		// Regular entries
		} else {
			# M, N, b and ! (minor, new, bot and unpatrolled)
			$s .= $this->recentChangesFlags( $rc_type == RC_NEW, $rc_minor, $unpatrolled, '', $rc_bot );
			
			$s .= ' ';
			
			if( $wgRCShowChangedSize ) {
				$s .=  ( $rc->getCharacterDifference() == '' ? '' : ' &mdash; ' . $rc->getCharacterDifference() );
			}
			
			$this->insertDiffHist($s, $rc, $unpatrolled);
		}

		# Mark revision as deleted if so
		if ( !$rc_log_type && $this->isDeleted($rc,Revision::DELETED_TEXT) )
		   $s .= ' <tt>' . wfMsgHtml( 'deletedrev' ) . '</tt>';
		if($rc->numberofWatchingusers > 0) {
			$s .= ' ' . wfMsg('number_of_watching_users_RCview',  $wgContLang->formatNum($rc->numberofWatchingusers));
		}

		$icon = $this->getChangeIcon(
			$rc->getTitle()->getNamespace(),
			$rc_type,
			$rc->mAttribs['rc_log_action'],
			$rc->mAttribs['rc_comment']
		);
		
		$title = $icon['title'];
		$icon = '/extensions/BiznesWikiViews/images/' . $icon['file'];
		
		$s = Xml::tags( 'li', array( 'title' => $title, 'class' => $this->gray ? 'rc_li gray' : 'rc_li', 'style' => 'background-image: url(' . $icon . ');' ), $s );
		
		$this->gray = $this->gray ? false : true;
		
		return $dateheader . $s;
	}
}
