<?php

class SpecialSearch2 extends SpecialSearch {
	private $gray;
	
	function setupPage( $term ) {
		global $wgOut;
		if( !empty( $term ) ){
			$wgOut->setPageTitle( wfMsg( 'searchresults') );
			$wgOut->setHTMLTitle( wfMsg( 'pagetitle', wfMsg( 'searchresults-title', $term) ) );
		}			
		$wgOut->setArticleRelated( false );
		$wgOut->setRobotPolicy( 'noindex,nofollow' );
	}	

	function showResults( $term ) {
		global $wgOut, $wgUser;
		$sk = $wgUser->getSkin();

		$this->setupPage( $term );

		if( '' === trim( $term ) ) {
			$wgOut->addHTML( $this->powerSearchBox( $term ) );
			$wgOut->addHTML( $this->powerSearchFocus() );
			return;
		} else {
			$wgOut->addHTML( $this->powerSearchBox( $term ) );
		}

		global $wgDisableTextSearch;
		if ( $wgDisableTextSearch ) {
			global $wgSearchForwardUrl;
			if( $wgSearchForwardUrl ) {
				$url = str_replace( '$1', urlencode( $term ), $wgSearchForwardUrl );
				$wgOut->redirect( $url );
				return;
			}
			global $wgInputEncoding;
			$wgOut->addHTML(
				Xml::openElement( 'fieldset' ) .
				Xml::element( 'legend', null, wfMsg( 'search-external' ) ) .
				Xml::element( 'p', array( 'class' => 'mw-searchdisabled' ), wfMsg( 'searchdisabled' ) ) .
				wfMsg( 'googlesearch',
					htmlspecialchars( $term ),
					htmlspecialchars( $wgInputEncoding ),
					htmlspecialchars( wfMsg( 'searchbutton' ) )
				) .
				Xml::closeElement( 'fieldset' )
			);
			return;
		}

		$search = SearchEngine::create();
		$search->setLimitOffset( $this->limit, $this->offset );
		$search->setNamespaces( $this->namespaces );
		$search->showRedirects = $this->searchRedirects;
		$rewritten = $search->replacePrefixes($term);

		$titleMatches = $search->searchTitle( $rewritten );

		// Sometimes the search engine knows there are too many hits
		if ($titleMatches instanceof SearchResultTooMany) {
			$wgOut->addWikiText( '==' . wfMsg( 'toomanymatches' ) . "==\n" );
			$wgOut->addHTML( $this->powerSearchBox( $term ) );
			$wgOut->addHTML( $this->powerSearchFocus() );
			return;
		}
		
		$textMatches = $search->searchText( $rewritten );

		// did you mean... suggestions
		if($textMatches && $textMatches->hasSuggestion()){
			$st = SpecialPage::getTitleFor( 'Search' );			
			$stParams = wfArrayToCGI( array( 
					'search' 	=> $textMatches->getSuggestionQuery(), 
					'fulltext' 	=> wfMsg('search')),
					$this->powerSearchOptions());
					
			$suggestLink = '<a href="'.$st->escapeLocalURL($stParams).'">'.
					$textMatches->getSuggestionSnippet().'</a>';
			 		
			$wgOut->addHTML('<div class="searchdidyoumean">'.wfMsg('search-suggest',$suggestLink).'</div>');
		}

		// show number of results
		$num = ( $titleMatches ? $titleMatches->numRows() : 0 )
			+ ( $textMatches ? $textMatches->numRows() : 0);
		$totalNum = 0;
		if($titleMatches && !is_null($titleMatches->getTotalHits()))
			$totalNum += $titleMatches->getTotalHits();
		if($textMatches && !is_null($textMatches->getTotalHits()))
			$totalNum += $textMatches->getTotalHits();
		if ( $num > 0 ) {
			if ( $totalNum > 0 ){
				$top = wfMsgExt('showingresultstotal', array( 'parseinline' ), 
					$this->offset+1, $this->offset+$num, $totalNum, $num );
			} elseif ( $num >= $this->limit ) {
				$top = wfShowingResults( $this->offset, $this->limit );
			} else {
				$top = wfShowingResultsNum( $this->offset, $this->limit, $num );
			}
			$wgOut->addHTML( "<p class='mw-search-numberresults'>{$top}</p>\n" );
		}

		if( $titleMatches ) {
			if( $titleMatches->numRows() ) {
				$wgOut->addHTML( Xml::tags( 'h2', array( 'style' => 'padding: 10px 0;' ), wfMsg( 'titlematches' ) ) );
				$wgOut->addHTML( $this->showMatches( $titleMatches ) );
			}
			$titleMatches->free();
		}

		if( $textMatches ) {
			// output appropriate heading
			if( $textMatches->numRows() ) {
				if($titleMatches)
					$wgOut->addHTML( Xml::tags( 'h2', array( 'style' => 'padding: 20px 0 10px;' ), wfMsg( 'textmatches' ) ) );
				else // if no title matches the heading is redundant
					$wgOut->addHTML("<hr/>");								
			} elseif( $num == 0 ) {
				# Don't show the 'no text matches' if we received title matches
				$wgOut->addHTML( Xml::tags( 'h2', array( 'style' => 'padding: 20px 0 10px;' ), wfMsg( 'notextmatches' ) ) );
			}
			// show interwiki results if any
			if( $textMatches->hasInterwikiResults() )
				$wgOut->addHtml( $this->showInterwiki( $textMatches->getInterwikiResults(), $term ));
			// show results
			if( $textMatches->numRows() )
				$wgOut->addHTML( $this->showMatches( $textMatches ) );

			$textMatches->free();
		}

		if ( $num == 0 ) {
			$wgOut->addWikiMsg( 'nonefound' );
		} elseif ( $this->offset ) {
			$prevnext = wfViewPrevNext( $this->offset, $this->limit,
				SpecialPage::getTitleFor( 'Search' ),
				wfArrayToCGI(
					$this->powerSearchOptions(),
					array( 'search' => $term ) ),
					($num < $this->limit) );
			$wgOut->addHTML( "<p class='mw-search-pager-bottom'>{$prevnext}</p>\n" );
		}
	}
	
	function showMatches( &$matches ) {
		global $wgContLang;
		
		$this->gray = true;
		
		$terms = $wgContLang->convertForSearchResult( $matches->termMatches() );

		$html = Xml::openElement( 'table', array( 'class' => 'mw-search-results', 'style' => 'width: 100%;' ) ) . "\n";

		while ( $result = $matches->next() ) {
			$html .= $this->showHit( $result, $terms );
		}
		
		$html .= Xml::closeElement( 'table' ) . "\n";

		return $html;
	}
	
	function showHit( $result, $terms ) {
		global $wgUser, $wgContLang, $wgLang;
		
		if ( $result->isBrokenTitle() ) return;
		
		$t = $result->getTitle();
		$sk = $wgUser->getSkin();
		
		$text = $result->getTitleSnippet( $terms );
		if ( empty( $text ) ) $text = $t->getBaseText();

		$link = $sk->link( $t, $text );

		if ( !$t->userCanRead() ) {
			$html =
				Xml::openElement( 'tr', $this->gray ? array( 'class' => 'gray' ) : null ) . "\n" .
				Xml::tags( 'td', array( 'colspan' => 2 ), $link ) . "\n" .
				Xml::closeElement( 'tr' ) . "\n";
				
			$this->gray = $this->gray ? false : true;
			
			return $html;
		}

		if ( $result->isMissingRevision() ) return;

		// format redirects / relevant sections
		$redirectTitle = $result->getRedirectTitle();
		$redirectText = $result->getRedirectSnippet( $terms );
		$sectionTitle = $result->getSectionTitle();
		$sectionText = $result->getSectionSnippet( $terms );
		$redirect = '';
		if( !is_null($redirectTitle) )
			$redirect = "<span class='searchalttitle'>"
				.wfMsg('search-redirect',$sk->makeKnownLinkObj( $redirectTitle, $redirectText))
				."</span>";
		$section = '';
		if( !is_null($sectionTitle) )
			$section = "<span class='searchalttitle'>" 
				.wfMsg('search-section', $sk->makeKnownLinkObj( $sectionTitle, $sectionText))
				."</span>";

		$extract = $result->getTextSnippet( $terms );
		if ( !empty( $extract ) ) {
			$extract =
				"\n" . Xml::element( 'br' ) . "\n" .
				Xml::tags( 'div', array( 'class' => 'searchresult' ), "(&hellip;) {$extract}" );
		}
		
		// format score
		if ( is_null( $result->getScore() ) ) {
			// Search engine doesn't report scoring info
			$score = '';
		} else {
			$percent = sprintf( '%2.1f', $result->getScore() * 100 );
			$score = wfMsg( 'search-result-score', $wgLang->formatNum( $percent ) )
				. ' - ';
		}

		// format description
		$byteSize = $result->getByteSize();
		$wordCount = $result->getWordCount();
		$timestamp = $result->getTimestamp();
				
		$name =
			"{$link} {$redirect} {$section}" .
			$extract . "\n";
			
		if ( $t->getNamespace() != NS_IMAGE ) {
			$info = wfMsg( 'search-result-size', $sk->formatSize( $byteSize ), $wordCount );
			global $wgStylePath, $wgDefaultSkin;
			$path = $wgStylePath . '/' . $wgDefaultSkin . '/PageIndex/images';
			switch ( $t->getNamespace() ) {
				case NS_MAIN:
					$icon = "{$path}/page_" . ( $this->gray ? 'gray' : 'white' ) . '.png'; break;
				case NS_CATEGORY:
					$icon = "{$path}/folder_" . ( $this->gray ? 'gray' : 'white' ) . '.png'; break;
				case NS_USER:
				    $icon = "{$path}/user_" . ( $this->gray ? 'gray' : 'white' ) . '.png'; break;
				default:
					$icon = '';
			}
		} else {
			$img = wfFindFile( $t );
			if ( $img ) {
				$info = $img->getShortDesc();
				$icon = Attachments::getIcon( $img );
			}
		}
			
		$html =
			Xml::openElement( 'tr', $this->gray ? array( 'class' => 'gray' ) : null ) . "\n" .
			Xml::tags( 'td', array( 'style' => "background: url({$icon}) no-repeat 1px 2px; padding-left: 30px;" ), $name ) . "\n" .
			Xml::element( 'td', array( 'style' => 'width: 200px;' ), "{$score}{$info}" ) . "\n" .
			Xml::closeElement( 'tr' ) . "\n";
			
		$this->gray = $this->gray ? false : true;

		return $html;
	}
	
	function powerSearchBox( $term ) {
		global $wgScript;

		$namespaces = array(
			NS_MAIN => wfMsg( 'namespace-pages' ),
			NS_USER => wfMsg( 'namespace-userprofiles' ),
			NS_CATEGORY => wfMsg( 'namespace-folders' ),
			NS_IMAGE => wfMsg( 'namespace-attachments' )
		);
		$checkboxes = '';
		foreach( $namespaces as $ns => $name ) {
			$name = str_replace( '_', ' ', $name );
			if( '' == $name ) {
				$name = wfMsg( 'blanknamespace' );
			}
			$checkboxes .= Xml::openElement( 'span', array( 'style' => 'white-space: nowrap' ) ) .
					Xml::checkLabel( $name, "ns{$ns}", "mw-search-ns{$ns}", in_array( $ns, $this->namespaces ) ) .
					Xml::closeElement( 'span' ) . "\n";
		}

		$searchField = Xml::input( 'search', 50, $term, array( 'type' => 'text', 'id' => 'powerSearchText' ) );
		$searchButton = Xml::submitButton( wfMsg( 'powersearch' ), array( 'name' => 'fulltext' ) ) . "\n";
		$searchTitle = SpecialPage::getTitleFor( 'Search' );
		
		$out = Xml::openElement( 'form', array(	'id' => 'powersearch', 'method' => 'get', 'action' => $wgScript ) ) .
			Xml::fieldset( wfMsg( 'powersearch-legend' ),
				Xml::hidden( 'title', $searchTitle->getPrefixedText() ) .
				"<p>" .
				$checkboxes .
				"</p>" .
				wfMsgExt( 'powersearch-field', array( 'parseinline' ) ) .
				"&nbsp;" .
				$searchField .
				"&nbsp;" .
				$searchButton ) .
			"</form>";

		return $out;
	}
}