<?php

if ( !defined( 'MEDIAWIKI' ) ) {
	// Eclipse helper - will be ignored in production
	require_once( "ApiBase.php" );
}


/**
 * Requires API write mode to be enabled.
 *
 * @ingroup API
 */
class ApiUpload extends ApiBase {
	public function __construct( $main, $action ) {
		parent::__construct( $main, $action );
	}

	public function execute() {
		$params = $this->extractRequestParams();
		
		$backend = new ApiUploadBackend( $params['file'], $params['destid'], $params['destname'] );
		
		if ( true === $backend->result ) {
			$name = $backend->mDestName;
			$title = Title::newFromText( $name, NS_IMAGE );
			$image = wfFindFile( $title );
			
			$r = array(
				'result' => 'success',
				'info' => array(
					'id' => $title->getArticleID(),
					'name' => $name,
					'path' => $image->getPath()
				)
			);
		} else {
			$r = array(
				'result' => 'failure',
				'error' => array(
					'message' => urldecode( $backend->result )
				)
			);
		}
		
		$this->getResult()->addValue( null, $this->getModuleName(), $r );
	}
	
	/**
	 * Using the settings determine the value for the given parameter
	 *
	 * @param $paramName String: parameter name
	 * @param $paramSettings Mixed: default value or an array of settings using PARAM_* constants.
	 * @param $parseMaxLimit Boolean: parse limit when max is given?
	 */
	protected function getParameterFromSettings( $paramName, $paramSettings, $parseMaxLimit ) {
		try {
			return parent::getParameterFromSettings( $paramName, $paramSettings, $parseMaxLimit );
		} catch ( MWException $e ) {
			$encParamName = $this->encodeParamName( $paramName );
			
			if ( array_key_exists( $encParamName, $_FILES ) ) {
				return $_FILES[$encParamName];
			} else {
				$type = gettype( $paramSettings );
				
				ApiBase :: dieDebug( __METHOD__, "Param {$encParamName}'s type is unknown - {$type}" );
			}
		}
	}
	
	public function mustBePosted() { return true; }

	public function getAllowedParams() {
		return array (
			'file' => array( 'file', false, 'file', false ),
			'destid' => null,
			'destname' => null
		);
	}

	public function getParamDescription() {
		return array (
			'file' => 'Uploaded file',
			'destid' => 'Destination page ID',
			'destname' => 'Destination file name'
		);
	}

	public function getDescription() {
		return array(
			'Upload API.'
		);
	}

	protected function getExamples() {
		return array (
			'api.php?action=upload',
			'api.php?action=upload&destid=1',
			'api.php?action=upload&destid=1&destname=filename'
		);
	}

	public function getVersion() {
		return __CLASS__ . ': $Id: ApiUpload.php pszreder $';
	}
}

class ApiUploadBackend extends UploadForm {
	public $result;
	
	/**
	 * Constructor : initialise object
	 *
	 * @param array $file
	 * @param int $destid
	 * @param string $destname
	 */
	public function __construct( array &$file, $destid, $destname ) {
		global $wgTitle;
		
		$_FILES['wpUploadFile'] = $file;
		
		$WebRequest = new WebRequest();
		
		if ( !empty( $destname ) ) {
			$WebRequest->setVal( 'wpDestFile', $destname );
		} else {		
			$WebRequest->setVal( 'wpDestFile', $file['name'] );
		}
		$WebRequest->setVal( 'action', 'submit' );
		$WebRequest->setVal( 'wpIgnoreWarning', '1' );
		$WebRequest->setVal( 'wpUploadFile', $file );
		if ( !empty( $destid ) ) {
			$WebRequest->setVal( 'wpArticleId', $destid );
		}
		
		$wgTitle = new Title();
		
		parent::__construct( $WebRequest );
		
		$this->result = $this->processUpload();
	}
	
	/**
	 * Do the upload
	 *
	 * @return mixed
	 */
	public function processUpload() {
		global $wgUser, $wgOut, $wgFileExtensions, $wgLang, $wgRequest;
	 	$details = null;
	 	$value = null;
	 	$value = $this->internalProcessUpload( $details );

	 	switch ( $value ) {
			case self::SUCCESS:
				return true;
				break;
				
			case self::BEFORE_PROCESSING:
				break;
				
			case self::LARGE_FILE_SERVER:
				return wfMsg( 'largefileserver' );
				break;
				
			case self::EMPTY_FILE:
				return wfMsg( 'emptyfile' );
				break;
				
			case self::MIN_LENGHT_PARTNAME:
				return wfMsg( 'minlength1' );
				break;
				
			case self::ILLEGAL_FILENAME:
				return wfMsg( 'illegalfilename', htmlspecialchars( $details['filtered'] ) );
				break;
				
			case self::OVERWRITE_EXISTING_FILE:
				return $details['overwrite'];
				break;
				
			case self::FILETYPE_MISSING:
				return wfMsg( 'filetype-missing' );
				break;
				
			case self::FILETYPE_BADTYPE:
				return wfMsg(
					'filetype-banned-type',
					htmlspecialchars( $details['finalExt'] ),
					implode( ', ', $wgFileExtensions ),
					count($wgFileExtensions)			
				);				
				break;
				
			case self::VERIFICATION_ERROR:
				return $details['veri']->toString();
				break;
				
			case self::UPLOAD_VERIFICATION_ERROR:
				return $details['error'];
				break;
				
			case self::UPLOAD_WARNING:
				return $details['warning'];
				break;

			case self::INTERNAL_ERROR:
				return $details['internal'];
				break;

			default:
				throw new MWException( __METHOD__ . ": Unknown value `{$value}`" );
	 	}
	}	
}