<?php

/**
 * MediaWiki Breadcrumbs extension
 */

if ( ! defined( 'MEDIAWIKI' ) )
	die( 1 );

$wgExtensionCredits['parserhook'][] = array(
	'name' => 'Breadcrumbs',
	'author' => 'Paweł Szreder',
	'version' => '0.1',
	'description' => 'Adds breadcrumbs bar',
	'descriptionmsg' => 'breadcrumbs_desc',
	'url' => 'http://www.bizneswiki.pl'
);

global $wgExtensionFunctions, $wgHooks;
	
require_once( 'Breadcrumbs.i18n.php' );

$wgExtensionFunctions[] = 'wfBreadcrumbs';
$wgHooks['LanguageGetMagic'][] = 'wfBreadcrumbsMagic';
$wgHooks['BeforePageDisplay'][] = 'Breadcrumbs::onBeforePageDisplay';

function wfBreadcrumbs() {
	global $wgParser, $wgMessageCache, $wgBreadcrumbsMessages;
	
	$wgParser->setHook( 'Breadcrumbs', 'Breadcrumbs' );
	
	$wgParser->setFunctionHook( 'breadcrumbs', 'wfBreadcrumbsRender' );
	
	foreach ( $wgBreadcrumbsMessages as $lang => $messages) {
		$wgMessageCache->addmessages( $messages, $lang );
	}
}
 
function wfBreadcrumbsMagic( &$magicWords, $langCode ) {
	$magicWords['breadcrumbs'] = array( 0, 'breadcrumbs' );

	return true;
}
 
function wfBreadcrumbsRender( &$parser ) {
	return array( Breadcrumbs::getHTML(), 'isHTML' => true );
}

class Breadcrumbs {
	private $title;
	private $data;
	
	static $customData;
	
	public function __construct( $title = null ) {
		global $wgTitle;

		$this->title = ( $title instanceof Title ) ? $title : $wgTitle;

		if ( self::isVisible() && ( null === self::$customData ) ) {
		    $this->data = $this->getData();
		}
	}
	
	public static function onBeforePageDisplay( &$out ) {
		global $wgStylePath, $wgDefaultSkin;
		
		if ( self::isVisible() ) {
			$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/Breadcrumbs.css" );
		}

		return true;
	}
	
	public static function isVisible() {
		global $wgRequest, $wgTitle;
		
		if ( null !== self::$customData ) return true;
		
		if ( !in_array( $wgTitle->getNamespace(), array( NS_MAIN, NS_CATEGORY ) ) ) return false;
		if ( !in_array( $wgRequest->getVal( 'action' ), array( null, 'edit') ) ) return false;
		if ( $wgTitle->getArticleID() == Title::newMainPage()->getArticleID() ) return false;
		
		return true;
	}
		
	private function getData( &$breadcrumbs = array(), $title = null ) {
		global $wgTitle, $wgRequest;
		
		$category = $wgRequest->getVal( 'category' );
		
		if ( is_null( $title ) ) {
		    $title = $this->title;
		    
		    $title = ( $category ) ? Title::newFromID( $category ) : $title;
		}
		
		if ( !($title instanceof Title) ) return;
		
		if ( !$title->exists() ) return;
		
		if ( in_array( $title->getNamespace(), array( NS_MAIN, NS_CATEGORY ) ) ) $breadcrumbs[] = $title;
		
		$dbr =& wfGetDB( DB_SLAVE );
		
		$row = $dbr->selectRow(
    		'categorylinks',
    		array( "{$dbr->addQuotes( NS_CATEGORY )} AS page_namespace", 'cl_to AS page_title' ),
    		array( 'cl_from' => $title->getArticleID(), 'cl_to_namespace' => NS_CATEGORY ),
    		__METHOD__
    	);
    	
    	if ( false !== $row ) {
    		$title = Title::newFromRow( $row );
    		if ( $title->exists() ) {
	    		$this->getData( $breadcrumbs, $title );
    		}
    	} else {
    		$breadcrumbs = array_reverse( $breadcrumbs );
    	}
    	
		return $breadcrumbs;			
	}
	
	public function __toString() {
		if ( !self::isVisible() ) return '';
		
		if ( null === self::$customData ) {
		global $wgUser;
		
		$skin = $wgUser->getSkin();
		
		$crumbs = array();
		$crumbs[] = $skin->link( SpecialPage::getTitleFor( 'PageIndex' ), wfMsg( 'folders' ) );
		if ( !empty( $this->data ) ) {
			foreach ( $this->data as $title ) {
    			$anchor = $title->getBaseText();
    			$anchor = ( mb_strlen( $anchor ) > 45 )
    			    ? mb_substr( $anchor, 0, 44 ) . "&hellip;"
    			    : $anchor;
				$crumbs[] = ( $title->equals( $this->title ) )
					? $anchor
					: $skin->link( $title, $anchor ); 
			}
		}
		} else {
		    $crumbs = self::$customData;
		}
		
		return
			'<span class="breadcrumbs">' .
			implode( "&nbsp;&gt;&nbsp;", $crumbs ) .
			'</span>';		
	}
}
