<?php

class BusinessWikiPermissions_Restrictions {
    static private $_instances = array();
    
    const group = 0;
    const user = 1;
    
    private $id;
    
    /**
     * @var BusinessWikiPermissions_Title
     */
    private $title;
    
    private $ns;
    private $restrictions = null;
    
    /**
     * @param Title $title
     * @param int $id
     * @return BusinessWikiPermissions_Restrictions
     */
    public function __construct( Title &$title, $id = null ) {
        if ( is_null( $id ) ) {
            $id = $title->getArticleID();
        }
        
        $this->id = $id;
        $this->title = BusinessWiki_Title::newFromID( $id );
        $this->ns = $this->title->getNamespace(); 
    }
    
    /**
     * @param int $id
     * @return BusinessWikiPermissions_Restrictions
     */
    public static function &getInstanceFromId( &$id ) {
        if ( empty( self::$_instances[$id] ) ) {
            self::$_instances[$id] = new self( new Title(), $id );
        }
        
        return self::$_instances[$id];
    }
    
    /**
     * @param Title|BusinessWiki_Title $title
     * @return BusinessWikiPermissions_Restrictions
     */
    public static function &getInstanceFromTitle( &$title ) {
        return self::getInstanceFromId( $title->getArticleID() );
    }
    
    private function load() {
        if ( 0 === $this->id ) {
            $this->restrictions = array();
            
            return;
        }
        
        if ( $this->ns != NS_IMAGE ) {
            $id = $this->id;
        } else {
    	    $dbr =& wfGetDB( DB_SLAVE );
    		$id = (int) $dbr->selectField(
    			'file',
    			'file_from',
    			array('file_name' => $this->title->getDBkey()),
    		    __METHOD__
    		);
    		
    		if ( !is_numeric( $id ) || !( $id > 0 ) ) {
                $this->restrictions = array();
                
                return;
    		}
        }
        
        $dbr =& wfGetDB( DB_SLAVE );
        
        $res = $dbr->select(
            'restrictions',
            array( 'r_action', 'r_group', 'r_user' ),
            array( 'r_page' => $id ),
            __METHOD__
        );
        
        $this->restrictions = array();
        
        global $wgBusinessWikiActions,$wgBusinessWikiUserCanDefault;
        foreach ( $wgBusinessWikiActions as $action ){
        	if ( true === $wgBusinessWikiUserCanDefault[$action] )
        		$this->restrictions[$action] = array( self::user => array(-1) );
        	else
        		$this->restrictions[$action] = array( self::user => array() );
        } 
        
        foreach ( $res as $row ) {
            if ( empty( $row->r_user) ) {
                if ( is_null( $this->restrictions[$row->r_action][self::group] ) ) {
                    $this->restrictions[$row->r_action][self::group] = array();
                }
                
                $this->restrictions[$row->r_action][self::group][] = $row->r_group;
            } else {
                if ( is_null( $this->restrictions[$row->r_action][self::user] ) ) {
                    $this->restrictions[$row->r_action][self::user] = array();
                }
                
                $this->restrictions[$row->r_action][self::user][] = $row->r_user;
            }
        }
    }
    
    public function restrictions( $params = null ) {
        global $wgBusinessWikiActions;
        
        if ( is_null( $params ) ) {
            if ( !is_array( $this->restrictions ) ) {
                $this->load();
            }
            
            if ( empty( $this->restrictions ) ) {
                global $wgBusinessWikiUserCanDefault;
                
                foreach ( $wgBusinessWikiActions as $action ) {
                    if ( true === $wgBusinessWikiUserCanDefault[$action] ) {
                        $this->restrictions[$action][self::user] = array( -1 );
                    }
                    
                    $this->restrictions[$action][self::group] = array();
                }
            }
            
            return $this->restrictions;
        }
        
        $dbw =& wfGetDB( DB_MASTER );
        
        $dbw->begin();
        $dbw->delete( 'restrictions', array( 'r_page' => $this->id ), __METHOD__ );
        $a = $b = array();
        foreach ( $params as $action => $array ) {
            if ( !in_array( $action, $wgBusinessWikiActions ) ) {
                continue;
            }
            foreach ( $array as $userId ) {
                if (strpos($userId, 'g-') !== false) {
                    $a[] = array(
                    	'r_page' => $this->id,
                    	'r_action' => $action,
                    	'r_group' => str_replace('g-', '', $userId)
                    );
                } else {
                    $b[] = array(
                    	'r_page' => $this->id,
                    	'r_action' => $action,
                    	'r_user' => $userId
                    );
                }
            }
        }
        if (!empty($a)) {
            $dbw->insert( 'restrictions', $a, __METHOD__ );
        }
        if (!empty($b)) {
            $dbw->insert( 'restrictions', $b, __METHOD__ );
        }
        $dbw->commit();
        
        if ( NS_CATEGORY == $this->ns ) {
            $dbr =& wfGetDB( DB_SLAVE );

	    	$res = $dbr->select(
	    		array( 'page', 'categorylinks' ),
	    		array( 'page_id' ),
	    		array(
	    			'page_namespace' => array( NS_MAIN, NS_CATEGORY ),
	    			'cl_to' => $this->title->getDBkey(), 'cl_to_namespace' => NS_CATEGORY
	    		),
	    		__METHOD__,
                null,
	    		array(
	    			'categorylinks' => array( 'LEFT JOIN', 'cl_from = page_id' )
	    		)
	    	);
	    	
	    	while ( $row = $dbr->fetchObject( $res ) ) {
                $r =& BusinessWikiPermissions_Restrictions::getInstanceFromId( $row->page_id );
                
                $r->restrictions( $params );
	    	}
        }
        
        return true;
    }
    
    /**
     * @param int $action
     * @param int $userId
     * @return bool
     */
    public function validate( $action, $userId ) {
        if ( $this->title instanceof BusinessWiki_Title ) {
            if ( $this->title->isFirstContributor( User::newFromId( $userId ) ) ) {
                return true;
            }
        }
        
        $r =& $this->restrictions();
        
        if ( is_array( $r[$action] ) ) {
            if ( !empty($r[$action][self::group]) ) {
                $dbr =& wfGetDB( DB_SLAVE );
                
                $count = $dbr->selectField(
                	'usergroups',
                    'COUNT(user_id)',
                    array('user_id' => $userId, 'user_group_id' => $r[$action][self::group]),
                    __METHOD__
                );
                if ( $count > 0 ) {
                    return true;
                }
            }
            
            if (
                in_array( -1, $r[$action][self::user] )
                || in_array( $userId, $r[$action][self::user] )
            ) {
                return true;
            } /* elseif ( count( array_intersect( // TODO: implement
                $r[$action][self::group],
                Fake::userGetGroups( $userId )
            ) ) > 0 ) {
                return true;
            } */ else {
                return false;
            }  
            
        } else {
            return false;
        }
    }
}
