<?php

class BusinessWikiPermissions_Title extends Title {
    public $mGroupsRestrictions = array();
    public $mUsersRestrictions = array();
	/**
	 * @param string $key
	 * @return BusinessWikiPermissions_Title
	 */
	public static function newFromDBkey( $key ) {
		$t = new self();
		$t->mDbkeyform = $key;

		return $t;
	}
	    
    /**
     * @param Resource $res
     */
    private function loadRestrictionsFromRow( $res ) {
		global $wgRestrictionTypes;
		
		$dbr =& wfGetDB( DB_SLAVE );

		foreach ( $wgRestrictionTypes as $type ){
			$this->mRestrictions[$type] = array();
			$this->mRestrictionsExpiry[$type] = Block::decodeExpiry( '' );
		}

		$this->mCascadeRestriction = false;

		if ( $dbr->numRows( $res ) ) {
			$now = wfTimestampNow();
			$purgeExpired = false;

			foreach( $res as $row ) {
				if ( !in_array( $row->pr_type, $wgRestrictionTypes ) ) {
					continue;
				}

				$expiry = Block::decodeExpiry( $row->pr_expiry );

				if ( !$expiry || $expiry > $now ) {
					$this->mRestrictionsExpiry[$row->pr_type] = $expiry;
					$this->mRestrictions[$row->pr_type] = explode( ',', trim( $row->pr_level ) );

					$this->mCascadeRestriction |= $row->pr_cascade;
				} else {
					$purgeExpired = true;
				}
			}

			if ( $purgeExpired ) {
				Title::purgeExpiredRestrictions();
			}
		}

		$this->mRestrictionsLoaded = true;
	}
	
	public function loadRestrictions() {
		if ( !$this->mRestrictionsLoaded ) {
			if ( $this->exists() ) {
				$dbr =& wfGetDB( DB_SLAVE );

				$res = $dbr->select(
					'page_restrictions',
					'*',
					array( 'pr_page' => $this->getArticleId() ),
					__METHOD__
			    );

				$this->loadRestrictionsFromRow( $res );
			} else {
				$title_protection = $this->getTitleProtection();

				if ( is_array( $title_protection ) ) {
					extract($title_protection );

					$now = wfTimestampNow();
					$expiry = Block::decodeExpiry( $pt_expiry );

					if ( !$expiry || $expiry > $now ) {
						$this->mRestrictionsExpiry['create'] = $expiry;
						$this->mRestrictions['create'] = explode( ',', trim( $pt_create_perm ) );
					} else {
						Title::purgeExpiredRestrictions();
					}
				} else {
					$this->mRestrictionsExpiry['create'] = Block::decodeExpiry( '' );
				}
				
				$this->mRestrictionsLoaded = true;
			}
		}
	}
	
	private function getTitleProtection() {
		if ( $this->getNamespace() < 0 ) {
			return false;
		}

		$dbr =& wfGetDB( DB_SLAVE );
		$res = $dbr->select( 'protected_titles', '*',
			array( 'pt_namespace' => $this->getNamespace(), 'pt_title' => $this->getDBkey() ),
			__METHOD__ );

		if ( $row = $dbr->fetchRow( $res ) ) {
			return $row;
		} else {
			return false;
		}
	}	
}