<?php

class BusinessWikiPermissions {
    static private $_instance;
    
    private $defaultRead;
    
    public function __construct( $args = array() ) {
        global $wgGroupPermissions, $wgRestrictionTypes;
        
        if ( $this->defaultRead = (
            isset( $wgGroupPermissions['*']['read'] )
            && $wgGroupPermissions['*']['read']
        ) ) {
			$wgGroupPermissions['*']['read'] = false;
        }
        
        $wgRestrictionTypes[] = 'read';
    }
    
    /**
     * @param array $args
     * @return unknown_type
     */
    public static function &getInstance( $args = array() ) {
        if ( !self::$_instance ) {
            self::$_instance = new self( $args );
        }
        
        return self::$_instance;
    }
    
    /**
     * @param Title $title
     * @param User $user
     * @param string $action
     * @param bool $result
     * @return bool
     */
    public function onuserCan( Title &$title, User &$user, $action, &$result ) {
        global $wgBusinessWikiActions;
        
        static $internalCache;
        
        if (
            !in_array( $action, $wgBusinessWikiActions )
            || !$title->exists()
            || $title->isCssJsSubpage()
            || !$user->isLoggedIn()
            || !in_array( $title->getNamespace(), array( NS_CATEGORY, NS_MAIN, NS_IMAGE ) )
        ) {
            return true;
        }
        
        $bTitle = BusinessWiki_Title::newFromID( $title->getArticleID() );
        
        if (
            in_array( 'sysop', $user->getGroups() )
            || $bTitle->isFirstContributor( $user )
        ) {
            $result = true;
            return $result;
        }
        
        $r =& BusinessWikiPermissions_Restrictions::getInstanceFromTitle( $title );
        
        if ( $action !== 'delete' ) {
            $result = $r->validate( $action, $user->getId() );
        } else {
            $result = $r->validate( $action, $user->getId() );
            $result = $result && $r->validate( 'edit', $user->getId() );
            $result = $result && $r->validate( 'move', $user->getId() );
        }
        
        return $result;
    }
    
    /**
     * @param User $user
     * @param array $aRights
     * @return bool
     */
    public function onUserGetRights( User &$user, &$aRights ) {
		global $wgGroupPermissions, $wgTitle, $wgRequest;
		
		if ( !is_object( $wgTitle ) ) {
		    return true;
		}
		
		$title = $wgTitle;
		if ( NS_SPECIAL == $title->getNamespace() ) {
			list( $name, $par ) = explode( '/', $title->getDBkey() . '/', 2 );
			if ( $par ) {
			    $title = Title::newFromText( $par );
			} elseif ( $par = $wgRequest->getVal( 'target' ) ) {
			    $title = Title::newFromText( $par );
			} elseif ( $par = $wgRequest->getVal( 'oldtitle' ) ) {
			    $title = Title::newFromText( $par );
			}
		}
		
		if ( !is_object( $title ) ) {
		    return true;
		}
		
		/* TODO: zdaje sie, ze tutaj wystarczy bazowy Title, jeszcze nie wiem */
		$title = BusinessWikiPermissions_Title::newFromDBkey( $title->getDBkey() );

		$groups = $user->getEffectiveGroups();

		if ( $this->defaultRead ) {
			$wgGroupPermissions['*']['read'] = true;
			$aRights[] = 'read';
		}

		if ( !$title->mRestrictionsLoaded ) {
		    $title->loadRestrictions();
		}
		foreach ( $title->mRestrictions as $a => $g ) {
    		if ( !empty( $g ) ) {
    			if ( array_intersect( $groups, $g ) ) {
    			    $aRights[] = $a;
    			}
    		}
		}

		if (
		    !in_array( 'sysop', $groups )
		    && !$this->userCan( $title, $user, 'read', $error )
		) {
			foreach ( $aRights as $i => $right ) {
			    if ( $right === 'read' || $right === 'move' ) {
			        unset( $aRights[$i] );
			    }
			}
			
		}

		return true;        
    }
    
    /**
     * @param OutputPage $output
     * @param Article $article
     * @param Title $title
     * @param User $user
     * @param WebRequest $request
     * @param MediaWiki $wiki
     * @return bool
     */
    public function onMediaWikiPerformAction(
        OutputPage $output, Article $article, Title $title,
        User $user, WebRequest $request, MediaWiki $wiki
    ) {
        if ( 'raw' === $wiki->getVal( 'Action' ) ) {
            $wiki->setVal( 'Action', 'view' );
        }
        
        return true;
    }

	/**
	 * @param Title $title
	 * @param User $user
	 * @param string $action
	 * @param string $error
	 * @return bool
	 */
	private function userCan( &$title, User &$user, $action, $error ) {
		static $internalCache;
		
	    $groups = $user->getEffectiveGroups();
		
		if ( !is_object( $title ) || in_array( 'sysop', $groups ) ) {
		    return true;
		}

		$cacheKey = $this->cacheKey( $user, $action, $title );
		if ( !empty( $internalCache[$cacheKey] ) ) {
			$error = $internalCache[$cacheKey][1];
			return $internalCache[$cacheKey][0];
		}
		
		$readable = true;

		$whiteList = $title->getRestrictions( $action );
		if (
		    !empty( $whiteList )
		    && count( array_intersect( $whiteList, $groups ) == 0 )
		) {
		    $readable = false;
		}

		$error = $readable ? '' : wfMsg( 'badaccess-read', $title->getPrefixedText() );
		$internalCache[$cacheKey] = array( $readable, $error );
		
		return $readable;
	}
	
	/**
	 * @param User $user
	 * @param string $action
	 * @param Title $title
	 * @return string
	 */
	private function cacheKey( User &$user, &$action, Title &$title ) {
	    return $user->getID() . '\x07' . $action . '\x07' . $title->getPrefixedText();
	}
}

/*
class BusinessWikiPermissions_DB {
    static private $_initialized = false;
    
    public static function initialize() {
        if ( self::$_initialized || !class_exists( 'Database' ) ) {
            return false;
        }
        
	    global $wgDBtype;
	
	    $wgOldDBtype = $wgDBtype;
	    $oldClass = ucfirst( $wgDBtype );
	    $wgDBtype = 'BusinessWikiPermissions';
	
	    eval("class Database{$wgDBtype} extends Database{$oldClass}".' {
    		public function query( $sql, $fname = "", $tempIgnore = false ) {
    			$count = false;
    			if (is_object($wgSimpleSecurity))
    				$patched = preg_replace_callback("/(?<=SELECT ).+?(?= FROM)/", array("SimpleSecurity", "patchSQL"), $sql, 1, $count);
    			return parent::query($count ? $patched : $sql, $fname, $tempIgnore);
    		}
    		
    		function fetchObject( &$res ) {
    			$row = parent::fetchObject($res);
    			if (is_object($wgSimpleSecurity) && isset( $row->old_text) ) {
    				$wgSimpleSecurity->validateRow($row);
    			}
    			
    			return $row;
    		}
    	}');
	    
		if (function_exists('wfGetLBFactory')) wfGetLBFactory()->forEachLB(array('SimpleSecurity', 'updateLB'));
		elseif (is_object($wgLoadBalancer)) SimpleSecurity::updateLB($wgLoadBalancer);
		else die("Can't hook in to Database class!");
		
	wfGetDB( DB_MASTER );
	$wgDBtype = $wgOldDBtype;		
			    
	    self::$_initialized = true;
    }
}
*/

class BusinessWikiPermissions_Adapter extends ProtectionForm {
    public function __construct() {
        // $this->save();
    }
}