<?php
/**
 * MediaWiki Comments extension
 */

if( !defined( 'MEDIAWIKI' ) ) {
    echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
    die( 1 );
}
 
$wgExtensionCredits['other'][] = array(
	'name' => 'Comments',
	'author' =>' Paweł Szreder',
	'version' => '0.1',
	'description' => 'Adds comments support',
	'url' => 'http://www.bizneswiki.pl'
);

$wgExtensionFunctions[] = 'wfComments';

$wgHooks['BeforePageDisplay'][] = 'Comments::onBeforePageDisplay';
$wgHooks['UnknownAction'][] = 'Comments::onUnknownAction';
$wgHooks['ArticleDeleteComplete'][] = 'Comments::onArticleDeleteComplete';
$wgHooks['LanguageGetSpecialPageAliases'][] = 'wfCommentsLocalizedSpecialPageName';
$wgHooks['ArticlePageDataBefore'][] = 'fnAddJSLanguageVars';

$dir = dirname(__FILE__) . '/';
require_once $dir . 'Comments.i18n.php';

function wfComments() {
	global $wgParser, $wgWikiCommentsMessages, $wgMessageCache;

	foreach ( $wgWikiCommentsMessages as $lang => $messages) {
		$wgMessageCache->addMessages( $messages, $lang );
	}
}

function fnAddJSLanguageVars(){
	global $wgOut;
	$wgOut->addScript( ''. Xml::openElement( 'script', array( 'type' => 'text/javascript' ) ) . '/*<![CDATA[*/ ' .
			'var textSaving = "'.wfMsg( 'sending' ).'"; var textPost = "'.wfMsg( 'add_comment' ).'";' .
			'/*]]>*/' . Xml::closeElement( 'script' ) . '' );
	return true;
}

function wfCommentsLocalizedSpecialPageName( &$specialPageArray, $code ) {
	$text = wfMsg( 'comments' );
	$title = Title::newFromText($text);
	$specialPageArray['Comments'][] = $title->getDBKey();

	return true;
}

class Comment {
	public $id;
	public $parentId;
	public $user;
	public $timestamp;
	public $text;
	
	public $subcomments = array();
	
	private function __construct() {  }
	
	public static function newFromRow( $row ) {
		$c = new Comment();
		
		$c->id = isset( $row->comment_id ) ? intval( $row->comment_id ) : -1;
		$c->parentId = isset( $row->comment_parent_id ) ? intval( $row->comment_parent_id ) : null;
		$c->user = User::newFromId( intval( $row->comment_user_id ) );
		$c->timestamp = wfTimestamp( TS_DB, $row->comment_timestamp );
		$c->text = $row->comment_text;
		
		return $c;
	}
	
	public static function newFromId( $id ) {
		$db =& wfGetDB( DB_SLAVE );
		
		$row = $db->selectRow(
			'comments',
			array(
				'comment_id', 'comment_parent_id', 'comment_user_id',
				'comment_timestamp', 'comment_text'
			),
			array( 'comment_id' => $id ),
			__METHOD__);
		
		return Comment::newFromRow( $row );			
	}
	
	public function isParent() {
		$db =& wfGetDB( DB_SLAVE );
		
		$count = $db->selectField(
    		'comments',
    		'COUNT(comment_id)',
			array( 'comment_parent_id' => $this->id ),
    		__METHOD__		
		);
		
		return ( $count > 0 );
	}
	
	public function userIsAuthor() {
		global $wgUser;
		
		return ( $this->user->getId() == $wgUser->getId() );
	}
}

class Comments {
    static $initialized = false;
    
	private $comments = array();
	private $flatComments = array();
	
	private $db;
	
	public function __construct( Article $article, $fetch = true ) {
	    self::$initialized = true;
	    
		if ( $fetch ) $this->getComments();
	}
	
	public static function onBeforePageDisplay( &$out ) {
		if ( self::$initialized ) {
    	    global $wgStylePath, $wgDefaultSkin;
    		
    		$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/Comments.css" );
    		// $out->addScriptFile( 'jquery.jeditable.js' );
    		$out->addScriptFile( "/extensions/Comments/Comments.js" );
		}

		return true;
	}
	
	public static function onUnknownAction( $action, $article ) {
		global $wgOut, $wgRequest;
		
		if ( $action == 'comment') {
			$wgOut->disable();
			wfResetOutputBuffers();
			
			list( $id ) = sscanf( $wgRequest->getText( 'comment_id' ), 'comment_%d' );
			list( $parentId ) = sscanf( $wgRequest->getText( 'comment_parent_id' ), 'comment_parent_%d' );
			$text = $wgRequest->getText( 'comment_text' );
			
			switch ( $wgRequest->getText( 'method' ) ) {
				case 'get':
					$comments = new Comments( $article, false );
					$comments->getComments( $id );
					echo $comments->get();
					break;
				case 'add':
					$comments = new Comments( $article );
					echo $comments->add( $parentId, $text );
					break;
				case 'edit':
					$comments = new Comments( $article, false );
					echo $comments->update( $id, $text );
					break;
				case 'delete':
					$comments = new Comments( $article, false );
					echo $comments->delete( $id );
					break;
			}
		}
		
		return true;
	}
	
	public static function onArticleDeleteComplete( Article &$article ) {
		$dbw =& wfGetDB( DB_MASTER );

		$dbw->delete( 'comments', array( 'comment_page_id' => $article->getID() ) );
		
		return true;
	}
	
	private function getComments( $parentId = null ) {
		global $wgArticle;
		
		$dbr =& wfGetDB( DB_SLAVE );
		
		$res = $dbr->select(
			'comments',
			array(
				'comment_id', 'comment_parent_id', 'comment_user_id',
				'comment_timestamp', 'comment_text'
			),
			array(
				'comment_page_id' => $wgArticle->getID(),
				'comment_parent_id' => $parentId
			),
			__METHOD__,
			array( 'ORDER BY' => 'comment_timestamp' )
		);
		
		while ( $row = $dbr->fetchObject( $res ) ) {
			array_push( $this->comments, Comment::newFromRow( $row ) );
		}
	}
	
	private function getFlatComments( Comment $comment ) {
		$dbr =& wfGetDB( DB_SLAVE );
		
		$res = $dbr->select(
			'comments',
			array(
				'comment_id', 'comment_parent_id', 'comment_user_id',
				'comment_timestamp', 'comment_text'
			),
			array( 'comment_parent_id' => $comment->id ),
			__METHOD__
		);
		
		while ( $row = $dbr->fetchObject( $res ) ) {
			$child = Comment::newFromRow( $row );
			$this->flatComments[] = $child->id;
			$this->getFlatComments( $child );
		}
		
		return $this->flatComments;
	}
	
	private function getNode( Comment $comment ) {
		global $wgUser;
		
		$out = Xml::openElement( 'div', array( 'class' => 'comment' ) );
		$out .= Xml::openElement( 'span', array( 'class' => 'head' ) );
		
		$out .= Xml::openElement( 'span', array( 'class' => 'author' ) );
		$out .= Xml::element( 'a', array(
			'href' => $comment->user->getUserPage()->getLocalURL()
		), $comment->user->getName() );
		$out .= Xml::escapeTagsOnly( " - {$comment->timestamp}" );
		$out .= Xml::closeElement( 'span' );
		
		$out .= Xml::openElement( 'span', array( 'class' => 'tools' ) );
		if ( $wgUser->isLoggedIn() ) {
			$out .= Xml::element( 'a', array( 
				'id' => "comment_response_{$comment->id}",
				'class' => 'response',
				'rel' => $comment->id
			), '', false );
		}
		if ( $comment->userIsAuthor() ) {
			$out .= Xml::element( 'a', array( 
				'id' => "comment_edit_{$comment->id}",
				'class' => 'edit',
				'rel' => $comment->id
			), '', false );
			if ( !$comment->isParent() ) {
				$out .= Xml::element( 'a', array( 
					'id' => "comment_delete_{$comment->id}",
					'class' => 'delete',
					'rel' => $comment->id
				), '', false );
			}
		}
		$out .= '</span>';

		$out .= '</span>';

		$out .= Xml::tags( 'p', array( 'class' => 'content', 'id' => "comment_{$comment->id}" ), nl2br( $comment->text ) );
		
		$out .= Xml::element( 'a', array(
			'id' => "comment_expand_{$comment->id}",
			'class' => ( $comment->isParent() ) ? 'expand close' : 'expand close hidden',
			'title' => wfMsg( 'comments-expand' ) . ' / ' . wfMsg( 'comments-collapse' )
		), wfMsg( 'comments-expand' ) );
		
		$out .= '</div>';

		$out .= '<div class="subcomments"></div>';		
		
		return $out;
	}
	
	public function getNodes( $out = '', $comments = array() ) {		
		foreach ( $this->comments as $comment ) {
			$comments[] = $comment->id;
			$out .= $this->getNode( $comment );
		}
		
		$out .= Xml::openElement( 'script', array( 'type' => 'text/javascript' ) );
		$out .= 'var comments = ' . json_encode( $comments ) . ';';
		$out .= Xml::closeElement( 'script' );
		
		return $out;
	}
	
	public function get() {
		return $this->getNodes();
	}
	
	public function add( $parentId, $value ) {
		if ( empty( $value ) ) return;
		
		global $wgUser, $wgArticle, $wgTitle;
		
		$dbw =& wfGetDB( DB_MASTER );
	
		$dbw->insert( 'comments', array(
			'comment_parent_id' => ( $parentId == 0 ) ? null : $parentId,
			'comment_page_id' => $wgArticle->getID(),
			'comment_user_id' => $wgUser->getId(),
			'comment_timestamp' => wfTimestamp(),
			'comment_text' => $value
		), __METHOD__ );
		
		$result = $this->getNode( Comment::newFromId( $dbw->insertId() ) );

		$size = strlen( $wgArticle->getContent() );
		$lastRevision = $revisionId = $wgArticle->getRevIdFetched();
		$now = wfTimestampNow();
		
		RecentChange::notifyEdit(
			$now,
			$wgTitle,
			true,
			$wgUser,
			'New comment',
			$lastRevision,
			$now,
			false,
			'',
			$size,
			$size,
			$revisionId
		);
		
		return $result;
	}
	
	public function update( $id, $text ) {
		$dbw =& wfGetDB( DB_SLAVE );
		
		$dbw->update(
			'comments',
			array( 'comment_text' => $text ),
			array( 'comment_id' => $id ),
			__METHOD__
		);
		
		return $text;
	}
	
	public function delete( $id ) {
		$comment = Comment::newFromId( $id );
		
		$comments = $this->getFlatComments( $comment );
		$comments[] = $comment->id;
		
		$dbw =& wfGetDB( DB_SLAVE );
		
		$dbw->delete( 'comments', array( 'comment_id' => $comments ) );
		
		$parentComment = Comment::newFromId( $comment->parentId );
		
		if ( !$parentComment->isParent() ) {
			return (string) $parentComment->id;
		}
		
		return '';
	}

	public function __toString() {
		global $wgUser;
		
		$out = 
			'<div id="mw-page-comments">' .
			'<div class="inner">' .
			'<h2>' .
			Xml::span( wfMsg( 'comments' ) . ' (' . count( $this->comments ) . ')', 'left' );
		
		if ( $wgUser->isLoggedIn() ) {
			$out .=
				'<span class="right">' .
				Xml::element( 'a', array( 'class' => 'response' ), wfMsg( 'add_comment' ), false ) .
				'</span>';
		}
		
		$out .= '</h2>';
		
		$out .= Xml::tags( 'div', array( 'class' => 'comments' ),
			( count( $this->comments ) > 0 ) ? $this->getNodes() : Xml::span( wfMsg( 'no_comments' ), 'nocomments' )
		);

		$out .= Xml::openElement( 'div', array( 'id' => 'comment_dummy', 'class' => 'comment dummy' ) );
		$out .= Xml::element( 'p', array( 'class' => 'msg' ), wfMsg( 'comment_body' ) );
		$out .= Xml::element( 'p', array( 'class' => 'content', 'id' => "comment_0" ), '' );
		$out .= Xml::closeElement( 'div' );
		$out .= Xml::openElement( 'div', array( 'class' => 'visualClear' ) ) . Xml::closeElement( 'div' );
		$out .= Xml::closeElement( 'div' );
		$out .= Xml::closeElement( 'div' );
		
		return $out;
	}
}
