<?php

class EmailNotifications {
	private function getDescription( $ns, $type, $action, $comment ) {
		switch ( $ns ) {
			case NS_MAIN:
				$iconObj = 'page'; break;
			case NS_USER:
				$iconObj = 'user'; break;
			case NS_IMAGE:
				$iconObj = 'file'; break;
			case NS_CATEGORY:
				$iconObj = 'folder'; break;
		}
		if ( defined( 'LIVEBLEND' ) ) {
			switch ( $ns ) {
				case NS_QUESTION:
					$iconObj = 'question'; break;
				case NS_CONCEPT:
					$iconObj = 'concept'; break;
			}
		}
		if ( empty( $iconObj ) ) {
			$iconObj = 'page';
		}
		switch ( $type ) {
			case RC_NEW:
				$iconAction = 'create'; break;
			case RC_EDIT:
				$iconAction = 'edit'; break;
			case RC_MOVE:
				$iconAction = 'move'; break;
			case RC_LOG:
				$iconAction = $action; break;
		}
		
		if ( $comment == 'New comment' ) {
			list( $iconObj, $iconAction ) = array( 'comment', 'create' );
		}
		
		return ucfirst( $iconObj ) . ' ' . ucfirst( $iconAction );
	}    
    
    private function extractRowInfo( $row ) {
		$title = Title::makeTitle( $row->rc_namespace, $row->rc_title );

		$vals = array();
		
		$vals['description'] = $this->getDescription(
		    $row->rc_namespace,
		    intval( $row->rc_type ),
		    $row->rc_log_action,
		    strip_tags( $row->rc_comment )
		);
		
		$vals['title'] = $title;

		$type = intval( $row->rc_type );

		switch ( $type ) {
			case RC_EDIT:
			    $vals['type'] = 'edit'; break;
			case RC_NEW:
			    $vals['type'] = 'new'; break;
			case RC_MOVE:
			    $vals['type'] = 'move'; break;
			case RC_LOG:
			    $vals['type'] = 'log'; break;
			case RC_MOVE_OVER_REDIRECT:
			    $vals['type'] = 'move over redirect'; break;
		    default:
		        $vals['type'] = $type;
		}

		$vals['rcid'] = intval($row->rc_id);

		$vals['user'] = $row->rc_user_text;
		if(!$row->rc_user)
			$vals['anon'] = '';

		if ($row->rc_bot)
			$vals['bot'] = '';
		if ($row->rc_new)
			$vals['new'] = '';
		if ($row->rc_minor)
			$vals['minor'] = '';

		$vals['timestamp'] = wfTimestamp( TS_DB, $row->rc_timestamp );

		$vals['comment'] = strip_tags( $row->rc_comment );
			
		if ($row->rc_type == RC_LOG) {
			$vals['logid'] = $row->rc_logid;
			$vals['logtype'] = $row->rc_log_type;
			$vals['logaction'] = $row->rc_log_action;
		}
		
		return $vals;        
    }    
    
    public function getData() {
		$dbr =& wfGetDB( DB_SLAVE );
        
        $res = $dbr->select(
            'recentchanges',
            array(
            	'rc_timestamp', 'rc_namespace', 'rc_title', 'rc_type',
            	'rc_id', 'rc_comment', 'rc_user_text', 'rc_logid',
            	'rc_log_type', 'rc_log_action'
            ),
			array(
				'DATE(rc_timestamp) = DATE(NOW())',
			    'rc_namespace' => array( NS_MAIN, NS_USER, NS_IMAGE, NS_CATEGORY ),
				'rc_deleted' => 0
			),
			__METHOD__,
			array( 'ORDER BY' => 'rc_timestamp DESC' ),
			array( 'USE INDEX' => 'rc_timestamp' )            
        );
        
        $changes = array();
        
        while ( $row = $dbr->fetchObject( $res ) ) {
            $changes[] = $this->extractRowInfo( $row );
        }
        
        return $changes;
    }
    
    private function getUsers() {
        $dbr =& wfGetDB( DB_SLAVE );

        $res = $dbr->select( 
            'user',
			array( 'user_id', 'user_name', 'user_email', 'user_options' ),
			array( 'user_blocked' => 0 ),
			__METHOD__
		);
		
		return UserArray::newFromResult( $res );
    }
    
    private function getMessage( $params ) {
        global $wgSitename, $wgServer;
        
        extract( $params );
        
        $sitename = $wgSitename;
        $server = $wgServer;
        
		ob_start();
		include( 'EmailNotifications.phtml' );
		$message = ob_get_contents();
		ob_end_clean();	
        
        return $message;
    }
    
    public function send() {
        global $wgOutputEncoding;
        
        $from = new MailAddress(
        	'businesswiki@onbusinesswiki.com',
        	'Businesswiki Notifications'
        );
        
        $params = array();
        $params['date'] = substr( wfTimestamp( TS_DB ), 0, 10 );
        $params['changes'] = $this->getData();
        
        if ( empty( $params['changes'] ) ) {
            return 0;
        }
        
        $userArray = $this->getUsers();
        
        $processed = 0;
        
        foreach ( $userArray as $user ) {
			if (
			    $user->getOption( 'emailnotifications' )
			    && User::isValidEmailAddr( $user->getEmail() )
			) {
                $params['user'] = $user->getName();
			    
			    $message = $this->getMessage( $params );
			    
			    $result = UserMailer::send(
        			new MailAddress( $user->getEmail() ),
        			$from,
        			'Businesswiki Notifications',
        			$message,
        			null,
        			"text/html; charset={$wgOutputEncoding}"
        		);
			    
			    if ( !WikiError::isError( $result ) ) {
			        ++ $processed;
			    }
			}
		}
		
		return $processed;
    }
}

class ApiEmailNotifications extends ApiBase {
	public function __construct( $main, $action ) {
		parent::__construct( $main, $action );
	}

	public function execute() {
		global $wgBotPassword, $wgUser, $wgPasswordSender;
		$params = $this->extractRequestParams();

		if ( $params['password'] !== $wgBotPassword ) {
			$this->dieUsage( 'Permissions error.', 'auth' );
		}
		
		$emailNotifications = new EmailNotifications();
		
		$r = array( 'processed' => $emailNotifications->send() );
		$this->getResult()->addValue( null, $this->getModuleName(), $r );
	}

	public function mustBePosted() { return false; }

	public function getAllowedParams() {
		return array (
			'password' => null
		);
	}

	public function getParamDescription() {
		return array (
			'password' => 'Password'
		);
	}

	public function getDescription() {
		return array(
			'Email Notifications.'
		);
	}

	protected function getExamples() {
		return array (
			'api.php?action=emailnotifications&password=PaSsWoRd123'
		);
	}

	public function getVersion() {
		return __CLASS__ . ': $Id: ApiLiveBlendMailer.php pszreder $';
	}
}
