<?php

/**
 * MediaWiki Favorites extension
 */

if ( ! defined( 'MEDIAWIKI' ) )
	die( 1 );

$wgExtensionCredits['parserhook'][] = array(
	'name' => 'Favorites',
	'author' => 'Paweł Szreder',
	'version'        => '0.1',
	'description' => 'Adds favorites box',
	'descriptionmsg' => 'favorites_desc',
	'url' => 'http://www.bizneswiki.pl'
);

$dir = dirname(__FILE__);
require_once( "{$dir}/Favorites.i18n.php" );

$wgExtensionFunctions[] = 'wfFavorites';
$wgHooks['LanguageGetMagic'][] = 'wfFavoritesMagic';
$wgHooks['BeforePageDisplay'][] = 'Favorites::onBeforePageDisplay';
$wgHooks['WatchArticleComplete'][] = 'Favorites::onWatchOrUnwatchArticleComplete';
$wgHooks['UnwatchArticleComplete'][] = 'Favorites::onWatchOrUnwatchArticleComplete';
$wgHooks['UploadComplete'][] = 'Favorites::onUploadComplete';

function wfFavorites() {
	global $wgParser, $wgFavoritesMessages, $wgMessageCache;
	
	$wgParser->setHook( 'Favorites', 'Favorites' );
	
	$wgParser->setFunctionHook( 'favorites', 'wfFavoritesGetHTML' );
	
	foreach ( $wgFavoritesMessages as $lang => $messages) {
		$wgMessageCache->addMessages( $messages, $lang );
	}	
}
 
function wfFavoritesMagic( &$magicWords, $langCode ) {
	$magicWords['favorites'] = array( 0, 'favorites' );

	return true;
}

function wfFavoritesGetHTML() {
	$mp = new Favorites();
	
	return array( $mp->getDashboardHTML(), 'isHTML' => true );
}
 
class Favorites {
	public function __construct() { }
	
	public static function onBeforePageDisplay( &$out ) {
		global $wgScriptPath, $wgUser;
		
		$url = "{$wgScriptPath}/extensions/" . __CLASS__ . '/';
			
		$out->addExtensionStyle( $url . __CLASS__ . '.css' );

		if ( !$wgUser->isLoggedIn() ) return true;
		
		return true;
	}
	
	public static function onWatchOrUnwatchArticleComplete( User &$user ) {
		global $wgDBname, $wgMemc;
		
		$key = "{$wgDBname}:favorites:" . (string) $user->getId();
		
		$wgMemc->delete( $key );
		
		self::getData( $user );
		
		return true;
	}
	
	public static function onUploadComplete( UploadForm &$image ) {
		global $wgUser;
		
		if ( $image->mWatchthis ) {
			self::onWatchOrUnwatchArticleComplete( $wgUser );
		}
		
		return true;
	}
	
	public static function getData( User $user ) {
		$dbr =& wfGetDB( DB_SLAVE );
		
		$res = $dbr->select(
			array( 'watchlist', 'page', 'recentchanges' ),
			array( 'page_id', 'page_namespace', 'page_title' ),
			array(
				'wl_user' => $user->getId(), 'page_namespace = wl_namespace',
				'page_title = wl_title', 'MOD(wl_namespace, 2) = 0'
			),
			__METHOD__,
			array( 'ORDER BY' => 'rc_timestamp DESC', 'LIMIT' => 5 ),
			array( 'recentchanges' => array( 'LEFT JOIN', 'rc_id = page_latest' ) )
		);
		
		$titleArray = array();
    	while ( $row = $dbr->fetchObject( $res ) ) {
    		array_push( $titleArray, $row );
    	}
		
    	return $titleArray;
	}
	
	public function getHTML() {
		global $wgUser, $wgMemc, $wgDBname;
		
		if ( !$wgUser->isLoggedIn() ) return;
		
		$key = "{$wgDBname}:favorites:" . (string) $wgUser->getId();
		
		$data = $wgMemc->get( $key );
		
		if ( !empty( $data ) ) return $data;
				
		$skin = $wgUser->getSkin();
		
		if ( !$wgUser->isLoggedIn() ) 
			return Xml::span( wfMsg( 'notloggedin' ), null, array( 'style' => 'display: block; text-align: center;' ) );		

		$titleArray = self::getData( $wgUser );
		
		if ( count( $titleArray ) > 0 ) {
			$html = Xml::openElement( 'ul', array( 'id' => 'mw-favorites-list' ) ) . "\n";
			foreach ( $titleArray as $row ) {
				$title = Title::newFromRow( $row );
				$text = $title->getBaseText();
				$text = ( mb_strlen( $text ) > 14 ) ? mb_substr( $text, 0, 13 ) . "&hellip;" : $text;
				$html .= Xml::tags( 'li', null, $skin->link( $title, $text ) ) . "\n";
			}		
			$html .=
				Xml::closeElement( 'ul' ) . "\n" .
				Xml::element( 'a', array(
		       		'href' => SpecialPage::getTitleFor( 'Watchlist' )->getLocalURL() . '&action=edit',
		       		'class' => 'more'
		       	), wfMsg( 'tags_more' ) ) . "\n";
		} else {
			global $wgContLanguageCode;

			if ($wgContLanguageCode == 'pl') {
				$html = 
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'Nie ma jeszcze stron na twojej liście.' ) . "\n" .
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'Aby obserwować stronę kliknij na' ) . "\n" .
					Xml::element( 'div', array( 'id' => 'mw-favorites-add-icon' ), '', false ) . "\n" .
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'aby dodać ją do listy obserwowanych.' ) . "\n";
			} else {
				$html = 
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'There are no pages on your watchlist.' ) . "\n" .
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'To watch a page, click on' ) . "\n" .
					Xml::element( 'div', array( 'id' => 'mw-favorites-add-icon' ), '', false ) . "\n" .
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'to add it to your watchlist.' ) . "\n";
			}
		}
				
		$wgMemc->set( $key, $html );
		
		return $html;
	}
	
	public function getDashboardHTML() {
		global $wgUser;
		
		if ( !$wgUser->isLoggedIn() ) return;
		
		$skin = $wgUser->getSkin();
		
		if ( !$wgUser->isLoggedIn() ) 
			return Xml::span( wfMsg( 'notloggedin' ), null, array( 'style' => 'display: block; text-align: center;' ) );		

		$titleArray = self::getData( $wgUser );
		
		if ( count( $titleArray ) > 0 ) {
			$html = Xml::openElement( 'ul', array( 'style' => 'list-style-type: none; list-style-image: none; padding-left: 5px; margin: 0;' ) ) . "\n";
			foreach ( $titleArray as $row ) {
				$title = Title::newFromRow( $row );
				$rev = Revision::newFromTitle( $title );
				$timestamp = wfTimestamp( TS_DB, $rev->getTimestampFromId( $title, $title->getLatestRevID() ) );
				$text = $title->getBaseText();
				$text = ( mb_strlen( $text ) > 14 ) ? mb_substr( $text, 0, 13 ) . "&hellip;" : $text;
				$html .=
					Xml::openElement( 'li', array( 'style' => 'height: 16px; margin: 0 0 5px; padding-left: 24px; background: url(extensions/Favorites/icon.png) no-repeat center left' ) ) .
					$skin->link( $title, $text ) . '  ' . Xml::element( 'span', array( 'style' => 'color: #777777;' ), $timestamp ) . ' ' . $skin->revUserLink( $rev ) .
					Xml::closeElement( 'li' ) . "\n";
			}		
			$html .= Xml::closeElement( 'ul' ) . "\n";
		} else {
			$html = 
				Xml::openElement( 'table', array( 'cellpadding' => 5 ) ) . "\n" .
				Xml::openElement( 'tr' ) . "\n" .
				Xml::tags( 'td', null, Xml::element( 'img', array( 'src' => '/extensions/Favorites/watch.png' ) ) ) . "\n" .
				Xml::openElement( 'td' ) . "\n" .
				'There are no pages on your watchlist. ' .
				Xml::element( 'strong', null, 'Watch pages' ) .
				' that are most important to you, so you can be informed about its changes and edits.' . "\n" .
				Xml::closeElement( 'td' ) . "\n" .
				Xml::closeElement( 'tr' ) . "\n" .
				Xml::closeElement( 'table' ) . "\n";
					}
				
		return $html;
	}	
}
