<?php

class Groups {
    public static function getGroups($user = false) {
        $groups = array();

        $dbr =& wfGetDB( DB_SLAVE );
        
        if (!$user) {
            $res = $dbr->select(
                'groups',
                array( 'group_name' ),
                null,
                __METHOD__,
                array( 'ORDER BY' => 'group_name ASC' )
            );
        } else {
            $res = $dbr->select(
                array('groups', 'usergroups'),
                array( 'group_name' ),
                array('user_id' => $user),
                __METHOD__,
                array( 'ORDER BY' => 'group_name ASC' ),
                array('usergroups' => array('LEFT JOIN', 'user_group_id = group_id'))
            );
        }
        
        while ( $row = $dbr->fetchObject( $res ) ) {
            $groups[] = $row->group_name;
        }
        
        return $groups;
    }
}

class Group {
    private $data = array();

    private $userIds = array();
    
    public function __set( $name, $value ) {
        if ( in_array( $name, array( 'name', 'users' ) ) ) {
            $dbw =& wfGetDB( DB_MASTER );
            $dbw->begin();
            switch ( $name ) {
                case 'name':
                    $dbw->update(
                    	'groups',
                        array( 'group_name' => $value ),
                        array( 'group_id' => $this->id ),
                        __METHOD__
                    );
                    break;
                case 'users':
                    unset( $this->users );
                    foreach ( $value as $user ) {
                        $dbw->insert(
                        	'usergroups',
                            array( 'user_id' => $user, 'user_group_id' => $this->id ),
                            __METHOD__
                        );
                    }
                    wfRunHooks( 'Group__modify', array( $this->id ) );
                    break;                
            }
            $dbw->commit();
        }
        
        $this->data[$name] = $value; 
    }
    
    public function __get( $name ) {
        if ( empty( $this->data[$name] ) ) {
            $dbr =& wfGetDB( DB_SLAVE );
            
            switch ( $name ) {
                case 'id':
                    $this->data[$name] = $dbr->selectField(
                    	'groups',
                    	"group_{$name}",
                        array( 'group_name' => $this->name ),
                        __METHOD__
                    );
                    break;                    
                case 'name':
                    $this->data[$name] = $dbr->selectField(
                    	'groups',
                    	"group_{$name}",
                        array( 'group_id' => $this->id ),
                        __METHOD__
                    );
                    break;                    
                case 'users':
                    $this->data[$name] = array();
                    $res = $dbr->select(
                    	'usergroups',
                        array( 'user_id' ),
                        array( 'user_group_id' => $this->id ),
                        __METHOD__
                    );
                    while ( $row = $dbr->fetchObject( $res ) ) {
                        $this->data[$name][] = $row->user_id;
                    }                    
                    break;
            }
        }
        
        return $this->data[$name];
    }
    
    public function __isset( $name ) {
        return isset( $this->data[$name] );
    }
    
    public function __unset( $name ) {
        switch ( $name ) {
            case 'users':
                $dbw =& wfGetDB( DB_MASTER );
                $dbw->begin();
                $dbw =& wfGetDB( DB_MASTER );
                
                $dbw->begin();
                $dbw->delete( 'usergroups', array(
                	'user_group_id' => $this->id
                ), __METHOD__ );
                $dbw->commit();
                break;
        }
        
        unset( $this->data[$name] );
    }    
    
    public static function newFromName( $name ) {
        $group = new self();
        $group->data['name'] = $name;
        
        return $group;
    }

    public static function newFromID( $id ) {
        $group = new self();
        $group->data['id'] = $id;
        
        return $group;
    }
    
    /**
     * Enter description here...
     *
     * @param string $name
     * @param array $roles
     * @return LiveBlend_Group
     */
    public function create() {
        $dbw =& wfGetDB( DB_MASTER );
        
        $dbw->begin();
        $dbw->insert(
        	'groups',
            array( 'group_name' => $this->name ),
            __METHOD__
        );
        if ( $dbw->affectedRows() !== 1 ) {
            return false;
        }
        $this->id = $dbw->insertId();
        $dbw->commit();
        
        wfRunHooks( 'Group__create', array( $this->id ) );
        
        return true;
    }
    
    public function exists() {
        return $this->id != 0;
    }
    
    public function delete() {
        $dbw =& wfGetDB( DB_MASTER );
        
        unset( $this->users );
        $dbw->begin();
        $dbw->delete( 'groups', array(
        	'group_id' => $this->id
        ), __METHOD__ );
        $dbw->commit();
        
        wfRunHooks( 'Group__delete', array( $this->id ) );
        
        return true;
    }
}
