<?php

class ManageGroups extends SpecialPage {
    const id = 'managegroups-';
    
    public function __construct() {
        parent::__construct( 'ManageGroups' );
        wfLoadExtensionMessages( 'ManageGroups' );
    }
    
    public function execute( $group ) {
        global $wgOut, $wgRequest, $wgUser;
        
        $userId = $wgRequest->getVal('user', false);
        
        $group = str_replace( '_', ' ', $group );
        
        if ( $wgRequest->getText( self::id . 'create' ) ) {
                $reqGroupname = $wgRequest->getText( self::id . 'group-name' );
            if (
                !empty( $reqGroupname )
                && $group !== $reqGroupname
            ) {
                $wgOut->redirect(
                    SpecialPage::getTitleFor( 'ManageGroups' )->getLocalUrl() .
                    '/' . $wgRequest->getText( self::id . 'group-name' )
                );
            }
        }
        
        if ( $wgRequest->getText( 'action' ) === 'remove' ) {
            $group = Group::newFromName( $wgRequest->getText( 'group' ) );
            $group->delete();
            
            PageIndexView::onArticleChangeComplete();

            $wgOut->redirect( SpecialPage::getTitleFor( 'ManageGroups' )->getLocalUrl() );
        }        
        
        if ( $wgRequest->getText( self::id . 'cancel' ) ) {
            PageIndexView::onArticleChangeComplete();
            
            $wgOut->redirect( SpecialPage::getTitleFor( 'ManageGroups' )->getLocalUrl() );
            
            unset( $group );
        }
        
        $this->setHeaders();
        
		$wgOut->addLink( array( 
			'rel' => 'stylesheet',
			'type' => 'text/css',
			'href' => '/extensions/ManageGroups/ManageGroups.css'
		) );
		
		$sysop = in_array( 'sysop', $wgUser->getGroups() );
		
		if ( !empty( $group ) ) {
    		if ( !$sysop ) {
    		    $view = new ManageGroups_View( $group );
        		
                $wgOut->addHTML( $view->getViewPage() );
    		} else {
    		    $edit = new ManageGroups_Edit( $group );
        		
                $wgOut->addHTML( $edit->getEditPage() );
    		}
		} else {
		    $out = Xml::openElement( 'div', array( 'id' => 'managegroups' ) );
		    
		    $out .= Xml::openElement( 'form', array( 'action' => '', 'method' => 'post' ) );
		    $out .= Xml::fieldset( wfMsg( self::id . 'create-new' ) );
		    if ( $sysop ) {
    		    $out .= Xml::inputLabel(
    		        wfMsg( self::id . 'group-name' ),
    		        self::id . 'group-name',
    		        self::id . 'group-name',
    		        false, false,
    		        array( 'class' => 'text' )
    		    );
    		    $out .= Xml::submitButton( wfMsg( self::id . 'button-create' ), array( 'name' => self::id . 'create' ) );
		    } else {
		        $out .= Xml::element( 'p', null, wfMsg( self::id . 'create-info' ) );
		    }
		    $out .= Xml::closeElement( 'fieldset' );
		    $out .= Xml::closeElement( 'form' );
		    
		    $groups = Groups::getGroups($userId);
		    
		    if ( !empty( $groups ) ) {		        
        		$head = array(
        		    array( 'class' => 'groupname', 'content' => wfMsg( self::id . 'group-name' ), 'sorter' => true ),
        		    /* array( 'class' => 'edit', 'content' => null, 'sorter' => false ) */
        		);
        		if ( $sysop ) {
        		    array_push( $head, array( 'class' => 'delete', 'content' => null, 'sorter' => false ) );
        		}
        		
        		$url = SpecialPage::getTitleFor( 'ManageGroups' )->getLocalURL();
        		
        		$records = array();
           		foreach ( $groups as $group ) {
        		    $record = array();

        		    $record[] = array( 'class' => 'groupname', 'content' => Xml::element( 'a', array( 'href' => "{$url}/{$group}" ), $group ) );
        		    // $record[] = Xml::element( 'a', array( 'href' => "{$url}/{$group}", 'title' => wfMsg( self::id . 'button-edit' ), 'class' => self::id . 'edit' ), '', false );
        		    if ( $sysop ) {
        		        $record[] = Xml::element( 'a', array( 'href' => "{$url}&group={$group}&action=remove", 'class' => self::id . 'remove' ), '', false );
        		    }    		    
        		    
        		    $records[] = $record; 
        		}
        		
        		$out .= Xml::table(
    	        	self::id . 'table', $head, $records,
    	            array( 'class' => self::id . 'table' )
    	        );
    	        
                $out .= Xml::openElement( 'script', array( 'type' => $wgJsMimeType ) ) . "\n";
                $id = self::id;
                $msg = wfMsg( self::id . 'confirm-delete' );
                $out .= <<<JS
$().ready(function() {
	$('.{$id}remove').click(function() {
	    text = $(this).parents('tr').eq(0).find('.groupname').eq(0).text();
		return confirm('$msg "' + text + '" ?');
	});
});
JS;
                $out .= "\n" . Xml::closeElement( 'script' ) . "\n";
		    } else {
		        $out .= Xml::element( 'p', null, wfMsg( self::id . 'no-groups' ) );
		    }
	        
	        $out .= Xml::closeElement( 'div' );
    		
		    $wgOut->addHTML( $out );
		}
    }
}

class ManageGroups_View {
    const id = 'managegroups-';
    
    private $group;
    
    public function __construct( $groupName ) {
        $this->group =& Group::newFromName( $groupName );
    }
    
    public function getViewPage() {
        $group =& $this->group;
        
        $out = Xml::element( 'br' ) . Xml::element( 'br' );
        $out .= Xml::tags( 'h2', null, wfMsg( self::id . 'group-name' ) . ': ' . Xml::element( 'strong', null, $group->name ) );
        $out .= Xml::element( 'h2', null, wfMsg( self::id . 'legend-roles' ) . ':' );
        $roles = $group->roles;
        if ( !empty( $roles ) ) {
            $out .= Xml::openElement( 'ul' );
            foreach ( $group->roles as $role ) {
                $out .= Xml::tags( 'li', null, wfMsg( self::id . $role ) );
            }
            $out .= Xml::closeElement( 'ul' );
        } else {
            $out .= Xml::tags( 'p', null, wfMsg( self::id . 'no-roles' ) );
        }
        $out .= Xml::element( 'h2', null, wfMsg( self::id . 'legend-users' ) . ':' );
        $users = array();
        foreach ( $group->users as $user ) {
            $users[] = $user;
        }
        if ( !empty( $users ) ) {
            $out .= Xml::openElement( 'ul' );
            foreach ( $users as $user ) {
                $user = User::newFromId( $user );
                $name = ( $name = $user->getRealName() ) ? $name : $user->getName();
                $out .= Xml::tags( 'li', null, $name );
            }        
            $out .= Xml::closeElement( 'ul' );
        } else {
            $out .= Xml::tags( 'p', null, wfMsg( self::id . 'no-users' ) );
        }
        
        return $out;
    }
}

class ManageGroups_Edit {
    const id = 'managegroups-';
    
    private $group;
    
    public function __construct( $groupName ) {
        global $wgOut, $wgRequest;
        
        $save = $wgRequest->getText( self::id . 'save' );
        $saveAsNew = $wgRequest->getText( self::id . 'save-as-new' );
        
        if ( $save || $saveAsNew ) {
            $name = $wgRequest->getText( self::id . 'group-name' );
            if ( $save ) {
                $oldName = $wgRequest->getText( self::id . 'old-group-name' );
            }
            
            $group = Group::newFromName( !empty( $oldName ) ? $oldName : $name );
            
            if ( !$group->exists() ) {
                $group->create();
            }

            $group->users = $wgRequest->getArray( self::id . 'in' );

            if ( $save ) {
                if ( !empty( $oldName ) && $name != $oldName ) {
                    $group->name = $name;
                }
            }
            
            PageIndexView::onArticleChangeComplete();

            $wgOut->redirect( SpecialPage::getTitleFor( 'ManageGroups' )->getLocalUrl() );
        } else {
            $group = Group::newFromName( $groupName );
        }
        
        $this->group =& $group;
    }
    
    private function getLists() {
        $group =& $this->group;
        
        $out = Xml::openElement( 'table', array(
        	'id' => self::id . 'selects',
            'cellpadding' => 0, 'cellspacing' => 0
        ) ) . "\n";
        
        $out .= Xml::openElement( 'thead' );
        
        $out .= Xml::closeElement( 'thead' );
        $out .= Xml::openElement( 'tr' );
        $out .= Xml::element( 'th', null, wfMsg( self::id . 'available-users' ) );
        $out .= Xml::element( 'th', null, '', false );
        $out .= Xml::element( 'th', null, wfMsg( self::id . 'selected-users' ) );
        $out .= Xml::closeElement( 'tr' );        
        $out .= Xml::openElement( 'tbody' );
        
        $out .= Xml::openElement( 'tr' ) . "\n";
        
        $dbr =& wfGetDB( DB_SLAVE );

        $res = $dbr->select( 
            'user',
			array( 'user_id', 'user_name', 'user_real_name' ),
			null,
			__METHOD__
		);
		
		$userArray = UserArray::newFromResult( $res );

		$out .= Xml::openElement( 'td', array( 'class' => 'notin' ) ) . "\n";
		
		$out .= Xml::openElement( 'select', array(
			'id' => self::id . 'notin',
			'multiple' => 'multiple'
		) ) . "\n";
		
		list( $outNotIn, $outIn ) = array( '', '' ); 
		foreach ( $userArray as $user ) {
		    $target = !in_array( $user->getId(), $group->users ) ? 'outNotIn' : 'outIn';
		    ${$target} .= Xml::option( ( $name = $user->getRealName() ) ? $name : $user->getName(), $user->getId() ) . "\n";
		}
		
		$out .= $outNotIn;
		
		$out .= Xml::closeElement( 'select' ) . "\n";
		
		$out .= Xml::closeElement( 'td' ) . "\n";
		
		$out .= Xml::openElement( 'td', array( 'class' => 'control' ) ) . "\n";
		
		$out .= Xml::element( 'a', array( 'id' => self::id . 'add', 'href' => '#' ), '', false );
		$out .= Xml::element( 'a', array( 'id' => self::id . 'remove', 'href' => '#' ), '', false );
		
		$out .= Xml::closeElement( 'td' ) . "\n";
		
		$out .= Xml::openElement( 'td', array( 'class' => 'in' ) ) . "\n";
		
		$out .= Xml::openElement( 'select', array(
			'id' => self::id . 'in',
			'name' => self::id . 'in[]',
			'multiple' => 'multiple'
		) ) . "\n";
		
		$out .= $outIn;
				
		$out .= Xml::closeElement( 'select' ) . "\n";
        
		$out .= Xml::closeElement( 'td' ) . "\n";
		
		$out .= Xml::closeElement( 'tr' ) . "\n";
		
		$out .= Xml::closeElement( 'tbody' );
		
		$out .= Xml::closeElement( 'table' ) . "\n";
        
        $out .= Xml::openElement( 'script', array( 'type' => $wgJsMimeType ) ) . "\n";
        $id = self::id;
        $out .= <<<JS
$().ready(function() {
	$('#{$id}add').click(function() {
		return !$('#{$id}notin option:selected').remove().appendTo('#{$id}in').bind('mousedown', function() {
    		$('#{$id}notin option').removeAttr('selected');
    	});
	});
	$('#{$id}remove').click(function() {
		return !$('#{$id}in option:selected').remove().appendTo('#{$id}notin').bind('mousedown', function() {
    		$('#{$id}in option').removeAttr('selected');
    	});
	});
	
	$('#{$id}notin option').bind('mousedown', function() {
		$('#{$id}in option:selected').removeAttr('selected');
	});	
	
	$('#{$id}in option').bind('mousedown', function() {
		$('#{$id}notin option:selected').removeAttr('selected');
	});
});
JS;
        $out .= "\n" . Xml::closeElement( 'script' ) . "\n";

        return $out;        
    }
    
    public function getEditPage() {
        global $wgJsMimeType;
        
        $group =& $this->group;
        
        $out = Xml::openElement( 'div', array( 'id' => 'managegroups' ) );
        
        $out .= Xml::openElement( 'form', array( 'id' => self::id . 'form', 'action' => '', 'method' => 'post' ) );
        
        $out .= Xml::openElement( 'div', array( 'id' => self::id . 'group' ) );
        
        $out .= Xml::fieldset( wfMsg( self::id . 'legend-name' ), false, array( 'class' => 'name' ) );
        $out .= Xml::inputLabel(
            wfMsg( self::id . 'group-name' ),
            self::id . 'group-name',
            self::id . 'group-name',
            false,
            $group->name,
            array( 'class' => 'text' )
        );
        $out .= Xml::hidden( self::id . 'old-group-name', $group->name );
        $out .= Xml::closeElement( 'fieldset' );
        
        $out .= Xml::fieldset( wfMsg( self::id . 'legend-users' ), false, array( 'class' => 'selects' ) );
        $out .= $this->getLists();
        $out .= Xml::closeElement( 'fieldset' );

        $out .= Xml::closeElement( 'div' );
        
        $out .= Xml::openElement( 'div', array( 'id' => self::id . 'buttons' ) );        
        $out .= Xml::submitButton( wfMsg( self::id . 'button-cancel' ), array( 'name' => self::id . 'cancel' ) );
        $out .= Xml::submitButton( wfMsg( self::id . 'button-save' ), array( 'name' => self::id . 'save' ) );
        if ( $group->exists() ) {
            $out .= Xml::submitButton( wfMsg( self::id . 'button-save-as-new' ), array( 'name' => self::id . 'save-as-new' ) );
        }
        $out .= Xml::closeElement( 'div' );
        
        $out .= Xml::closeElement( 'form' );
        
        $out .= Xml::openElement( 'script', array( 'type' => $wgJsMimeType ) ) . "\n";
        $id = self::id;
        $out .= <<<JS
$().ready(function() {
	$('#{$id}form').submit(function () {
		$("#{$id}in option").attr("selected", "selected");
	
		return true;
	});
});
JS;
        $out .= "\n" . Xml::closeElement( 'script' ) . "\n";
        
        $out .= Xml::closeElement( 'div' );
        
        return $out;
    }
}