<?php

/**
 * MediaWiki MyPages extension
 */

if ( ! defined( 'MEDIAWIKI' ) )
	die( 1 );
	
$wgAutoloadClasses['MyPagesDashboardTemplate'] = dirname(__FILE__) . '/MyPagesDashboardTemplate.php';

$wgExtensionCredits['parserhook'][] = array(
	'name' => 'MyPages',
	'author' => 'Paweł Szreder',
	'version'        => '0.1',
	'description' => 'Adds my pages box',
	'descriptionmsg' => 'mypages_desc',
	'url' => 'http://www.bizneswiki.pl'
);

$dir = dirname(__FILE__);
require_once( "{$dir}/MyPages.i18n.php" );

$wgExtensionFunctions[] = 'wfMyPages';
$wgHooks['LanguageGetMagic'][] = 'wfMyPagesMagic';
$wgHooks['BeforePageDisplay'][] = 'MyPages::onBeforePageDisplay';
$wgHooks['ArticleInsertComplete'][] = 'MyPages::onArticleChangeComplete';
$wgHooks['ArticleDeleteComplete'][] = 'MyPages::onArticleChangeComplete';

require_once( 'SpecialPage.php' );

$wgSpecialPages['MyPages'] = 'SpecialMyPages';
// $wgHooks['LanguageGetSpecialPageAliases'][] = 'wfSpecialMyPagesLocalizedPageName';

function wfMyPages() {
	global $wgParser, $wgMyPagesMessages, $wgMessageCache;
	
	$wgParser->setHook( 'MyPages', 'MyPages' );
	
	$wgParser->setFunctionHook( 'mypages', 'wfMyPagesGetHTML' );
	
	foreach ( $wgMyPagesMessages as $lang => $messages) {
		$wgMessageCache->addMessages( $messages, $lang );
	}	
}
 
function wfMyPagesMagic( &$magicWords, $langCode ) {
	$magicWords['mypages'] = array( 0, 'mypages' );

	return true;
}

function wfMyPagesGetHTML() {
    global $wgUser;
		
	if ( !$wgUser->isLoggedIn() ) return;
    
	$skin = $wgUser->getSkin();
		
	$titleArray = MyPages::getData( $wgUser );

	$titles = array();
	if ( count( $titleArray ) > 0 ) {
	    foreach ( $titleArray as $row ) {
			$title = Title::newFromRow( $row );				
			$text = $title->getBaseText();
			$text = ( mb_strlen( $text ) > 14 ) ? mb_substr( $text, 0, 13 ) . "&hellip;" : $text;
			$titles[] = $skin->link( $title, $text );
	    }
	}
	
	$template = new MyPagesDashboardTemplate();
	$template->set( 'titles', $titles );
	
	return array( $template->fetch(), 'isHTML' => true );
}

class MyPages {
	public function __construct() { }
	
	public static function onBeforePageDisplay( OutputPage &$out ) {
		global $wgScriptPath, $wgUser;
		
		$url = "{$wgScriptPath}/extensions/" . __CLASS__ . '/';
			
		$out->addExtensionStyle( $url . __CLASS__ . '.css' );
		
		if ( !$wgUser->isLoggedIn() ) return true;
		 
		return true;
	}
	
	public static function onArticleChangeComplete( Article &$article ) {
		global $wgDBname, $wgMemc, $wgUser;
		
		$key = "{$wgDBname}:mypages:" . (string) $wgUser->getId();
		
		$wgMemc->delete( $key );
		
		self::getData( $wgUser );
		
		return true;
	}
	
	private static function firstContributor( $pageId, $revId ) {
		 $dbr =& wfGetDB( DB_SLAVE );
		 
		 $firstRevId = $dbr->selectField(
		 	'revision',
		 	'MIN(rev_id)',
		 	array( 'rev_page' => $pageId ),
		 	__METHOD__,
		 	array( 'GROUP BY' => 'rev_page' )
		 );
		 
		 return ( $firstRevId == $revId ) ? true : false;
	}
	
	public static function getData( User $user ) {
		$dbr =& wfGetDB( DB_SLAVE );
		
    	$sql = $dbr->selectSQLText(
    		array( 'page', 'revision' ),
    		array( 'page_id', 'page_namespace', 'page_title', 'rev_user', 'MAX(rev_timestamp) AS max_rev_timestamp' ),
    		array( 'page_namespace' => NS_MAIN ),
    		__METHOD__,
    		array( 'GROUP BY' => 'rev_page', 'ORDER BY' => 'rev_page DESC' ),
    		array( 'revision' => array( 'LEFT JOIN', 'rev_page = page_id' ) )
    	);
    	$res = $dbr->select(
    		"({$sql}) AS sub",
    		array( 'page_id', 'page_namespace', 'page_title' ),
    		array( 'rev_user' => $user->getId() ),
    		__METHOD__,
    		array( 'ORDER BY' => 'max_rev_timestamp DESC', 'LIMIT' => 5 )
    	);
				
		$titleArray = array();
		while ( $row = $dbr->fetchObject( $res ) ) {
			array_push( $titleArray, $row );
		}
		
		return $titleArray;
	}
	
	public function getHTML() {
		global $wgUser, $wgMemc, $wgDBname;
		
		if ( !$wgUser->isLoggedIn() ) return;
		
		$key = "{$wgDBname}:mypages:" . (string) $wgUser->getId();
		
		$data = $wgMemc->get( $key );
		
		unset( $data ); /* debug */
		
		if ( !empty( $data ) ) return $data;
		
		$skin = $wgUser->getSkin();
		
		if ( !$wgUser->isLoggedIn() )
			return Xml::span( wfMsg( 'notloggedin' ), null, array( 'style' => 'display: block; text-align: center;' ) );

		$titleArray = self::getData( $wgUser );

		if ( count( $titleArray ) > 0 ) {
			$html = Xml::openElement( 'ul', array( 'id' => 'mw-mypages-list' ) ) . "\n";
			foreach ( $titleArray as $row ) {
				$title = Title::newFromRow( $row );				
				$text = $title->getBaseText();
				$text = ( mb_strlen( $text ) > 14 ) ? mb_substr( $text, 0, 13 ) . "&hellip;" : $text;
				$html .=
					Xml::openElement( 'li' ) .
					$skin->link( $title, $text ) .
					Xml::closeElement( 'li' ) . "\n";
			}
			$html .=
				Xml::closeElement( 'ul' ) . "\n" .
				Xml::element( 'a', array(
		       		'href' => SpecialPage::getTitleFor( 'MyPages' )->getLocalURL(),
		       		'class' => 'more'
		       	), wfMsg( 'tags_more' ) ) . "\n";
			
		} else {
			global $wgContLanguageCode;

			if ($wgContLanguageCode == 'pl') {
				$html = 
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'Nie utworzyłeś jeszcze żadnej strony. ' ) . "\n" .
					Xml::element( 'div', array( 'id' => 'mw-mypages-create-icon' ), '', false ) . "\n" .
					Xml::tags( 'p', array( 'style' => 'padding: 5px;' ), 'Aby wyświetlić strony tutaj, utwórz ' . Xml::element( 'strong', null, 'nową stronę' ) . '.' ) . "\n";
			} else {
				$html = 
					Xml::element( 'p', array( 'style' => 'padding: 5px;' ), 'You haven\'t created any page yet.' ) . "\n" .
					Xml::element( 'div', array( 'id' => 'mw-mypages-create-icon' ), '', false ) . "\n" .
					Xml::tags( 'p', array( 'style' => 'padding: 5px;' ), 'To display your pages here, create a ' . Xml::element( 'strong', null, 'new page' ) . '.' ) . "\n";
			}
		}
		
		$wgMemc->set( $key, $html );
		
		return $html;
	}
}

function wfSpecialMyPagesLocalizedPageName( &$specialPageArray, $code ) {
	wfLoadExtensionMessages( 'CategoryPageView' );
	
	// $title = Title::newFromText( wfMsg( 'pageindex' ) );
	$specialPageArray[ 'MyPages' ][] = 'My pages';

	return true;
}

class MyPagesView extends PageIndexView {
	public function __construct() {
		parent::__construct();
	}
	
	public function viewCacheKey( $type ) {
		global $wgUser, $wgTitle, $wgMemc, $wgDBname, $wgCategoryPagingLimit;
		
		$userId = $wgUser->getId();
		$letter = !empty( $this->letter ) ? $this->letter : 'any';
		$order = $this->order;
		$page = $this->page;
		
		switch ( $type ) {
			case PAGEINDEX_VIEW_CACHE:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:{$userId}:mypages:{$letter}:{$order}:{$page}";
			case PAGEINDEX_VIEW_CACHE_ALL:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:all:mypages";
			case PAGEINDEX_VIEW_CACHE_USER:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:user:{$userId}:mypages";
			case PAGEINDEX_VIEW_CACHE_EVERYTHING:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		}
	}
	
	public function getPagesData( $cheap = false ) {
		global $wgTitle, $wgUser, $wgCategoryPagingLimit;
		
		$userId = $wgUser->getId();
		
		$dbr =& wfGetDB( DB_SLAVE );
		
		if ( !$cheap && !empty( $this->letter ) ) {
			$condition = "page_title LIKE {$dbr->addQuotes( $this->letter . '%' )}";
		} else {
			$condition = '1 = 1';
		}
		
		switch ( $this->order ) {
			case 'n':
				$order = 'page_namespace DESC, page_title'; break;
			case 'nd':
				$order = 'page_namespace DESC, page_title DESC'; break;
			case 'r':
				$order = 'page_namespace DESC, max_rev_timestamp'; break;
			case 'rd':
				$order = 'page_namespace DESC, max_rev_timestamp DESC'; break;				
			default:
				$order = 'page_namespace DESC, page_title';	
		}
		
		$ns = $wgTitle->getNamespace();
		
		if ( !$cheap) {
	    	$sql = $dbr->selectSQLText(
	    		array( 'page', 'revision' ),
	    		array( 'page_id', 'page_namespace', 'page_title', 'rev_user', 'MAX(rev_timestamp) AS max_rev_timestamp' ),
	    		array( 'page_namespace' => array( NS_MAIN, NS_IMAGE ), $condition ),
	    		__METHOD__,
	    		array( 'GROUP BY' => 'rev_page', 'ORDER BY' => 'rev_page DESC' ),
	    		array( 'revision' => array( 'LEFT JOIN', 'rev_page = page_id' ) )
	    	);
	    	$count = $dbr->selectField(
	    		"({$sql}) AS sub",
	    		'COUNT(page_id)',
	    		array( 'rev_user' => $userId ),
	    		__METHOD__
	    	);
		}
		
    	$sql = $dbr->selectSQLText(
    		array( 'page', 'revision' ),
    		array( 'page_id', 'page_namespace', 'page_title', 'rev_user', 'MAX(rev_timestamp) AS max_rev_timestamp' ),
    		array( 'page_namespace' => array( NS_MAIN, NS_IMAGE ), $condition ),
    		__METHOD__,
    		array( 'GROUP BY' => 'rev_page', 'ORDER BY' => 'rev_page DESC' ),
    		array( 'revision' => array( 'LEFT JOIN', 'rev_page = page_id' ) )
    	);
    	$res = $dbr->select(
    		"({$sql}) AS sub",
    		array( 'page_id', 'page_namespace', 'page_title' ),
    		array( 'rev_user' => $userId ),
    		__METHOD__,
    		!$forLetters
    			? array( 'ORDER BY' => $order, 'LIMIT' => $wgCategoryPagingLimit, 'OFFSET' => $this->offset )
    			: array( 'ORDER BY' => $order )
    	);
		
		$titleArray = array();
		while ( $row = $dbr->fetchObject( $res ) ) {
			array_push( $titleArray, Title::newFromRow( $row ) );
		}
		
    	return ( !$cheap ) ? array( 'count' => $count, 'data' => $titleArray ) : $titleArray;	
	}	
}

class SpecialMyPages extends SpecialPage {
	public function __construct() {
		SpecialPage::SpecialPage( 'MyPages' );
	}
	
	public function execute() {
		global $wgOut, $wgRequest;
		
		// wfLoadExtensionMessages( 'CategoryPageView' );
		
		$this->setHeaders();
		$wgOut->setPagetitle( 'My pages' );
		
		$view = new MyPagesView();
				
		$wgOut->addHTML( $view->getPagesHTML() );	
	}
}
