<?php

$wgExtensionCredits['other'][] = array(
	'name' => 'Optimizer',
	'author' => 'Paweł Szreder',
	'url' => 'http://onbusinesswiki.com/',
	'version' => '0.0.1',
);

$wgHooks['SkinTemplateOutputPageBeforeExec'][] = 'Optimizer::onSkinTemplateOutputPageBeforeExec';

class Optimizer
{
    private $headscripts;
    
    private $csslinks;
    
    public function __construct($tpl)
    {
        $this->headscripts =& $tpl->data['headscripts'];
        
        $this->csslinks =& $tpl->data['csslinks'];
    }
    
    private function prepareStyleFileString($file, $version = true)
    {
        global $wgStyleVersion;

        return "<link rel=\"stylesheet\" href=\"{$file}" . ($version ? "?{$wgStyleVersion}" : '') . "\" type=\"text/css\" media=\"screen\" />" . "\n";
    }
    
    private function prepareScriptFileString($file, $version = true)
    {
        global $wgStyleVersion, $wgJsMimeType;
        
        return "<script type=\"{$wgJsMimeType}\" src=\"{$file}" . ($version ? "?{$wgStyleVersion}" : '') . "\"></script>" . "\n";
    }
    
    private function optimizeStyles()
    {
        global $wgStyleVersion, $IP, $wgUploadDirectory, $wgUploadPath;
        
        @preg_match_all('/<link rel="stylesheet" href="(.*?)"/', $this->csslinks, $matches);
        
        $timestamp = '';
        $stitched = $external = array();
        $styles = array_unique($matches[1]);
        foreach ($styles as $style) {
            $style = str_replace("?{$wgStyleVersion}", '', $style);
            $file = $IP . $style;
            if (@file_exists($file)) {
                $timestamp .= '|' . @filemtime($file);
                $stitched[] = $style;
            } elseif (strpos($file, 'http://')) {
                $external[] = $style;
            } else {
                var_dump($file);
            }
        }
        
        $result = '';
        
        foreach ($external as $style) {
            $result .= $this->prepareStyleFileString($style);
        }
        
        $filename = md5(implode('|', $stitched) . $timestamp) . '.css';
        $file = $wgUploadDirectory . '/' . $filename;
        if (!file_exists($file)) {
            foreach ($stitched as &$style) {
                $content = @file_get_contents($IP . $style);
                $content = @preg_replace('/url\s*?\(\s*?[\'"]?[\/]?([^\s\)\'"]+)[\'"]?\s*?\)/i', 'url(' . dirname($style) . '/$1)', $content);
                
                $style = $content;
            }
            
            require_once dirname(__FILE__) . '/library/cssmin/cssmin.php';
            
            @file_put_contents($file, cssmin::minify(implode("\n\n", $stitched)));
        }
        $style = $wgUploadPath . '/' . $filename;
        $result .= $this->prepareStyleFileString($style);
         
        $this->csslinks = $result;
    }
    
    private function optimizeScripts()
    {
        global $wgStyleVersion, $IP, $wgJsMimeType, $wgUploadDirectory, $wgUploadPath;
        
        @preg_match_all('/<script type="text\/javascript" src="(.*?)"/', $this->headscripts, $matches);
        
        $timestamp = '';
        $stitched = $external = array();
        $scripts = array_unique($matches[1]);
        foreach ($scripts as $script) {
            $script = str_replace("?{$wgStyleVersion}", '', $script);
            $file = $IP . $script;
            if (@file_exists($file)) {
                $timestamp .= '|' . @filemtime($file);
                $stitched[] = $script;
                $this->headscripts = str_replace(
                    $this->prepareScriptFileString($script), '', $this->headscripts
                );
            } elseif (strpos($file, 'http://')) {
                $external[] = $script;
                $this->headscripts = str_replace(
                    $this->prepareScriptFileString($script, false), '', $this->headscripts
                );
            }
        }
        
        $result = '';
        
        foreach ($external as $script) {
            $result .= $this->prepareScriptFileString($script);
        }

        $filename = md5(implode('|', $stitched) . $timestamp) . '.js';
        $file = $wgUploadDirectory . '/' . $filename;
        if (!file_exists($file)) {
            foreach ($stitched as &$script) {
                $script = @file_get_contents($IP . $script);
            }
            
            require_once dirname(__FILE__) . '/library/jsmin/jsmin.php';
            
            @file_put_contents($file, JSMin::minify(implode("\n\n", $stitched)));
        }
        $script = $wgUploadPath . '/' . $filename;
        $result .= $this->prepareScriptFileString($script);
         
        $this->headscripts = $result . $this->headscripts;
    }
    
    public static function onSkinTemplateOutputPageBeforeExec(&$st, &$tpl)
    {
        $optimizer = new self($tpl);
        
        $optimizer->optimizeStyles();
        $optimizer->optimizeScripts();
        
        return true;
    }
}
