<?php

/**
 * MediaWiki PageIndex special page
 */

if ( !defined( 'MEDIAWIKI' ) )
	die( 1 );

define( 'PAGEINDEX_VIEW_CACHE', 0 );
define( 'PAGEINDEX_VIEW_CACHE_ALL', 1 );
define( 'PAGEINDEX_VIEW_CACHE_USER', 2 );
define( 'PAGEINDEX_VIEW_CACHE_EVERYTHING', 3 );
	
require_once( 'SpecialPage.php' );

$wgSpecialPages['PageIndex'] = 'SpecialPageIndex';
$wgHooks['BeforePageDisplay'][] = 'PageIndexView::onBeforePageDisplay';
$wgHooks['ArticleInsertComplete'][] = 'PageIndexView::onArticleChangeComplete';
$wgHooks['ArticleSaveComplete'][] = 'PageIndexView::onArticleChangeComplete';
$wgHooks['ArticleDeleteComplete'][] = 'PageIndexView::onArticleChangeComplete';
$wgHooks['ArticleMoveComplete'][] = 'PageIndexView::onArticleChangeComplete';
$wgHooks['NewRevisionFromEditComplete'][] = 'PageIndexView::onArticleChangeComplete';
$wgHooks['UserRights'][] = 'PageIndexView::onUserRights';
$wgHooks['LanguageGetSpecialPageAliases'][] = 'wfSpecialPageIndexLocalizedPageName';

$wgAjaxExportList[] = 'wfSajaxPageIndexRename';

function wfSpecialPageIndexLocalizedPageName( &$specialPageArray, $code ) {
	wfLoadExtensionMessages( 'CategoryPageView' );
	
	$title = Title::newFromText( wfMsg( 'pageindex' ) );
	$specialPageArray['PageIndex'][] = $title->getDBKey();

	return true;
}

function wfSajaxPageIndexRename( $articleId, $newText ) {
	$title = Title::newFromID( $articleId );
	if ( !( $title instanceof Title ) ) {
		return 'false1|Unknown error.';
	}
	
	if ( $title->getBaseText() == $newText ) {
		return 'true';
	}
	
	$newTitle = Title::newFromText( $newText, $title->getNamespace() );
	if ( !( $newTitle instanceof Title ) || ( $newTitle->getBaseText() != ucfirst( $newText ) ) ) {
		return 'false|Invalid name.';
	} elseif ( $newTitle->exists() ) {
		if ( $newTitle->getNamespace() == NS_MAIN ) {
			return 'false3|Page with specified name: "' . ucfirst( $newText ) . '" already exists.';
		} elseif ( $newTitle->getNamespace() == NS_CATEGORY ) {
			return 'false3|Folder with specified name: "' . ucfirst( $newText ) . '" already exists.';
		} else {
			return 'true';
		}		
	}
	
	$result = $title->moveTo( $newTitle, true, '', true );
	if ( $result !== true ) {
		return 'false4|Unknown error.';
	}
	
	// TODO: Dlaczego Title::moveTo nie aktualizuje samo?..
	$dbw =& wfGetDB( DB_MASTER );
	$dbw->update(
		'categorylinks',
		array( 'cl_to' => $newTitle->getDBkey(), 'cl_sortkey' => $newTitle->getPrefixedText() ),
		array( 'cl_sortkey' => $title->getPrefixedText() ),
		__METHOD__
	);
	
	$article = new Article( $title );
	$article->doDeleteArticle( '', true );
	
	PageIndexView::onArticleChangeComplete();
	
	return $newText;
}

class SpecialPageIndex extends SpecialPage {
	public function __construct() {
		SpecialPage::SpecialPage( 'PageIndex' );
	}
	
	public function execute() {
		global $wgOut, $wgRequest;
		
		wfLoadExtensionMessages( 'CategoryPageView' );
		
		$this->setHeaders();
		$wgOut->setPagetitle( wfMsg( 'pageindex' ) );
		
		$view = new CategoryPageView( new Article( SpecialPageIndex::getTitle() ) );
		
		$wgOut->addHTML( $view->getPagesHTML() );	
	}
}

class PageIndexView {
    static $initialized = false;
    
	private $letters = array(
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
		'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
	);
	
	private $article;
	private $title;
	public $page;
	public $offset;
	public $letter;
	public $order;

	private $pages;
	
	public function __construct() {
	    self::$initialized = true;
	    
		global $wgRequest, $wgCategoryPagingLimit;
		
		$this->article =& $article;
		$page = $wgRequest->getInt( 'page' );
		$this->page = ( $page == 0 ) ? 1 : $page;
		$this->offset = ($this->page - 1) * $wgCategoryPagingLimit;
		$this->letter = $wgRequest->getText( 'letter', '' );
		$order = $wgRequest->getText( 'order', '' );
		$this->order = ( is_null( $order ) || !in_array( $order, array( 'n', 'nd', 'r', 'rd' ) ) ) ? 'n' : $order;
		
		$this->letters = array_combine( $this->letters, array_fill( 0, count( $this->letters ), 0 ) );
		
		$this->pages = $this->getPagesData();
	}
	
	public static function onBeforePageDisplay( &$out ) {
		if ( self::$initialized ) {
    	    global $wgStylePath, $wgDefaultSkin;
    		
    		$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/PageIndex.css" );
    		$out->addScriptFile( "/extensions/PageIndex/PageIndex.js" );
		}

		return true;
	}
	
	public static function onUserRights( User $user ) {
		global $wgMemc, $wgDBname, $wgCategoryPagingLimit/*, $wgTitle*/;
		
		// $userId = $user->getId();
		// $titleId = $wgTitle->getArticleID();
		
		// $key = "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:user:{$userId}:{$titleId}"; // TODO: ...
		$key = "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		
		$group = $wgMemc->get( $key );
		
		foreach ( $group as $item ) {
			$wgMemc->delete( $item );
		}
		
		$wgMemc->delete( $key );
		
		return true;
	}
	
	public static function onArticleChangeComplete() {
		global $wgMemc, $wgDBname, $wgCategoryPagingLimit/*, $wgTitle*/;
		
		// $titleId = $wgTitle->getArticleID();
		
		// $key = "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:all:{$titleId}"; // TODO: ...
		$key = "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		
		$group = $wgMemc->get( $key );
		
		if ( is_array( $group ) ) {
			foreach ( $group as $item ) {
				$wgMemc->delete( $item );
			}
		}
		
		$wgMemc->delete( $key );
		
		return true;
	}
	
	public function viewCacheKey( $type ) {
		global $wgUser, $wgTitle, $wgMemc, $wgDBname, $wgCategoryPagingLimit;
		
		$userId = $wgUser->getId();
		$titleId = $wgTitle->getArticleID();
		$letter = !empty( $this->letter ) ? $this->letter : 'any';
		$order = $this->order;
		$page = $this->page;
		
		switch ( $type ) {
			case PAGEINDEX_VIEW_CACHE:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:{$userId}:{$titleId}:{$letter}:{$order}:{$page}";
			case PAGEINDEX_VIEW_CACHE_ALL:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:all:{$titleId}";
			case PAGEINDEX_VIEW_CACHE_USER:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:user:{$userId}:{$titleId}";
			case PAGEINDEX_VIEW_CACHE_EVERYTHING:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		}
	}
	
	private function viewCacheGroupGet( $type ) {
		global $wgMemc;
		
		$group = $wgMemc->get( $this->viewCacheKey( $type ) );
		
		return empty( $group ) ? array() : $group;
	}
	
	private function viewCacheGroupAdd( $type, $add ) {
		global $wgMemc;
		
		$group = $this->viewCacheGroupGet( $type );
		array_push( $group, $add );
		
		$wgMemc->set( $this->viewCacheKey( $type ), $group, 3600 );
	}
		
	public function viewCacheGet() {
		global $wgMemc;
		
		$key = $this->viewCacheKey( PAGEINDEX_VIEW_CACHE );
		
		$view = $wgMemc->get( $key );
		
		return empty( $view ) ? false : $view;
	}
	
	public function viewCacheSet( $view ) {
		global $wgMemc;
		
		$key = $this->viewCacheKey( PAGEINDEX_VIEW_CACHE );
		
		$wgMemc->set( $key, $view, 3600 );
		
		$this->viewCacheGroupAdd( PAGEINDEX_VIEW_CACHE_ALL, $key );
		$this->viewCacheGroupAdd( PAGEINDEX_VIEW_CACHE_USER, $key );
		$this->viewCacheGroupAdd( PAGEINDEX_VIEW_CACHE_EVERYTHING, $key );
	}
	
	private function getLettersLinks() {
		global $wgUser, $wgTitle, $wgRequest;
		
		$skin = $wgUser->getSkin();
		
		$titleArray = $this->getPagesData( true );
		
		$params = array();
		$tag = $wgRequest->getVal( 'tag' );
		if ( !empty( $tag ) ) {
			$params['tag'] = $tag;
		}
				
		foreach ( $titleArray as $title ) {
			$basename = $title->getBaseText();
			$letter = strtoupper( $basename[0] );
			if ( array_key_exists( $letter, $this->letters ) ) {
				++ $this->letters[$letter];
			}
		}

		$html =
			'<table id="mw-letters">' . "\n" .
			'<tr>' .
			'<td>' . wfMsg( 'table_pager_limit_submit' ) . ': </td>' .
			'<td class="link">' . $skin->link( $wgTitle, wfMsg( 'rc_categories_any' ), array(), $params ) . '</td>';
		
		foreach ( $this->letters as $letter => $count ) {
			$html .= '<td class="link letter' . ( ( $count > 0 ) ? ' notempty' : '' ) . '">';
			if ( $count > 0 ) {
				$link = $skin->link( $wgTitle, $letter,
					array( 'title' => wfMsg( 'pageindex_pagecount', $count ) ),
					array_merge( $params, array( 'letter' => $letter ) )
				);
				$html .= ( $letter == $this->letter ) ? "[{$link}]" : $link;
			} else {
				$html .= $letter;
			}
			$html .= '</td>';
		}
		
		$html .=
			'</tr>' . "\n" .
			'</table>' . "\n";
		
		return $html;
	}

	public function getPagesData( $cheap = false ) {
		global $wgTitle, $wgCategoryPagingLimit;
		
		$dbr =& wfGetDB( DB_SLAVE );
		
		if ( !$cheap && !empty( $this->letter ) ) {
			$condition = "page_title LIKE {$dbr->addQuotes( $this->letter . '%' )}";
		} else {
			$condition = '1 = 1';
		}
		
		switch ( $this->order ) {
			case 'n':
				$order = 'page_namespace DESC, page_title'; break;
			case 'nd':
				$order = 'page_namespace DESC, page_title DESC'; break;
			case 'r':
				$order = 'page_namespace DESC, COALESCE(rc_timestamp,page_touched)'; break;
			case 'rd':
				$order = 'page_namespace DESC, COALESCE(rc_timestamp,page_touched) DESC'; break;				
			default:
				$order = 'page_namespace DESC, page_title';	
		}
		
		$ns = $wgTitle->getNamespace();
		
		if ( !$cheap) {
			if ( $ns == NS_CATEGORY ) {
				$count = $dbr->selectField(
		    		array( 'page', 'categorylinks' ),
		    		'COUNT(page_id)',
		    		array(
		    			'page_namespace' => array( NS_MAIN, NS_CATEGORY, NS_IMAGE ), 'cl_from = page_id',
		    			'cl_to' => $wgTitle->getDBkey(), 'cl_to_namespace' => NS_CATEGORY,
		    			$condition
		    		),
		    		__METHOD__		
				);
			} else {
				$count = $dbr->selectField(
					'page',
					'COUNT(page_id)',
					array( 'page_namespace' => array( NS_MAIN, NS_CATEGORY, NS_IMAGE ), $condition ),
					__METHOD__
				);
			}
		}
		
		if ( $ns == NS_CATEGORY ) {
	    	$res = $dbr->select(
	    		array( 'page', 'categorylinks', 'recentchanges' ),
	    		array( 'page_id', 'page_namespace', 'page_title' ),
	    		array(
	    			'page_namespace' => array( NS_MAIN, NS_CATEGORY, NS_IMAGE ), $condition,
	    			'cl_to' => $wgTitle->getDBkey(), 'cl_to_namespace' => NS_CATEGORY
	    		),
	    		__METHOD__,
    			array_merge(
    				array( 'ORDER BY' => $order, 'GROUP BY' => 'page_id' ),
    				!$cheap ? array( 'LIMIT' => $wgCategoryPagingLimit, 'OFFSET' => $this->offset ) : array()
    			),
	    		array(
	    			'categorylinks' => array( 'LEFT JOIN', 'cl_from = page_id' ),
	    			'recentchanges' => array( 'LEFT JOIN', 'rc_id = page_latest' )
	    		)
	    	);
		} else {
			$res = $dbr->select(
				array( 'page', 'recentchanges' ),
				array( 'page_id', 'page_namespace', 'page_title' ),
				array( 'page_namespace' => array( NS_MAIN, NS_CATEGORY, NS_IMAGE ), $condition ),
				__METHOD__,
    			array_merge(
    				array( 'ORDER BY' => $order, 'GROUP BY' => 'page_id' ),
    				!$cheap ? array( 'LIMIT' => $wgCategoryPagingLimit, 'OFFSET' => $this->offset ) : array()
    			),
				array( 'recentchanges' => array( 'LEFT JOIN', 'rc_id = page_latest' ) )
			);
		}
		
		$titleArray = array();
		while ( $row = $dbr->fetchObject( $res ) ) {
			array_push( $titleArray, Title::newFromRow( $row ) );
		}
		
    	return ( !$cheap ) ? array( 'count' => $count, 'data' => $titleArray ) : $titleArray;	
	}
	
	private function getPagesList() {
		global $wgTitle, $wgUser, $wgCategoryPagingLimit;
		
		$skin = $wgUser->getSkin();
		
		$params = $_GET;
		unset( $params['title'] ); 
		
		if ( !empty( $this->letter ) ) $params['letter'] = $this->letter;
		
		$pageIndex = SpecialPage::getTitleFor( wfMsg( 'pageindex' ) )->equals( $wgTitle );
		
		$arrows = array_fill_keys( array( 'n', 'r' ), array() );
		switch ( $this->order ) {
			case 'n':
				$arrows['n'] = array( 'class' => 'sort-up' ); break;
			case 'nd':
				$arrows['n'] = array( 'class' => 'sort-down' ); break;
			case 'r':
				$arrows['r'] = array( 'class' => 'sort-up' ); break;
			case 'rd':
				$arrows['r'] = array( 'class' => 'sort-down' ); break;
		}
		
		$html =
			'<table class="pages-list" cellspacing="0">' . "\n" .
			'<thead>' .
			'<tr>' .
			'<td>' .
			$skin->link( $wgTitle, wfMsg( 'pageindex_name' ), $arrows['n'],
				array_merge( $params, array( 'order' => ( $this->order == 'n' ) ? 'nd' : 'n' ) )		
			) .
			'</td>' .	
			( $pageIndex ? '<td>' . wfMsg( 'pageindex_parent' ) . '</td>' : '' ) .
			'<td>' .
			$skin->link( $wgTitle, wfMsg( 'pageindex_rc' ), $arrows['r'],
				array_merge( $params, array( 'order' => ( $this->order == 'r' ) ? 'rd' : 'r' ) )
			) .
			'</td>' .
			'<td>' . wfMsg( 'pageindex_rename' ) . '</td>' .
			'<td>' . wfMsg( 'pageindex_edit' ) . '</td>' .
			'<td>' . wfMsg( 'pageindex_move' ) . '</td>' .
			'<td>' . wfMsg( 'pageindex_delete' ) . '</td>' .
			'</tr>' .
			'</thead>' . "\n" .
			'<tbody>';
		
		$gray = true;
		
		foreach ( $this->pages['data'] as $title ) {
		    $canRead = $title->userCan( 'read' );
    		$canEdit = $title->userCan( 'edit' ); 
    		$canMove = $title->userCan( 'move' );
    		$canDelete = $title->userCan( 'delete' );
    		$canRename = $canEdit && $canMove && $canDelete;
		    
		    $revId = $title->getLatestRevID();
			$revision = Revision::newFromId( $revId );
			if ( $revision instanceof Revision ) {
				$user = $skin->revUserLink( $revision );
				$timestamp = wfTimestamp( TS_DB, $revision->getTimestamp() );
			} else {
				$user = $timestamp = '';
			}
			$ns = $title->getNamespace();
			
			// @todo Wciagnac parentFolder w kwerende
			$pId = jQueryCategoryTree::getParentCategoryId( $title, false );
			
			$anchor = $title->getBaseText();
			$anchor = ( mb_strlen( $anchor ) > 75 )
			    ? mb_substr( $anchor, 0, 74 ) . "&hellip;"
			    : $anchor;

			$link = ( $canRead ) ? $skin->link( $title, $anchor ) : $anchor;
			
			$html .=
				Xml::openElement( 'tr', $gray ? array( 'class' => 'gray' ) : null ) . "\n" .
				Xml::tags( 'td', array( 'class' => "name ns{$ns}" ), $link ) . "\n";
				
			if ( $pageIndex ) {
				if ( $pId === 0 || !( ( $parentTitle = Title::newFromID( $pId ) ) instanceof Title ) ) {
					$parentTitle = SpecialPage::getTitleFor( wfMsg( 'pageindex' ) );
					$parentTitleText = wfMsg( 'pageindex_folders' );				
				} else {
					$parentTitleText = $parentTitle->getBaseText();
				}
				
				$html .= Xml::tags( 'td', null, $skin->link( $parentTitle, $parentTitleText, array( 'rel' => 100 ) ) ) . "\n"; 				
			}
			
			$html .= Xml::tags( 'td', array( 'class' => 'rc' ), $user . ' ' . Xml::span( $timestamp, 'ts' ) ) . "\n";
			
			if ( $canRename ) {
				$rel = $title->getArticleID() . '-' . $pId . '-' . $title->getNamespace();
				$html .=
					'<td class="opt">' .
					( in_array( $ns, array( NS_MAIN, NS_CATEGORY, NS_IMAGE ) )
						?	'<a href="#" rel="' . $rel . '" class="rename"></a>' .
							'<a href="#" rel="' . $title->getArticleID() . '" class="rename-save" style="display: none;"></a>' .
							'<a href="#" class="rename-cancel" style="display: none;"></a>'
						: '&nbsp;'
					) .
					'</td>' . "\n";
			} else {
			    $html .= '<td class="opt"></td>';
			}
			if ( $canEdit ) {
				$html .=
					'<td class="opt">' .
					( in_array( $ns, array( NS_MAIN ) )
						? $skin->link( $title, '', array( 'class' => 'edit' ), array( 'action' => 'edit' ) )
						: '&nbsp;'
					) .
					'</td>' . "\n";
			} else {
			    $html .= '<td class="opt"></td>';
			}
			if ( $canMove ) {
				$rel = $title->getArticleID() . '-' . $pId . '-' . $title->getNamespace();
				$html .=
					'<td class="opt">' .
					( in_array( $ns, array( NS_MAIN, NS_CATEGORY, NS_IMAGE ) )
						? '<a href="#" rel="' . $rel . '" class="move"></a>'
						: '&nbsp;'
					) .
					'</td>' . "\n";
			} else {
			    $html .= '<td class="opt"></td>';
			}
			if ( $canDelete ) {
				$html .=
					'<td class="opt">' .
					( in_array( $ns, array( NS_MAIN, NS_CATEGORY, NS_IMAGE ) )
						? $skin->link( $title, '', array( 'class' => 'delete' ), array( 'action' => 'delete' ) )
						: '&nbsp;'
					) .
					'</td>' . "\n";
			} else {
			    $html .= '<td class="opt"></td>';
			}
			$html .= '</tr>' . "\n";
			
			$gray = $gray ? false : true;
		}
		
		$html .=
			'</tbody>' . "\n" .
			'</table>' . "\n" .
			Xml::hidden( 'wpParentCategoryId', $wgTitle->getArticleID(), array( 'id' => 'wpParentCategoryId' ) );
		
		$count = $this->pages['count'];
		$page = $this->page;
		
		if ( ( $count / $wgCategoryPagingLimit ) > 1 ) {
			if ( !empty( $this->order ) ) $params['order'] = $this->order;
			
			$links = array();
			
			$first = 1;
			$last = (int) ceil( $count / $wgCategoryPagingLimit );
			
			$links[] = ( 1 == $page )
				? '<strong>1</strong>'
				: $skin->link( $wgTitle, 1, array(), array_merge( $params, array( 'page' => 1 ) ) );
				
			if ( ( $page - 5 ) > ( $first + 1 ) ) {
				$links[] = '...'; // TODO: &hellip;
			}
			
			$start = max( ( $first + 1 ), ( $page - 5 ) );
			$end = min( ( $last - 1 ), ( $page + 5 ) );
			
			for ( $i = $start; $i <= $end; $i ++ ) {
				$links[] = ( $i == $page )
					? '<strong>' . $i . '</strong>'
					: $skin->link( $wgTitle, $i, array(), array_merge( $params, array( 'page' => $i ) ) );
			}
			
			if ( ( $page + 5 ) < ( $last - 1 ) ) {
				$links[] = '...'; // TODO: &hellip;
			}
			
			$links[] = ( $last == $page )
				? '<strong>' . $last . '</strong>'
				: $skin->link( $wgTitle, $last, array(), array_merge( $params, array( 'page' => $last ) ) );
						
			$html .= '<span class="nav">' . implode( " ", $links ) . '</span>' . "\n"; 
		}
		
		return $html;
	}
	
	public function getPagesHTML() {
	    return (string) $this;
	}
	
	public function __toString() {
		$html = $this->viewCacheGet();
		
		if ( false !== $html ) return $html;
		
		$count = count( $this->getPagesData( true ) );
		
		$html = '<div id="mw-pages">';
		
		if ( $count > 0 ) {
			$html .= $this->getLettersLinks();
		}
		
		$html .= ( count( $this->pages['data'] ) > 0 )
			? $this->getPagesList()
			: Xml::span( ( $count > 0 ) ? wfMsg( 'pageindex_noresult' ) : wfMsg( 'pageindex_nopages' ), 'msg' );
			   
		$html .= '</div>' . "\n";
		
		$this->viewCacheSet( $html );

		return $html;
	}
}
