<?php

class PdfSupport {
	private $author;
    
	private $title;
	private $content;
	
	private $keywords = '';
		
	public function __construct() {
	    global $wgUser;
	    
	    $this->author = $wgUser->getRealName();
	    if ( empty( $this->author ) ) {
	        $wgUser->getName();
	    }
	}
	
	public function setContentFromArticle( Article $article ) {
		global $wgUser, $wgParser;
		
		$title = $article->getTitle();
		
		$this->title = $title->getBaseText();

		$parserOptions = ParserOptions::newFromUser( $wgUser );
		$parserOptions->setEditSection( false );
		$parserOptions->setTidy( true );

		$parserOutput = $wgParser->parse(
			"{$article->fetchContent()}__NOTOC__",
		    $title, $parserOptions, true, true
		);
		
		$this->content = $parserOutput->getText();
		
		if ( class_exists( 'Tags' ) ) {
		    $this->keywords = implode( ', ', Tags::getTags( $title ) );
		}
	}
	
	public function setContent( $content, $title = 'Document', $keywords = '' ) {
	    $this->title = ( mb_strlen( $title ) > 50 )
			    ? mb_substr( $title, 0, 48 ) . " (...)"
			    : $title;
	    
	    $this->content = $content;
	    
	    $this->keywords = $keywords;
	}
	
	private function fixContent() {
	    global $wgServer;
	    
        $this->content = str_replace(
        	'href="/index.php?title=',
        	'href="' . $wgServer . '/index.php?title=',
            $this->content
        );
	}
	
	public function output() {
	    $this->fixContent();
	    
        require_once( dirname( __FILE__ ) . '/tcpdf/config/lang/eng.php' );
	    
	    ob_start();
        
        $pdf = new PdfSupportBackend( PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true ); 
        
        $pdf->SetCreator( PDF_CREATOR );
        
        $pdf->SetAuthor( $this->author );
        $pdf->SetTitle( $this->title );
        $pdf->SetKeywords( $this->keywords );
        
        $pdf->SetHeaderData(
        	'titlebar_logo.png', PDF_HEADER_LOGO_WIDTH,
            $this->title, $this->author
        );
        
        $pdf->setHeaderFont( array( 'dejavusans', '', PDF_FONT_SIZE_MAIN ) );
        $pdf->setFooterFont( array( 'dejavusans', '', PDF_FONT_SIZE_DATA ) );
        
        $pdf->SetMargins( PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT );
        $pdf->SetHeaderMargin( PDF_MARGIN_HEADER );
        $pdf->SetFooterMargin( PDF_MARGIN_FOOTER );
        
        $pdf->SetAutoPageBreak( true, PDF_MARGIN_BOTTOM );
        
        $pdf->setImageScale( PDF_IMAGE_SCALE_RATIO ); 
        
        $pdf->setLanguageArray( $l ); 
        
        $pdf->AliasNbPages();
        
        $pdf->AddPage();
        
        $pdf->SetFont( 'dejavusans', '', 10 );
        
        $pdf->writeHTML( $this->content );
        
        $pdf->Output( "{$this->title}.pdf", 'D' );
        
        ob_end_flush();
        
		exit;
	}
}

class PdfSupportBackend extends TCPDF {
	public function Header() {
	    $logoPath = dirname( __FILE__ ) . '/../../skins/bizneswiki/';
	    
		$ormargins = $this->getOriginalMargins();
		$headerfont = $this->getHeaderFont();
		$headerdata = $this->getHeaderData();
		if ( ( $headerdata['logo'] ) AND ( $headerdata['logo'] != K_BLANK_IMAGE ) ) {
			$this->Image( $logoPath . $headerdata['logo'], $this->GetX(), $this->getHeaderMargin(), $headerdata['logo_width'] );
			$imgy = $this->getImageRBY();
		} else {
			$imgy = $this->GetY();
		}
		$cell_height = round( ( $this->getCellHeightRatio() * $headerfont[2] ) / $this->getScaleFactor(), 2 );
		// set starting margin for text data cell
		if ( $this->getRTL() ) {
			$header_x = $ormargins['right'] + ( $headerdata['logo_width'] * 1.1 );
		} else {
			$header_x = $ormargins['left'] + ( $headerdata['logo_width'] * 1.1 );
		}
		$this->SetTextColor( 0, 0, 0 );
		// header title
		$this->SetFont( $headerfont[0], 'B', $headerfont[2] + 1 );
		$this->SetX( $header_x );			
		$this->Cell( 0, $cell_height, $headerdata['title'], 0, 1, '', 0, '', 0 );
		// header string
		$this->SetFont( $headerfont[0], $headerfont[1], $headerfont[2] );
		$this->SetX( $header_x );
		$this->MultiCell( 0, $cell_height, $headerdata['string'], 0, '', 0, 1, '', '', true, 0, false );
		// print an ending header line
		$this->SetLineStyle( array( 'width' => 0.85 / $this->getScaleFactor(), 'cap' => 'butt', 'join' => 'miter', 'dash' => 0, 'color' => array( 0, 0, 0 ) ) );
		$this->SetY( 1 + max( $imgy, $this->GetY() ) );
		if ( $this->getRTL() ) {
			$this->SetX( $ormargins['right'] );
		} else {
			$this->SetX( $ormargins['left'] );
		}
		$this->Cell( 0, 0, '', 'T', 0, 'C' );
	}
}
