<?php

class SpecialUpdateProfile_BusinessWiki extends SpecialUpdateProfile {
	public function __construct() {
		parent::__construct();
	}

	public function execute( $section ) {
		global $wgUser, $wgOut, $wgRequest, $wgSiteView, $IP, $wgUserProfileScripts, $wgStyleVersion, $wgUpdateProfileInRecentChanges, $wgSupressPageTitle, $wgScriptPath;
		$wgSupressPageTitle = true;

		$wgOut->setHTMLTitle( wfMsg('pagetitle', wfMsg('edit-profile-title')));

		if( !$wgUser->isLoggedIn() ) {
			$wgOut->setPagetitle( wfMsgForContent( 'user-profile-update-notloggedin-title' ) );
			$wgOut->addHTML( wfMsgForContent(  'user-profile-update-notloggedin-text',  Title::makeTitle(NS_SPECIAL, "UserLogin" . $which)->escapeFullUrl(), Title::makeTitle(NS_SPECIAL, "UserRegister" . $which)->escapeFullUrl() ) );
			return;
		}

		if( $wgUser->isBlocked() ){
			$wgOut->blockedPage( false );
			return false;
		}
		
		$wgOut->setPageTitle( wfMsg( 'edit-profile-title' ) );
		
		$wgOut->addExtensionStyle( "{$wgScriptPath}/extensions/SocialProfile/BusinessWiki/UserProfile/UserProfile.css" );
		$wgOut->addScript("<script type=\"text/javascript\" src=\"{$wgUserProfileScripts}/UpdateProfile.js\"></script>\n");
		
 		if($wgRequest->wasPosted()){

 		    if(!$section)$section="basic";

 		    switch($section){
				case "basic":
					$this->saveProfileBasic();
					$this->saveWikiaSettings_basic();
					break;
				case "contact":
					$this->saveProfileCustom();
					break;
			}

			UserProfile::clearCache( $wgUser->getID() );

			$log = new LogPage( wfMsgForContent( 'user-profile-update-profile' ) );
			if( ! $wgUpdateProfileInRecentChanges ){
				$log->updateRecentChanges = false;
			}
			$log->addEntry( wfMsgForContent( 'user-profile-update-profile' ), $wgUser->getUserPage(), wfMsgForContent( 'user-profile-update-log-section' ) . " '{$section}'" );
			$title = Title::makeTitle( NS_USER, $wgUser->getName() );
			$success =
			    '<div class="successbox">' .
			    '<strong>' . wfMsg( 'user-profile-update-saved' ) . '.</strong>' .
			    ' <a href="' . $title->getLocalURL() . '">' . wfMsg( 'user-profile-update-saved-see' ) . '</a>' .
			    '</div>';
			$wgOut->addHTML( $success );

			//create user page if not exists
			$article = new Article( $title );
			if( !$article->exists() ){
				$article->doEdit( "", "create user page", EDIT_SUPPRESS_RC );
			}
		}
		
		$wgOut->addHTML( Xml::openElement( 'div', array( 'id' => 'profile', 'class' => 'ui-tabs ui-widget-content ui-corner-all' ) ) );
		
		if ( empty( $section ) ) {
		    $section = 'basic';
		}

		switch ( $section ) {
		    case 'basic':
				$tab = 0;
		        $panel .= $this->displayBasicForm();
				break;
			case 'contact':
				$tab = 1;
			    $panel .= $this->displayContactForm();
				break;
		}
		
		$out = UserProfile_BusinessWiki::getEditProfileNav( $section, $tab );
		$out .= Xml::openElement( 'div', array( 'id' => 'mw-profile-tab-' . $tab ) );
        $out .= $panel;
        $out .= Xml::closeElement( 'div' );
		
		$script = <<<HEREDOC
<script type="text/javascript">
$(document).ready(function() {
	$('#mw-profile').tabs({
		selected: $tab,
		select: function(e, ui) {
			location.href = $(ui.tab).attr('rel');
			return false;
		}
	});
});
</script>		
HEREDOC;

		$wgOut->addHTML( $script );
		
		$wgOut->addHTML( $out );

        $wgOut->addHTML( Xml::closeElement( 'div' ) );
	}
	
	public function displayBasicForm(){
		global $wgUser;

		$template = new UserProfileBasicFormTemplate();
		
		$template->set( 'realName', $wgUser->getRealName() );
		$template->set( 'email', $wgUser->getEmail() );
		$template->set( 'emailAuth', $wgUser->isEmailConfirmed() );
		
		$dbr =& wfGetDB( DB_MASTER );
		$row = $dbr->selectRow(
			'user_profile',
			array(
				'up_location_city', 'up_location_state', 'up_location_country',
				'up_about', 'up_occupation', 'up_websites'
			),
            array( 'up_user_id' => $wgUser->getID() ),
            __METHOD__
        );

        if ( false !== $row ) {
			$template->set( 'locationCity', $row->up_location_city );
			$template->set( 'locationState', $row->up_location_state );
			$template->set(
				'locationCountry',
			    !empty( $row->up_location_country )
			        ? $row->up_location_country
			        : wfMsg( 'user-profile-default-country' )
			);
			$template->set( 'about', $row->up_about );
    		$template->set( 'occupation', $row->up_occupation );
    		$template->set( 'websites', $row->up_websites );
		}

		$countries = array("Afghanistan","Albania","Algeria","American Samoa","Andorra","Angola","Anguilla","Antarctica","Antigua and Barbuda","Argentina","Armenia","Aruba","Australia","Austria","Azerbaijan","Bahamas","Bahrain","Bangladesh","Barbados","Belarus","Belgium","Belize","Benin","Bermuda","Bhutan","Bolivia","Bosnia and Herzegovina","Botswana","Bouvet Island","Brazil","British Indian Ocean Territory","Brunei Darussalam","Bulgaria","Burkina Faso","Burundi","Cambodia","Cameroon","Canada","Cape Verde","Cayman Islands","Central African Republic","Chad","Chile","China","Christmas Island","Cocos Islands","Colombia","Comoros","Congo","Congo, Democratic Republic of the","Cook Islands","Costa Rica","Cote d'Ivoire","Croatia","Cuba","Cyprus","Czech Republic","Denmark","Djibouti","Dominica","Dominican Republic","Ecuador","Egypt","El Salvador","Equatorial Guinea","Eritrea","Estonia","Ethiopia","Falkland Islands","Faroe Islands","Fiji","Finland","France","French Guiana","French Polynesia","Gabon","Gambia","Georgia","Germany","Ghana","Gibraltar","Greece","Greenland","Grenada","Guadeloupe","Guam","Guatemala","Guinea","Guinea-Bissau","Guyana","Haiti","Heard Island and McDonald Islands","Honduras","Hong Kong","Hungary","Iceland","India","Indonesia","Iran","Iraq","Ireland","Israel","Italy","Jamaica","Japan","Jordan","Kazakhstan","Kenya","Kiribati","Kuwait","Kyrgyzstan","Laos","Latvia","Lebanon","Lesotho","Liberia","Libya","Liechtenstein","Lithuania","Luxembourg","Macao","Macedonia","Madagascar","Malawi","Malaysia","Maldives","Mali","Malta","Marshall Islands","Martinique","Mauritania","Mauritius","Mayotte","Mexico","Micronesia","Moldova","Monaco","Mongolia","Montserrat","Morocco","Mozambique","Myanmar","Namibia","Nauru","Nepal","Netherlands","Netherlands Antilles","New Caledonia","New Zealand","Nicaragua","Niger","Nigeria","Norfolk Island","North Korea","Norway","Oman","Pakistan","Palau","Palestinian Territory","Panama","Papua New Guinea","Paraguay","Peru","Philippines","Pitcairn","Poland","Portugal","Puerto Rico","Qatar","Romania","Russian Federation","Rwanda","Saint Helena","Saint Kitts and Nevis","Saint Lucia","Saint Pierre and Miquelon","Saint Vincent and the Grenadines","Samoa","San Marino","Sao Tome and Principe","Saudi Arabia","Senegal","Serbia and Montenegro","Seychelles","Sierra Leone","Singapore","Slovakia","Slovenia","Solomon Islands","Somalia","South Africa","South Georgia","South Korea","Spain","Sri Lanka","Sudan","Suriname","Svalbard and Jan Mayen","Swaziland","Sweden","Switzerland","Syrian Arab Republic","Taiwan","Tajikistan","Tanzania","Thailand","Timor-Leste","Togo","Tokelau","Tonga","Trinidad and Tobago","Tunisia","Turkey","Turkmenistan","Tuvalu","Uganda","Ukraine","United Arab Emirates","United Kingdom","United States","United States Minor Outlying Islands","Uruguay","Uzbekistan","Vanuatu","Vatican City","Venezuela","Vietnam","Virgin Islands, British","Virgin Islands, U.S.","Wallis and Futuna","Western Sahara","Yemen","Zambia","Zimbabwe");
		
		$template->set( 'countries', $countries );
		
		return $template->fetch();
	}
	
	public function displayContactForm(){
		global $wgRequest, $wgSiteView, $wgUser, $wgOut, $wgDBprefix;
		
		$template = new UserProfileContactFormTemplate();
		
		$dbr =& wfGetDB( DB_SLAVE );
		$row = $dbr->selectRow(
			'user_profile',
			array( 'up_custom_1', 'up_custom_2', 'up_custom_3' ),
            array( 'up_user_id' => $wgUser->getID() ),
            __METHOD__
        );
		
		$template->set( 'email', $wgUser->getEmail() );
		$template->set( 'phones', $row->up_custom_1 );
		$template->set( 'ims', $row->up_custom_2 );
		$template->set( 'others', $row->up_custom_3 );
		
		return $template->fetch();
	}
	
	public function saveWikiaSettings_basic(){
		global $wgUser, $wgRequest, $wgEmailAuthentication;

		$wgUser->setRealName( $wgRequest->getVal("real_name") );
		
		$newEmail = $wgRequest->getVal("email");
		if (!empty($newEmail) && $newEmail != $wgUser->getEmail()) {
    		$wgUser->setEmail( $newEmail );

			$wgUser->mEmailAuthenticated = null;

    		if ($wgEmailAuthentication) {
    			$result = $wgUser->sendConfirmationMail();
    			if( WikiError::isError( $result ) ) {
    				$error = wfMsg( 'mailerror', htmlspecialchars( $result->getMessage() ) );
    			} else {
    				$error = wfMsg( 'eauthentsent', $wgUser->getName() );
    			}
    		}
		}
		
		$wgUser->saveSettings();
	}
}
