<?php

/**
 * MediaWiki Tags extension
 */

if( !defined( 'MEDIAWIKI' ) ) {
    echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
    die( 1 );
}

$wgExtensionCredits['parserhook'][] = array(
	'name' => 'Tags',
	'author' => 'Paweł Szreder',
	'version'        => '0.1',
	'description' => 'Adds tags support',
	'descriptionmsg' => 'tags_desc',
	'url' => 'http://www.bizneswiki.pl'
);

$wgExtensionCredits['specialpage'][] = array(
	'name' => 'Tags',
	'author' => 'Paweł Szreder',
	'version'        => '0.1',
	'description' => 'Adds tags special page',
	'descriptionmsg' => 'tags_desc',
	'url' => 'http://www.bizneswiki.pl'
);

$wgHooks['UnknownAction'][] = 'Tags::onUnknownAction';

$wgExtensionFunctions[] = 'wfTags';
$wgHooks['BeforePageDisplay'][] = 'Tags::onBeforePageDisplay';
$wgHooks['ArticleSaveComplete'][] = 'Tags::onArticleSaveComplete';
$wgHooks['ArticleDeleteComplete'][] = 'wfTagsArticleDeleteComplete';
$wgHooks['LanguageGetSpecialPageAliases'][] = 'wfTagsLocalizedSpecialPageName';
$wgHooks['LanguageGetMagic'][] = 'wfTagsMagic';

$wgAjaxExportList[] = 'Tags::ajaxGetTags';
$wgAjaxExportList[] = 'Tags::ajaxGetTagsHTML';
$wgAjaxExportList[] = 'Tags::ajaxSetTags';

$dir = dirname(__FILE__) . '/';
require_once $dir . 'Tags.i18n.php';

$wgAutoloadClasses['SpecialTags'] = $dir . 'SpecialTags.php';
$wgSpecialPages['Tags'] = 'SpecialTags';

function wfTags() {
	global $wgParser, $wgTagsMessages, $wgMessageCache;

	$wgParser->setHook( 'Tags', 'Tags' );

	$wgParser->setFunctionHook( 'tagcloud', 'wfTagCloudRender' );
	$wgParser->setFunctionHook( 'tagpages', 'wfTagsRender' );

	foreach ( $wgTagsMessages as $lang => $messages) {
		$wgMessageCache->addMessages( $messages, $lang );
	}
}

function wfTagsArticleDeleteComplete( &$article, &$user, $reason, $id ) {
    $title = new Title();
    $title->mArticleID = $id;
    
	return Tags::delTags( $title );
}

function wfTagsLocalizedSpecialPageName( &$specialPageArray, $code ) {
	$text = wfMsg( 'tags' );
	$title = Title::newFromText($text);
	$specialPageArray['Tags'][] = $title->getDBKey();

	return true;
}

function wfTagsMagic( &$magicWords, $langCode ) {
	$magicWords['tagcloud'] = array( 0, 'tagcloud' );
	$magicWords['tagpages'] = array( 0, 'tagpages' );

	return true;
}

function wfTagCloudRender( &$parser, $tag = '*' ) {
	return array( Tags::getCloudHTML(), 'isHTML' => true );
}

function wfTagsRender( &$parser, $tag = '*' ) {
	return array( Tags::getPagesHTML( $tag ), 'isHTML' => true );
}

class Tags {
	private $title;
	
	static $tag;

	static $minimalSize = 90;
	static $increaseFactor = 75;

	function __construct( Title &$title ) {
		$this->title =& $title;
	}
	
	public static function onBeforePageDisplay( &$out ) {
		global $wgStylePath, $wgDefaultSkin;
		
		$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/Tags.css" );
		$out->addScriptFile( "/extensions/Tags/Tags.js" );
			
		return true;
	}
	
	public static function onArticleSaveComplete( Article &$article ) {
		global $wgRequest;
		
		$tags = $wgRequest->getVal( 'wpPageTags', '' );
		
		if ( !empty( $tags ) ) {
			self::setTags( $article->getTitle(), $tags );
		}
		
		return true;
	}

	public static function onUnknownAction( $action ) {
		global $wgOut, $wgRequest;

		if ( $action == 'tagsLike' ) {
			$wgOut->disable();
			wfResetOutputBuffers();

			$query = $_GET;
			if(isset($query['q'])){
				echo self::getTagsLike($query['q']);
			}
		}

		return true;
	}

	/**
	 * @param Title $title
	 * @return string
	 */
	public static function getTags( Title &$title ) {
		$dbr =& wfGetDB( DB_SLAVE );
		$dbw =& wfGetDB( DB_MASTER );

		$id = $title->getArticleID();
		
		if ( empty( $id ) ) {
			return array();
		}

		$res = $dbr->select(
			'pagetags',
			'page_tag',
			array( 'page_id' => $id ),
			__METHOD__
		);

		$tags = array();
		foreach ( $res as $row ) {
			$tag = trim( $row->page_tag );
			if ( !empty( $tag ) ) {
				$tags[] = $tag;
			} else {
				$dbw->delete( 'pagetags', array( 'page_id' => $id, 'page_tag' => $tag ) );
			}
		}

		$dbr->freeResult( $res );

		return $tags;
	}
	
	public static function ajaxGetTags( $pageId ) {
		$title = Title::newFromID( $pageId );
		
		return implode( ', ', self::getTags( $title ) );
	}

	/*
	 * @param Title $title
	 * @return array
	 * Pobiera tagi do autocomplete
	 */
	public static function getTagsLike( $part ) {
		$dbr =& wfGetDB( DB_SLAVE );

		$res = $dbr->query(" SELECT * FROM `bizneswiki_pagetags` WHERE `page_tag` LIKE '{$part}%' GROUP BY `page_tag` LIMIT 0 , 30 ");

		$tags = array();
		foreach ( $res as $row ) {
			$tag = trim( $row->page_tag );
			if ( !empty( $tag ) ) {
				$tags[]['tag'] = $tag;
			}
		}

		$dbr->freeResult( $res );

		return json_encode( $tags );
	}

	/**
	 * @param Title $title
	 * @param string $tags
	 */
	public static function setTags( Title &$title, $tags ) {
		$allowed = "/[^a-zA-Z0-9ąĄćĆęĘłŁńŃóÓśŚźŹżŻ\\040\,]/i";

  		$tags = preg_replace($allowed,' ',$tags);

		$tags = mb_strtolower($tags);
		$tags = preg_replace( '/\s\s+/', ' ', $tags );
		$tags = explode( ',', $tags );
		array_walk( $tags, create_function( '&$value', '$value = trim( $value );') );
		$tags = array_values( $tags );
		$tags = array_unique( $tags ); 

		$dbw =& wfGetDB( DB_MASTER );

		self::delTags( $title );

		$dbw->begin();
		foreach ($tags as &$tag) {
			$tag = trim( $tag );

			if ( empty( $tag ) ) continue;

			$dbw->insert( 'pagetags', array(
				'page_id' 			=> $title->getArticleId(),
				'page_namespace'	=> $title->getNamespace(),
				'page_tag'			=> $tag
			) );
		}
		$dbw->commit();
		
		PageIndexView::onArticleChangeComplete();
	}
	
	public static function ajaxSetTags( $pageId, $tags ) {
		global $wgUser;
		
		$title = Title::newFromID( $pageId );
		
		self::setTags( $title, $tags );
		
		$tags = self::getTags( $title );
		
		$skin = $wgUser->getSkin();
		if ( !empty( $tags ) ) {
			foreach ( $tags as &$tag ) {
				$href = SpecialPage::getTitleFor( 'Tags' )->getLocalURL() . "&tag={$tag}";
				$tag = Xml::element( 'a', array( 'href' => $href ), $tag );
			}
			$value = implode( ', ', $tags );
		} else {
			$value = Xml::escapeTagsOnly( wfMsg( 'tags_empty' ) );
		}
			
		return $value;
	}

	/**
	 * @param Title $title
	 * @return bool
	 */
	public static function delTags( Title &$title ) {
		$dbw =& wfGetDB( DB_MASTER );

		$dbw->begin();
		$dbw->delete( 'pagetags', array( 'page_id' => $title->getArticleID() ) );
		$dbw->commit();

		return true;
	}

	/**
	 * @param integer $min
	 * @param integer $max
	 * @return array
	 */
	public static function getData( &$min, &$max, $limit ) {
		if ( is_null( $min ) ) $min = 1000000;
		if ( is_null( $max ) ) $max = -1;

		$dbr =& wfGetDB( DB_SLAVE );

		$res = $dbr->select(
			'pagetags',
			array( 'page_tag', 'COUNT(*) AS count'),
			null,
			__METHOD__,
			array( 'GROUP BY' => 'page_tag', 'HAVING' => 'count >= 0', 'ORDER BY' => 'count DESC, page_tag ASC', 'LIMIT' => $limit )
		);

		$tags = array();
		foreach ( $res as $row ) {
			$tags[$row->page_tag] = $row->count;
			$min = ( $row->count < $min ) ? $row->count : $min;
			$max = ( $row->count > $max ) ? $row->count : $max;
		}

		$dbr->freeResult( $res );
		
		ksort( $tags );

		return $tags;
	}
	
	public static function ajaxGetTagsHTML( $pageId ) {	    
		$title = Title::newFromID( $pageId );
		
		$tags = self::getTags( $title );
		
		if ( !empty( $tags ) ) {
		    global $wgUser;
		    
		    $skin = $wgUser->getSkin();
		    
		    $title = SpecialPage::getTitleFor( 'Tags' );
		    
			foreach ( $tags as &$tag ) {
				$tag = $skin->link( $title, $tag, array(), array( 'tag' => $tag ) );
			}
			
			return implode( ', ', $tags );
		} else {
			return Xml::escapeTagsOnly( wfMsg( 'tags_empty' ) );
		}		
	}

	/**
	 * @return string
	 */
	public static function getCloudHTML( $big = false ) {
	    global $wgUser;
	    
	    $skin = $wgUser->getSkin();
	    
		$limit = ( $big ) ? null : 20;
		
		$tags = self::getData( $min, $max, $limit );
		$title = SpecialPage::getTitleFor( 'Tags' );

		$style = 'text-align: center;';
		if ( $big ) $style .= '  font-size: 2em; line-height: 125%;';

		$out = array();
		$out[] = Xml::openElement( 'div', array( 'style' => $style ) );
		foreach ( $tags as $tag => $count ) {
			$textSize = self::$minimalSize + ( self::$increaseFactor * ( $count ) ) / ( $max );
			$out[] = $skin->link(
			    $title,
			    $tag,
			    array( 'style' => "font-size: {$textSize}%;" ),
			    array( 'tag' => $tag )
			);
			$out[] = ' ';
		}
       	$out[] = Xml::closeElement( 'div' );
		if ( !empty( $tags ) && !$big ) {
	       	$out[] = $skin->link(
	       	    $title,
	       	    wfMsg( 'tags_more' ),
	       	    array( 'class' => 'more' )
	       	); 
		} elseif ( empty( $tags ) ) {
			$out[] = wfMsg('tags_notags');
		}

       	return implode( "\n", $out );
	}
	
	/**
	 * @todo to ExtendedQuickTemplate 
	 * @return string
	 */
	public function __toString() {
		$out = '<div id="mw-page-tags">';
		$out .= '<span class="inner">';
		$out .= Xml::openElement( 'span', array( 'style' => 'float: left; display: inline; width: 80%;' ) );
		$out .= '<h2>' . wfMsg( 'Tags' ) . '</h2>';

		$tags = self::getTags( $this->title );
		if ( !empty( $tags ) ) {
		    global $wgUser;
		    
		    $skin = $wgUser->getSkin();
		    
		    $title = SpecialPage::getTitleFor( 'Tags' );
		    
			foreach ( $tags as &$tag ) {
				$tag = $skin->link( $title, $tag, array(), array( 'tag' => $tag ) );
			}
			$value = implode( ', ', $tags );
			$notags = false;
		} else {
			$value = Xml::escapeTagsOnly( wfMsg( 'tags_empty' ) );
			$notags = true;
		}
		
		$out .= Xml::openElement( 'span', array_merge(
			array( 'class' => 'tags', 'style' => 'margin-left: 10px;' ),
			$notags ? array( 'rel' => 'notags' ) : array()
		) );
		$out .= $value;
		$out .= Xml::closeElement( 'span' );
		$out .= Xml::closeElement( 'span' );
		$out .= Xml::element( 'a', array( 'class' => 'tags-edit', 'style' => 'float: right; display: inline; cursor: pointer;' ), ucfirst( wfMsg( 'edit' ) ) );
		$out .= Xml::element( 'a', array( 'class' => 'tags-save', 'style' => 'float: right; display: none; cursor: pointer;' ), ucfirst( wfMsg( 'saveprefs' ) ) );
		$out .= Xml::openElement( 'div', array( 'class' => 'visualClear' ) ) . Xml::closeElement( 'div' );
		$out .= Xml::closeElement( 'span' );
		$out .= Xml::closeElement( 'div' );

		return $out;
	}
}

class TagsView extends PageIndexView {
	public function __construct() {
		parent::__construct();
	}
	
	public function viewCacheKey( $type ) {
		global $wgUser, $wgTitle, $wgMemc, $wgDBname, $wgCategoryPagingLimit;
		
		$userId = $wgUser->getId();
		$letter = !empty( $this->letter ) ? $this->letter : 'any';
		$order = $this->order;
		$page = $this->page;
		
		$tag = Tags::$tag;
		
		switch ( $type ) {
			case PAGEINDEX_VIEW_CACHE:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:{$userId}:tags:{$tag}:{$letter}:{$order}:{$page}";
			case PAGEINDEX_VIEW_CACHE_ALL:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:all:tags:{$tag}";
			case PAGEINDEX_VIEW_CACHE_USER:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:user:{$userId}:tags:{$tag}";
			case PAGEINDEX_VIEW_CACHE_EVERYTHING:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		}
	}
	
	public function getPagesData( $cheap = false ) {
		global $wgTitle, $wgUser, $wgCategoryPagingLimit;
		
		$userId = $wgUser->getId();
		
		$dbr =& wfGetDB( DB_SLAVE );
		
		if ( !$cheap && !empty( $this->letter ) ) {
			$condition = "page_title LIKE {$dbr->addQuotes( $this->letter . '%' )}";
		} else {
			$condition = '1 = 1';
		}
		
		switch ( $this->order ) {
			case 'n':
				$order = 'page_title'; break;
			case 'nd':
				$order = 'page_title DESC'; break;
			case 'r':
				$order = 'rc_timestamp'; break;
			case 'rd':
				$order = 'rc_timestamp DESC'; break;				
			default:
				$order = 'page_title';	
		}
		
		$array = array();
		$res = $dbr->select(
			'pagetags',
			'page_id',
			array( 'page_tag' => Tags::$tag ),
			__METHOD__,
			array( 'GROUP BY' => 'page_id' )
		);
		while ( $row = $dbr->fetchObject( $res ) ) {
			array_push( $array, $row->page_id );
		}
		
		$titleArray = array();

		if ( !empty( $array ) ) {
			if ( !$cheap ) {
				$count = $dbr->selectField(
		    		array( 'page' ),
		    		'COUNT(page_id)',
		    		array( 'page_id' => $array,	$condition ),
		    		__METHOD__		
				);
			}
			
			$res = $dbr->select(
				array( 'page', 'recentchanges' ),
				array( 'page_id', 'page_namespace', 'page_title' ),
				array( 'page_id' => $array, $condition ),
				__METHOD__,
	    		array_merge(
	    			array( 'ORDER BY' => $order ),
	    			!$forLetters ? array( 'LIMIT' => $wgCategoryPagingLimit, 'OFFSET' => $this->offset ) : array()
	    		),
				array( 'recentchanges' => array( 'LEFT JOIN', 'rc_id = page_latest' ) )
			);
					
			while ( $row = $dbr->fetchObject( $res ) ) {
				array_push( $titleArray, Title::newFromRow( $row ) );
			}
		}
		
    	return ( !$cheap ) ? array( 'count' => $count, 'data' => $titleArray ) : $titleArray;	
	}	
}