<?php

/**
 * MediaWiki Toolbar extension
 */

if ( ! defined( 'MEDIAWIKI' ) )
	die( 1 );

define( 'TB_RULE_LOGGEDIN', 0 );
define( 'TB_RULE_NAMESPACE', 1 );
define( 'TB_RULE_ACTION', 2 );
define( 'TB_RULE_TITLE', 3 );
define( 'TB_RULE_CUSTOM', 4 );
define( 'TB_RULE_USERCAN', 5 );

define( 'TB_RULE_IN', true );
define( 'TB_RULE_NOTIN', false );

$wgAutoloadClasses['ToolbarTemplate'] = dirname(__FILE__) . '/ToolbarTemplate.php';

$wgHooks['BeforePageDisplay'][] = 'Toolbar::onBeforePageDisplay';
$wgHooks['SkinAfterBottomScripts'][] = 'Toolbar::onSkinAfterBottomScripts';

$wgToolbarItems = array();
$wgToolbarItems[] = 'Edit';
// $wgToolbarItems[] = 'AddComment';
// $wgToolbarItems[] = 'AddAttachment';
$wgToolbarItems[] = 'History';
$wgToolbarItems[] = 'Print';
$wgToolbarItems[] = 'Watch';
$wgToolbarItems[] = 'Unwatch';
$wgToolbarItems[] = 'Save';
$wgToolbarItems[] = 'Move';
$wgToolbarItems[] = 'SelectFolder';
// $wgToolbarItems[] = 'Template';
$wgToolbarItems[] = 'Attachments';
// $wgToolbarItems[] = 'AddCategory';
$wgToolbarItems[] = 'AddPage';
$wgToolbarItems[] = 'Cancel';
$wgToolbarItems[] = 'Article';
$wgToolbarItems[] = 'Security';
$wgToolbarItems[] = 'Watchlist';
$wgToolbarItems[] = 'ManageWatchlist';
$wgToolbarItems[] = 'Profile';
$wgToolbarItems[] = 'Upload';
if ( defined( 'LIVEBLEND' ) ) {
$wgToolbarItems[] = 'LiveBlendNewQuestion';
$wgToolbarItems[] = 'LiveBlendMain';
$wgToolbarItems[] = 'LiveBlendMy';
//$wgToolbarItems[] = 'LiveBlendMethods';
// $wgToolbarItems[] = 'LiveBlendAbout';
$wgToolbarItems[] = 'LiveBlendMyActivities';
$wgToolbarItems[] = 'LiveBlendHallOfFame';
$wgToolbarItems[] = 'LiveBlendManageGroups';
}
$wgToolbarItems[] = 'AddDashboardGadget';

//$wgToolbarItems[] = 'DashboardOldStyle';
if ( defined( 'LIVEBLEND' ) ) {
$wgToolbarItems[] = 'LiveBlendInstructions';
}
//$wgToolbarItems[] = 'DashboardNewStyle';

BusinessWiki_Dialogs::prepare( 'Folders' );
BusinessWiki_Dialogs::prepare( 'NewPage' );
BusinessWiki_Dialogs::prepare( 'Move' );
BusinessWiki_Dialogs::prepare( 'PageRestrictions' );
BusinessWiki_Dialogs::prepare( 'SelectFolder' );

class Toolbar {
    public static function onBeforePageDisplay( OutputPage &$out )
    {
        global $wgToolbarItems, $wgStylePath, $wgDefaultSkin;
        
    	if ( !empty( $wgToolbarItems ) ) {
			$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/Toolbar.css" );
			$out->addScriptFile( "/extensions/Toolbar/Toolbar.js" );
		}
		
		return true;
    }
    
	/**
	 * Hook function, prepares CSS ans JS for the entire toolbar
	 *
	 * @param string $bottomScriptText
	 * @return boolean
	 */
	public static function onSkinAfterBottomScripts($skin, &$bottomScriptText) {
		global $wgToolbarItems, $wgJsMimeType;

		foreach ( $wgToolbarItems as $key => $item ) {
			$toolbarItem = new ToolbarItem( $item );
			if ( $toolbarItem->isVisible() ) {
				$wgToolbarItems[$key] = $toolbarItem;
			} else {
				unset( $wgToolbarItems[$key] );
			}
		}
		
		if ( !empty( $wgToolbarItems ) ) {
			$script = array();
			$script[] = 'addOnloadHook(function() {';
			foreach ( $wgToolbarItems as $item ) {
				$script[] = "$('#item-{$item->name}').click(function() { {$item->action} });";
			}
			$script[] = '});';
			
			$bottomScriptText = "<script type=\"{$wgJsMimeType}\">/*<![CDATA[*/\n" . implode( "\n", $script ) . "\n/*]]>*/</script>" . $bottomScriptText;
		}

		return true;
	}
	
	/**
	 * @return string
	 */
	public function __toString() {
		global $wgToolbarItems, $wgLang, $wgStylePath, $wgDefaultSkin;
		
		$items = array();
		if ( !empty( $wgToolbarItems ) ) {
			foreach ( $wgToolbarItems as $item ) {
				$items[] = array(
					'name' => $item->name,
					'image' => $item->image,
					'title' => $item->label[$wgLang->getCode()]
				);
			}
		}
		
		$background = "{$wgStylePath}/{$wgDefaultSkin}/Toolbar/blank.gif";
		
		$template = new ToolbarTemplate();
		$template->set( 'items', $items );
		$template->set( 'blank', $background );
		
		return $template->fetch();
	}
}

class ToolbarItem {	
	public $name;
	
	public $label = array();
	public $title = array();
	public $image;
	public $action;
	public $rules;

	private $variables = array( 'label', 'title', 'action', 'rules' );
	
	public function __construct( $name ) {
		global $wgStylePath, $wgDefaultSkin;
		
		$this->name = $name;
		
		$dir = dirname(__FILE__);
		$file = "{$dir}/items/{$name}.php";
		
		if ( !file_exists( $file ) ) {
			throw new MWException( "Toolbar item \"{$name}\" not exists!" );
		}
		
		$image = "{$wgStylePath}/{$wgDefaultSkin}/Toolbar/{$name}.png"; // TODO: Powiedzmy sobie szczerze, to nie jest gornolotne.. :-/
		$this->image = $image;
		
		require_once( "{$dir}/items/{$name}.php" );
		$className = "ToolbarItem{$name}";
		$item = new $className(); 

		foreach (  $this->variables as $var ) {
			if ( is_null( $item->{$var} ) ) {
				throw new MWException( "Toolbar item \"{$name}\" has not \"{$var}\" defined!" );
			} else {
				$this->{$var} = $item->{$var};
			}
		}
	}
	
	/**
	 * Determine whether the toolbar item should be visible
	 *
	 * @return bool
	 */
	public function isVisible() {
		global $wgUser, $wgTitle, $wgRequest;
		
		foreach ( $this->rules as $rule ) {
			switch ( $rule[0] ) {
				case TB_RULE_LOGGEDIN:
					if ( $wgUser->isLoggedIn() != $rule[1] ) {
						return false;
					}
					break;
				case TB_RULE_NAMESPACE:
				case TB_RULE_ACTION:
					$test = ( $rule[0] == TB_RULE_NAMESPACE )
						? $wgTitle->getNamespace()
						: $wgRequest->getText( 'action', 'view' );
					if ( $rule[1] != in_array( $test, $rule[2] ) ) {
						return false;
					}
					break;
				case TB_RULE_TITLE:
					if ( $rule[1] != in_array( $wgTitle, $rule[2] ) ) {
						return false;
					}
					break;
				case TB_RULE_CUSTOM:
					if ( !$rule[2] ) {
						return false;
					}
					break;
				case TB_RULE_USERCAN:
					foreach ( $rule[2] as $can ) {
						if ( 
							!$wgTitle->quickUserCan( $can )
							&& $wgTitle->getNamespace() != NS_SPECIAL
						) {
							return false;
						}
					}
					break;
			}
		}
		
		return true;
	}
}
