<?php

if( !defined( 'MEDIAWIKI' ) ) {
    die( 1 );
}

require_once( "SpecialPage.php" );

$wgSpecialPages['UserDirectory'] = 'SpecialUserDirectory';
$wgHooks['BeforePageDisplay'][] = 'UserDirectoryView::onBeforePageDisplay';

class SpecialUserDirectory extends SpecialPage {
	public function __construct() {
		SpecialPage::SpecialPage( 'UserDirectory', '', true );
	}

	public function execute() {
		global $wgOut, $wgRequest, $wgContLang;
		
		wfLoadExtensionMessages( 'CategoryPageView' );
		
		$this->setHeaders();
		$wgOut->setPagetitle(
			( $wgContLang->getCode() == 'pl' )
				? 'Katalog użytkowników'
				: 'User Directory'		
		);
		
		$view = new UserDirectoryView( new Article( SpecialPageIndex::getTitle() ) );
		
		$wgOut->addHTML( $view );	
	}
	
	public function getDescription() {
	    global $wgContLang;
	    
		return ($wgContLang->getCode() == 'pl') ? 'Katalog użytkowników' : 'User Directory';
	}
}

class UserDirectoryView {
    static $initialized = false;
    
	private $letters = array(
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
		'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'
	);
	
	private $article;
	private $title;
	public $page;
	public $offset;
	public $letter;
	public $order;

	private $pages;
	
	public function __construct() {
	    self::$initialized = true;
	    
		global $wgRequest, $wgCategoryPagingLimit;
		
		$this->article =& $article;
		$page = $wgRequest->getInt( 'page' );
		$this->page = ( $page == 0 ) ? 1 : $page;
		$this->offset = ($this->page - 1) * $wgCategoryPagingLimit;
		$this->letter = $wgRequest->getText( 'letter', '' );
		$order = $wgRequest->getText( 'order', '' );
		$this->order = ( is_null( $order ) || !in_array( $order, array( 'n', 'nd', 'e', 'ed', 'b', 'bd', 't', 'td' ) ) ) ? 'n' : $order;
		
		$this->letters = array_combine( $this->letters, array_fill( 0, count( $this->letters ), 0 ) );
		
		$this->pages = $this->getPagesData();
	}
	
	public static function onBeforePageDisplay( &$out ) {
		if ( self::$initialized ) {
    	    global $wgStylePath, $wgDefaultSkin;
    		
    		$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/PageIndex.css" );
    		$out->addScriptFile( "/extensions/PageIndex/PageIndex.js" );
		}

		return true;
	}
		
	public function viewCacheKey( $type ) {
		global $wgUser, $wgTitle, $wgMemc, $wgDBname, $wgCategoryPagingLimit;
		
		$userId = $wgUser->getId();
		$letter = !empty( $this->letter ) ? $this->letter : 'any';
		$order = $this->order;
		$page = $this->page;
		
		$tag = Tags::$tag;
		
		switch ( $type ) {
			case PAGEINDEX_VIEW_CACHE:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:{$userId}:users:{$letter}:{$order}:{$page}";
			case PAGEINDEX_VIEW_CACHE_ALL:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:all:users";
			case PAGEINDEX_VIEW_CACHE_USER:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:user:{$userId}:users";
			case PAGEINDEX_VIEW_CACHE_EVERYTHING:
				return "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		}
	}
	
	private function viewCacheGroupGet( $type ) {
		global $wgMemc;
		
		$group = $wgMemc->get( $this->viewCacheKey( $type ) );
		
		return empty( $group ) ? array() : $group;
	}
	
	private function viewCacheGroupAdd( $type, $add ) {
		global $wgMemc;
		
		$group = $this->viewCacheGroupGet( $type );
		array_push( $group, $add );
		
		$wgMemc->set( $this->viewCacheKey( $type ), $group );
	}
		
	public function viewCacheGet() {
		global $wgMemc;
		
		$key = $this->viewCacheKey( PAGEINDEX_VIEW_CACHE );
		
		$view = $wgMemc->get( $key );
		
		return empty( $view ) ? false : $view;
	}
	
	public function viewCacheSet( $view ) {
		global $wgMemc;
		
		$key = $this->viewCacheKey( PAGEINDEX_VIEW_CACHE );
		
		$wgMemc->set( $key, $view );
		
		$this->viewCacheGroupAdd( PAGEINDEX_VIEW_CACHE_ALL, $key );
		$this->viewCacheGroupAdd( PAGEINDEX_VIEW_CACHE_USER, $key );
		$this->viewCacheGroupAdd( PAGEINDEX_VIEW_CACHE_EVERYTHING, $key );
	}
	
	private function getLettersLinks() {
		global $wgUser, $wgTitle, $wgRequest;
		
		$skin = $wgUser->getSkin();
		
		$userArray = $this->getPagesData( true );
		
		$params = array();
				
		foreach ( $userArray as $user ) {
			$basename = $user->getName();
			$letter = strtoupper( $basename[0] );
			if ( array_key_exists( $letter, $this->letters ) ) {
				++ $this->letters[$letter];
			}
		}

		$html =
			'<table id="mw-letters">' . "\n" .
			'<tr>' .
			'<td>' . wfMsg( 'table_pager_limit_submit' ) . ': </td>' .
			'<td class="link">' . $skin->link( $wgTitle, wfMsg( 'rc_categories_any' ), array(), $params ) . '</td>';
		
		foreach ( $this->letters as $letter => $count ) {
			$html .= '<td class="link letter' . ( ( $count > 0 ) ? ' notempty' : '' ) . '">';
			if ( $count > 0 ) {
				$link = $skin->link( $wgTitle, $letter,
					array( 'title' => wfMsg( 'pageindex_pagecount', $count ) ),
					array_merge( $params, array( 'letter' => $letter ) )
				);
				$html .= ( $letter == $this->letter ) ? "[{$link}]" : $link;
			} else {
				$html .= $letter;
			}
			$html .= '</td>';
		}
		
		$html .=
			'</tr>' . "\n" .
			'</table>' . "\n";
		
		return $html;
	}
	
	public function getPagesData( $cheap = false ) {
		global $wgTitle, $wgCategoryPagingLimit;
		
		$dbr =& wfGetDB( DB_SLAVE );
		
		if ( !$cheap && !empty( $this->letter ) ) {
			$condition = "user_name LIKE {$dbr->addQuotes( $this->letter . '%' )}";
		} else {
			$condition = '1 = 1';
		}
		
		switch ( $this->order ) {
			case 'n':
				$order = 'user_name'; break;
			case 'nd':
				$order = 'user_name DESC'; break;
			case 'e':
				$order = 'user_email'; break;
			case 'ed':
				$order = 'user_email DESC'; break;				
			case 'b':
				$order = 'user_blocked'; break;
			case 'bd':
				$order = 'user_blocked DESC'; break;				
			case 't':
				$order = 'user_touched'; break;
			case 'td':
				$order = 'user_touched DESC'; break;				
			default:
				$order = 'user_name';	
		}
		
		if ( !$cheap) {
			$count = $dbr->selectField(
				'user',
				'COUNT(user_id)',
				array( $condition ),
				__METHOD__
			);
		}
		
		$res = $dbr->select(
			'user',
			array( 'user_id', 'user_name', 'user_email', 'user_touched' ),
			array( $condition ),
			__METHOD__,
			array_merge(
				array( 'ORDER BY' => $order ),
				!$cheap ? array( 'LIMIT' => $wgCategoryPagingLimit, 'OFFSET' => $this->offset ) : array()
			)
		);
		
		$userArray = array();
		while ( $row = $dbr->fetchObject( $res ) ) {
			array_push( $userArray, User::newFromRow( $row ) );
		}
		
    	return ( !$cheap ) ? array( 'count' => $count, 'data' => $userArray ) : $userArray;	
	}
	
	private function getPagesList() {
		global $wgTitle, $wgUser, $wgCategoryPagingLimit, $wgUploadPath;
		
		$skin = $wgUser->getSkin();
		
		$params = $_GET;
		unset( $params['title'] ); 
		
		if ( !empty( $this->letter ) ) $params['letter'] = $this->letter;
				
		$arrows = array_fill_keys( array( 'n', 'e', 'b' , 't' ), array() );
		switch ( $this->order ) {
			case 'n':
				$arrows['n'] = array( 'class' => 'sort-up' ); break;
			case 'nd':
				$arrows['n'] = array( 'class' => 'sort-down' ); break;
			case 'e':
				$arrows['e'] = array( 'class' => 'sort-up' ); break;
			case 'ed':
				$arrows['e'] = array( 'class' => 'sort-down' ); break;
			case 'b':
				$arrows['b'] = array( 'class' => 'sort-up' ); break;
			case 'bd':
				$arrows['b'] = array( 'class' => 'sort-down' ); break;
			case 't':
				$arrows['t'] = array( 'class' => 'sort-up' ); break;
			case 'td':
				$arrows['t'] = array( 'class' => 'sort-down' ); break;
		}
		
		$html =
			'<table class="pages-list" cellspacing="0">' . "\n" .
			'<thead>' .
			'<tr>' .
			'<td>' .
			$skin->link( $wgTitle, wfMsg( 'pageindex_name' ), $arrows['n'],
				array_merge( $params, array( 'order' => ( $this->order == 'n' ) ? 'nd' : 'n' ) )		
			) .
			'</td><td>' .
			$skin->link( $wgTitle, wfMsg( 'pageindex_email' ), $arrows['e'],
				array_merge( $params, array( 'order' => ( $this->order == 'e' ) ? 'ed' : 'e' ) )
			) .
			'</td>';
		if (in_array('sysop', $wgUser->getGroups())) {
		    $html .=
		        '<td>' . wfMsg( 'pageindex_groups' ) . '</td>' .
		    	'<td>' . $skin->link( $wgTitle, wfMsg( 'pageindex_block' ), $arrows['b'],
				    array_merge( $params, array( 'order' => ( $this->order == 'b' ) ? 'bd' : 'b' ) )
			    ) . '</td>' .
		        '<td>' . $skin->link( $wgTitle, wfMsg( 'pageindex_lastLogin' ), $arrows['t'],
				    array_merge( $params, array( 'order' => ( $this->order == 't' ) ? 'td' : 't' ) )
			    ) . '</td>';
		}	
	    $html .=
			'</tr>' .
			'</thead>' . "\n" .
			'<tbody>';
		
		$gray = true;
		
		$spTitle = SpecialPage::getPage('BlockUser')->getTitle();
		$gTitle = SpecialPage::getPage('ManageGroups')->getTitle();
		
		foreach ( $this->pages['data'] as $user ) {
		    $blocked = BusinessWiki_User::newFromId($user->getId())->isBlocked();
		    $lastLogin = BusinessWiki_User::newFromId($user->getId())->getLastLoginTime();
		    
		    $avatar = new wAvatar( $user->getId(), 's' );
		    
			$html .=
				Xml::openElement( 'tr', $gray ? array( 'class' => 'gray' ) : null ) . "\n" .
				Xml::tags( 'td', array( 'class' => 'name', 'style' => "background-image: url({$wgUploadPath}/avatars/{$avatar->getAvatarImage()});" ), $skin->link( $user->getUserPage(), $user->getName() ) ) . "\n";
			$email = $user->getEmail();
			if ( !empty( $email ) ) {
			    $email = Xml::openElement( 'a', array( 'href' => "mailto:{$email}") ) . $email . Xml::closeElement( 'a' ); 
			}
			$html .= Xml::tags( 'td', array( 'class' => 'email' ), $email ) . "\n";
			
			if (in_array('sysop', $wgUser->getGroups())) {
			    $html .= 
			    	'<td class="opt">';
			    $groups = Groups::getGroups($user->getId());
			    if (!empty($groups)) {
			        $html .= '<a href="' . $gTitle->getLocalURL() . '&user=' . $user->getId() . '" class="groups"></a>';
			    }
			    $html .= '</td>' .
			    	'<td class="opt">';
			    if (!in_array('sysop', $user->getGroups())) {
			        $html .= '<a href="' . $spTitle->getLocalURL() . '/' . $user->getId() . '" class="' . ($blocked ? 'blocked' : 'unblocked') . '"></a>';
			    }
			    $html .=
			        '</td>' .
			        '<td class="opt" style="width: auto">' . $lastLogin . '</td>';
			}
			
			$html .= '</tr>' . "\n";
			
			$gray = $gray ? false : true;
		}
		
		$html .=
			'</tbody>' . "\n" .
			'</table>' . "\n";
		
		$count = $this->pages['count'];
		$page = $this->page;
		
		if ( ( $count / $wgCategoryPagingLimit ) > 1 ) {
			if ( !empty( $this->order ) ) $params['order'] = $this->order;
			
			$links = array();
			
			$first = 1;
			$last = (int) ceil( $count / $wgCategoryPagingLimit );
			
			$links[] = ( 1 == $page )
				? '<strong>1</strong>'
				: $skin->link( $wgTitle, 1, array(), array_merge( $params, array( 'page' => 1 ) ) );
				
			if ( ( $page - 5 ) > ( $first + 1 ) ) {
				$links[] = '...'; // TODO: &hellip;
			}
			
			$start = max( ( $first + 1 ), ( $page - 5 ) );
			$end = min( ( $last - 1 ), ( $page + 5 ) );
			
			for ( $i = $start; $i <= $end; $i ++ ) {
				$links[] = ( $i == $page )
					? '<strong>' . $i . '</strong>'
					: $skin->link( $wgTitle, $i, array(), array_merge( $params, array( 'page' => $i ) ) );
			}
			
			if ( ( $page + 5 ) < ( $last - 1 ) ) {
				$links[] = '...'; // TODO: &hellip;
			}
			
			$links[] = ( $last == $page )
				? '<strong>' . $last . '</strong>'
				: $skin->link( $wgTitle, $last, array(), array_merge( $params, array( 'page' => $last ) ) );
						
			$html .= '<span class="nav">' . implode( " ", $links ) . '</span>' . "\n"; 
		}
		
		return $html;
	}
	
	public function __toString() {
		$html = $this->viewCacheGet();
		
		if ( false !== $html ) return $html;
		
		$count = count( $this->getPagesData( true ) );
		
		$html = '<div id="mw-pages">';
		
		if ( $count > 0 ) {
			$html .= $this->getLettersLinks();
		}
		
		$html .= ( count( $this->pages['data'] ) > 0 )
			? $this->getPagesList()
			: Xml::span( ( $count > 0 ) ? wfMsg( 'pageindex_noresult' ) : wfMsg( 'pageindex_nopages' ), 'msg' );
			   
		$html .= '</div>' . "\n";
		
		$this->viewCacheSet( $html );

		return $html;
	}
}
