<?php

/**
 * MediaWiki jQueryCategoryTree extension
 */

if( !defined( 'MEDIAWIKI' ) ) {
    echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
    die( 1 );
}

$wgExtensionCredits['parserhook'][] = array(
	'name' => 'jQueryCategoryTree',
	'author' => 'Paweł Szreder',
	'version'        => '0.1',
	'description' => 'Adds jQuery category tree',
	'descriptionmsg' => 'jquerycategorytree_desc',
	'url' => 'http://www.bizneswiki.pl'
);

$wgExtensionFunctions[] = 'wfJQueryCategoryTree';
array_unshift($wgHooks['ArticleInsertComplete'], 'wfJQueryCategoryTreeArticleInsertOrSaveComplete');
array_unshift($wgHooks['ArticleSaveComplete'], 'wfJQueryCategoryTreeArticleInsertOrSaveComplete');
array_unshift($wgHooks['ArticleDeleteComplete'], 'wfJQueryCategoryTreeArticleDeleteComplete');
//$wgHooks['ArticleInsertComplete'][] = 'wfJQueryCategoryTreeArticleInsertOrSaveComplete';
//$wgHooks['ArticleSaveComplete'][] = 'wfJQueryCategoryTreeArticleInsertOrSaveComplete';
//$wgHooks['ArticleDeleteComplete'][] = 'wfJQueryCategoryTreeArticleDeleteComplete';
$wgHooks['LanguageGetMagic'][]       = 'wfJQueryCategoryTreeMagic';
$wgHooks['BeforePageDisplay'][] = 'jQueryCategoryTree::onBeforePageDisplay';

$wgHooks['MakeGlobalVariablesScript'][] = 'jQueryCategoryTree::onMakeGlobalVariablesScript';
 
$dir = dirname(__FILE__) . '/';
require_once $dir . 'jQueryCategoryTree.i18n.php';

function wfJQueryCategoryTree() {
	global $wgParser, $wgCategoryTreeMessages, $wgMessageCache;
	
	$wgParser->setFunctionHook( 'jquerycategorytree', 'wfJQueryCategoryTreeRender' );

	foreach ( $wgCategoryTreeMessages as $lang => $messages) {
		$wgMessageCache->addMessages( $messages, $lang );
	}
}

function wfJQueryCategoryTreeArticleInsertOrSaveComplete( Article &$article ) {
	global $wgRequest, $wgDBname, $wgMemc;
	
	/*
	$key = "{$wgDBname}:jcqt:{$pageId}:" . (string) $returnSelf . ':' . (string) $showSelf;
	
	$wgMemc->delete( $key );
	
	self::getData( $user );
	*/
	
	$parentTitle = Title::newFromID( $wgRequest->getVal( 'wpParentCategoryId' ), 0 );
	if ( $parentTitle instanceof Title  ) {
		$title = $article->getTitle();
		if ( !$parentTitle->equals( $title ) ) {
			jQueryCategoryTree::move( $title, $parentTitle );
		}
	}
	
	return true;
}
 
function wfJQueryCategoryTreeArticleDeleteComplete( Article &$article ) {
	return jQueryCategoryTree::clear( $article->getTitle() );
}
 
function wfJQueryCategoryTreeMagic( &$magicWords, $langCode ) {
	$magicWords['jquerycategorytree'] = array( 0, 'jquerycategorytree' );

	return true;
}
 
function wfJQueryCategoryTreeRender( &$parser, $id = 'jqct' ) {
	$jQCT = new jQueryCategoryTree($id);
	
	return array( $jQCT->getHTML(), 'isHTML' => true );
}

class jQueryCategoryTree {
    private $id;
    
	public function __construct($id = 'jqct') {
	    $this->id = $id;
	}
	
	public static function newTree($id) {
		$jQCT = new jQueryCategoryTree($id);
		return $jQCT;
	}
	
	public static function onBeforePageDisplay( OutputPage &$out ) {
		global $wgScriptPath, $wgStylePath, $wgDefaultSkin;
		
		$style = ".tree li a {	background-image:url('{$wgStylePath}/{$wgDefaultSkin}/jQueryCategoryTree/f.gif'); }\n";
		
		$out->addExtensionStyle( "{$wgStylePath}/{$wgDefaultSkin}/jQueryCategoryTree.css" );
		$out->addScript( Xml::element( 'style', array( 'type' => 'text/css'), $style ) . "\n" );

		$dir = "{$wgScriptPath}/extensions/" . __CLASS__ . '/';
			
		$out->addScriptFile( $dir . __CLASS__ . '.js' );		
		
		return true;
	}
	
	public static function onMakeGlobalVariablesScript( &$vars ) {
		global $wgTitle;
		
		$parentId = self::getParentCategoryId( $wgTitle, false );
		
		$vars['wgArticleParentId'] = $parentId; 
		$vars['wgTreeArticleId'] = $wgTitle->getArticleID();
		$vars['wgTreeArticleParentId'] = $parentId;
		$vars['wgTreeNamespaceNumber'] = $wgTitle->getNamespace();
		$vars['wgTreeShowSelf'] = true;
		$vars['wgTreeManageOnly'] = false;
		
		return true;
	}
	
	public static function getParentCategoryId( Title &$title, $returnSelf = true ) {
		global $wgRequest/*, $wgMemc, $wgDBname*/;
		
		$pageId = $title->getArticleID();
		
		$categoryId = $wgRequest->getVal( 'category' );
		
		/*
		$key = "{$wgDBname}:jcqtparent:{$pageId}:" . (string) $categoryId . ':' . (string) $returnSelf;
		
		$data = $wgMemc->get( $key );
		
		if ( !empty( $data ) ) return $data;
		*/
		
	    $dbr =& wfGetDB( DB_SLAVE );
	    
		$ns = $title->getNamespace();
		
		switch (true) {
	    	case ( $ns == NS_MAIN ) && !$categoryId:
	    	case ( $ns == NS_CATEGORY ) && !$returnSelf && !$categoryId:
		    	$cl_to = $dbr->selectField(
		    		'categorylinks',
		    		'cl_to',
		    		array( 'cl_from' => $title->getArticleID(), 'cl_to_namespace' => NS_CATEGORY ),
		    		__METHOD__
		    	);
			    if ( false !== $cl_to ) {
			    	$parentTitle = Title::newFromText( $cl_to, NS_CATEGORY );
			    	$result = ( $parentTitle instanceof Title ) ? $parentTitle->getArticleID() : 0;
			    	// $wgMemc->set( $key, $result );
			    	return $result;
			    } else {
			    	// $wgMemc->set( $key, 0 );
			    	return 0;
			    }
	    		break;
	    	case ( $ns == NS_MAIN )	&& $categoryId:
	    	case ( $ns == NS_CATEGORY )	&& !$returnSelf && $categoryId:
	    		// $wgMemc->set( $key, $categoryId );
	    		return $categoryId;
	    		break;		    	
	    	case ( $ns == NS_CATEGORY ):
	    		// $wgMemc->set( $key, $title->getArticleID() );
	    		return $title->getArticleID();
	    		break;
	    }
		
		return 0;
	}
	
	/**
	 * @param Database $db
	 * @param string $pageTable
	 * @param string $categorylinksTable
	 * @param string $catName
	 * @return array
	 */
	private function getChildren( &$dbr, Title $title, $pageId, $showSelf ) {
    	$res = $dbr->select(
    		array( 'categorylinks', 'page' ),
    		'cl_from',
    		array( 'page_namespace' => NS_CATEGORY, 'cl_to_namespace' => NS_CATEGORY, 'cl_to' => $title->getDBkey() ),
    		__METHOD__,
    		array( 'ORDER BY' => 'page_title' ),
    		array( 'page' => array( 'LEFT JOIN', 'page_id = cl_from' ) )    	
    	);
    	
		if ( $dbr->numRows( $res ) == 0 ) return null;

		$children = array();
		foreach ( $res as $row ) {
			if ( !$showSelf && ( $pageId == $row->cl_from ) ) {
				continue;
			}
			
	    	$childTitle = Title::newFromID( $row->cl_from );
	    	$children[] = array(
	    		'attributes' => array(
	    			'id' => $this->id . "-node_{$row->cl_from}",
	    			'rel' => 'folder',
	    			'href' => $childTitle->getFullURL()
	    		),
	    		'data' => $childTitle->getBaseText(),
	    		'children' => $this->getChildren( $dbr, $childTitle, $pageId, $showSelf )
	    	);
		}
		
		return $children;
	}	

	public function getData( $title, $returnSelf = true, $showSelf = true ) {
		global $wgMemc, $wgDBname;
		
		$pageId = ( $title instanceof Title ) ? $title->getArticleID() : 0;
		
		/*
		$key = "{$wgDBname}:jcqt:{$pageId}:" . (string) $returnSelf . ':' . (string) $showSelf;
		
		$data = $wgMemc->get( $key );
		
		if ( !empty( $data ) ) return $data;
		*/
		
		$arrayOfCategories = array();
	
	    $dbr =& wfGetDB( DB_SLAVE );
		
		$res = $dbr->select(
			array( 'page', 'categorylinks' ),
			'page_id',
			array( 'page_namespace' => NS_CATEGORY, 'cl_from' => null ),
			__METHOD__,
			array( 'ORDER BY' => 'page_title' ),
			array( 'categorylinks' => array( 'LEFT JOIN', 'page_id = cl_from' ) )
		);
		
		foreach ( $res as $row ) {
			if ( !$showSelf && ( $pageId == $row->page_id ) ) {
				continue;
			}

			$childTitle = Title::newFromID( $row->page_id );
	        array_push( $arrayOfCategories, array( 
	        	'attributes' => array(
	        		'id' => $this->id . "-node_{$row->page_id}",
	        		'rel' => 'folder',
	        		'href' => $childTitle->getFullURL()
	        	),
	        	'data' => $childTitle->getBaseText(),
	        	'children' => $this->getChildren( $dbr, $childTitle, $pageId, $showSelf )
	        ) );
	    }
	    
	    $parentCategoryId = ( $title instanceof Title )
	    	? $this->getParentCategoryId( $title, $returnSelf )
	    	: 0;
	    
	    $return = array(
	    	'attributes' => array(
	    		'id' => $this->id . '-node_0',
	    		'rel' => 'categories',
	    		'class' => ($parentCategoryId == 0) ? 'open' : 'closed',
	            'href' => SpecialPage::getPage('PageIndex')->getTitle()->getLocalURL()
	    	),
	    	'data' => wfMsg( 'catalogs' ),
	    	'children' => $arrayOfCategories
	    );
	    
	    /*
	    $wgMemc->add( $key, $return );
		*/
	    
    	return $return;
	}
	
	public function getDataJSON( $title, $returnSelf = true, $showSelf = true ) {
		if ( !function_exists( 'json_encode' ) ) {
			function json_encode( $data ) {
				$json = new Services_JSON();
				return( $json->encode( $data ) );
			}
		}
	
		if( !function_exists( 'json_decode' ) ) {
			function json_decode( $data ) {
				$json = new Services_JSON();
				return( $json->decode( $data ) );
			}
		}		
		
		return json_encode( $this->getData( $title, $returnSelf, $showSelf ) );
	}
	
	private function nodeToHTML( $data ) {
		$result =
			Xml::openElement( 'li', $data['attributes'] ) .
			'<a>' . htmlspecialchars( $data['data'] ) . '</a>';
		
		if ( !empty( $data['children'] ) ) {
			$result .= '<ul>';
			foreach ( $data['children'] as $child ) {
				$result .= $this->nodeToHTML( $child );	
			}
			$result .= '</ul>';
		}
		
		$result .= '</li>';		
		
		return $result;
	}
	
	private function dataToHTML( $data ) {
		$html = 
			'<div id="' . $this->id . '" class="bwi-folders tree">' . "\n" .
			'<ul>' .
			Xml::openElement( 'li', $data['attributes'] ) .
			'<a>' . htmlspecialchars( $data['data'] ) . '</a>';
		if ( !empty( $data['children'] ) ) {
            $html .=
    		    '<ul>' ;
    		foreach ( $data['children'] as $node ) {
    			$html .= $this->nodeToHTML( $node );
    		}
    		$html .= '</ul>';
		}
		$html .= '</ul>' . "\n" . '</div>' . "\n";
		
		return $html;
	}
	
	public function getDataHTML( $title, $returnSelf = true, $showSelf = true ) {
		$data = $this->getData( $title, $returnSelf, $showSelf );
		
		$html = $this->dataToHTML( $data );
		
		return $html;
	}
	
	public function getHTML( $returnSelf = true ) {
		global $wgTitle;
		
		return $this->getDataHTML( $wgTitle );		
	}

	public static function delete( $id ) {
		$article = Article::newFromID( $id );
		
		if ( $article instanceof Article ) {
			$title = $article->getTitle();
			$text = $title->getBaseText();
			$parentId = self::getParentCategoryId( $article->getTitle(), false );
			
			$article->doDeleteArticle( '' );
		} else {
			return 'false';
		}
		
		$dbw =& wfGetDB( DB_MASTER );
		
		$dbw->begin();
		$dbw->delete( 'categorylinks', array( 'cl_from' => $id ) );
		if ( $parentId > 0 ) {
			$newTitle = Title::newFromID( $parentId );
			$dbw->update(
				'categorylinks',
				array( 'cl_to' => $newTitle->getBaseText(), 'cl_sortkey' => $newTitle->getPrefixedText() ),
				array( 'cl_to' => $text, 'cl_to_namespace' => NS_CATEGORY ),
				__METHOD__
			);
		} else {
			$dbw->delete(
				'categorylinks',
				array( 'cl_to' => $text, 'cl_to_namespace' => NS_CATEGORY ),
				__METHOD__
			);
		}
		$dbw->commit();
			
		return 'true';
	}
	
	public static function rename( $id, $value ) {
		$dbw =& wfGetDB( DB_MASTER );
		
		$newTitle = Title::newFromText( $value, NS_CATEGORY );
				
		if ( strpos( $id, 'sub-' ) !== false ) {
			if ( !is_object( $newTitle ) || ( $newTitle->getBaseText() != ucfirst( $value ) ) ) {
				return 'false-delete-invalid';
			}

			list( $refId ) = sscanf( $id, 'sub-%d' );
			
			global $wgParentCategoryId;
			
			$wgParentCategoryId = $refId;	    
			
			$refTitle = Title::newFromID( $refId, NS_CATEGORY );
			
			if ( $newTitle->exists() ) {
				if ( !( $refTitle instanceof Title ) ) {
					return 'false-delete-invalid';
				}
				$newName = $newTitle->getBaseText() . ' (' . $refTitle->getBaseText() . ')';
				$newTitle = Title::newFromText( $newName, NS_CATEGORY );
				
				$renamed = true;
			}
						
			$article = new Article( $newTitle );
			$article->doEdit( ' ', ' ', EDIT_NEW | EDIT_SUPPRESS_RC );
			
			$id = $article->getID();
			
			if ( $refTitle instanceof Title ) {	
				$dbw->begin();
				$dbw->insert( 'categorylinks', array(
					'cl_from' => $id,
					'cl_to' => $refTitle->getDBkey(),
					'cl_to_namespace' => $refTitle->getNamespace(),
					'cl_sortkey' => $refTitle->getPrefixedText()
				), __METHOD__ );
				$dbw->commit();
			}
		} else {
			$title = Title::newFromID( $id );
			
			if ( !is_object( $title ) /* || ( $title->getBaseText() != ucfirst( $value ) ) */ ) {
				return 'false-invalid';
			}
			
			if ( $newTitle->exists() ) {
				if ( $newTitle->equals( $title ) ) {
					return 'false-nothing';
				} else {
					$newName = $newTitle->getBaseText();
					$parentId = self::getParentCategoryId( $newTitle, false );
					$parentName = ( $parentId == 0 ) ? '0' : Title::newFromId( $parentId )->getBaseText();
					
					return "false|{$newName}|{$parentName}";
				}
			}
			
			$id = $title->getArticleID();
						
			$result = $title->moveTo( $newTitle, true, '', true );
			if ( $result !== true ) {
				return 'false-nothing';
			}
			
			$text = $title->getBaseText();
			
			$article = new Article( $title );
			$article->doDeleteArticle( '', true );
			
			$dbw->begin();
			$dbw->update(
				'categorylinks',
				array( 'cl_to' => $newTitle->getDBkey(), 'cl_sortkey' => $newTitle->getPrefixedText() ),
				array( 'cl_to' => $text ),
				__METHOD__
			);
			$dbw->commit();
			
			$id = $newTitle->getArticleID();
			
			
			/*
			$dbw->begin();
			$dbw->update(
				'page',
				array( 'page_title' => ucfirst($value) ),
				array( 'page_id' => $id ),
				__METHOD__
			);
			$dbw->update(
				'categorylinks',
				array( 'cl_to' => $newTitle->getBaseText(), 'cl_sortkey' => $newTitle->getPrefixedText() ),
				array( 'cl_to' => $title->getBaseText() ),
				__METHOD__
			);
			$dbw->commit();
			*/
		}

		settype( $id, 'string' );
		
		if ( $renamed ) {
			return 'false-renamed|' . $id . '|' . $newName;	
		} else {		
			return $id;
		}
	}
	
	public static function clear( $title ) {
		if ( !($title instanceof Title) ) return;
		
		$dbw =& wfGetDB( DB_MASTER );
		
		$dbw->begin();
		$dbw->delete( 'categorylinks', array( 'cl_from' => $title->getArticleID() ) );
		$dbw->commit();

		global $wgMemc, $wgDBname, $wgCategoryPagingLimit;
		
		$key = "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		
		$group = $wgMemc->get( $key );
		
		if ( !empty( $group ) ) {
			foreach ( $group as $item ) {
				$wgMemc->delete( $item );
			}
		}
		
		$wgMemc->delete( $key );		
		
		return true;
	}
	
	public static function move( $title, $refTitle ) {
		if ( !( $title instanceof Title ) ) return 'false';
		if ( ( $refTitle instanceof Title ) && $refTitle->getNamespace() != NS_CATEGORY ) return 'false';
		
		$dbw =& wfGetDB( DB_MASTER );
		
		if ( !( $refTitle instanceof Title ) ) {
			$dbw->begin();
			self::clear( $title );
			$dbw->commit();
			
			return 'true';
		}
		
		$dbw->begin();
		self::clear( $title );
		$dbw->insert( 'categorylinks', array(
			'cl_from' => $title->getArticleID(),
			'cl_to' => $refTitle->getDBkey(),
			'cl_to_namespace' => $refTitle->getNamespace(),
			'cl_sortkey' => $refTitle->getPrefixedText()
		), __METHOD__ );
		$dbw->commit();
		
		global $wgMemc, $wgDBname, $wgCategoryPagingLimit;
		
		$key = "{$wgDBname}:pageindexview:{$wgCategoryPagingLimit}:everything";
		
		$group = $wgMemc->get( $key );
		
		if ( !empty( $group ) ) {
			foreach ( $group as $item ) {
				$wgMemc->delete( $item );
			}
		}
		
		$wgMemc->delete( $key );

		return 'true';
	}
}
